/****************************************************************************
** Meta object code from reading C++ file 'qquickdroparea_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquickdroparea_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickdroparea_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickDropEvent_t {
    QByteArrayData data[29];
    char stringdata0[293];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickDropEvent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickDropEvent_t qt_meta_stringdata_QQuickDropEvent = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QQuickDropEvent"
QT_MOC_LITERAL(1, 16, 11), // "QML.Element"
QT_MOC_LITERAL(2, 28, 9), // "anonymous"
QT_MOC_LITERAL(3, 38, 15), // "getDataAsString"
QT_MOC_LITERAL(4, 54, 0), // ""
QT_MOC_LITERAL(5, 55, 15), // "QQmlV4Function*"
QT_MOC_LITERAL(6, 71, 20), // "getDataAsArrayBuffer"
QT_MOC_LITERAL(7, 92, 20), // "acceptProposedAction"
QT_MOC_LITERAL(8, 113, 6), // "accept"
QT_MOC_LITERAL(9, 120, 1), // "x"
QT_MOC_LITERAL(10, 122, 1), // "y"
QT_MOC_LITERAL(11, 124, 6), // "source"
QT_MOC_LITERAL(12, 131, 4), // "keys"
QT_MOC_LITERAL(13, 136, 16), // "supportedActions"
QT_MOC_LITERAL(14, 153, 15), // "Qt::DropActions"
QT_MOC_LITERAL(15, 169, 14), // "proposedAction"
QT_MOC_LITERAL(16, 184, 6), // "action"
QT_MOC_LITERAL(17, 191, 14), // "Qt::DropAction"
QT_MOC_LITERAL(18, 206, 8), // "accepted"
QT_MOC_LITERAL(19, 215, 8), // "hasColor"
QT_MOC_LITERAL(20, 224, 7), // "hasHtml"
QT_MOC_LITERAL(21, 232, 7), // "hasText"
QT_MOC_LITERAL(22, 240, 7), // "hasUrls"
QT_MOC_LITERAL(23, 248, 9), // "colorData"
QT_MOC_LITERAL(24, 258, 4), // "html"
QT_MOC_LITERAL(25, 263, 4), // "text"
QT_MOC_LITERAL(26, 268, 4), // "urls"
QT_MOC_LITERAL(27, 273, 11), // "QList<QUrl>"
QT_MOC_LITERAL(28, 285, 7) // "formats"

    },
    "QQuickDropEvent\0QML.Element\0anonymous\0"
    "getDataAsString\0\0QQmlV4Function*\0"
    "getDataAsArrayBuffer\0acceptProposedAction\0"
    "accept\0x\0y\0source\0keys\0supportedActions\0"
    "Qt::DropActions\0proposedAction\0action\0"
    "Qt::DropAction\0accepted\0hasColor\0"
    "hasHtml\0hasText\0hasUrls\0colorData\0"
    "html\0text\0urls\0QList<QUrl>\0formats"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickDropEvent[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       4,   16, // methods
      17,   48, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // methods: name, argc, parameters, tag, flags
       3,    1,   36,    4, 0x02 /* Public */,
       6,    1,   39,    4, 0x02 /* Public */,
       7,    1,   42,    4, 0x02 /* Public */,
       8,    1,   45,    4, 0x02 /* Public */,

 // methods: parameters
    QMetaType::Void, 0x80000000 | 5,    4,
    QMetaType::Void, 0x80000000 | 5,    4,
    QMetaType::Void, 0x80000000 | 5,    4,
    QMetaType::Void, 0x80000000 | 5,    4,

 // properties: name, type, flags
       9, QMetaType::QReal, 0x00095001,
      10, QMetaType::QReal, 0x00095001,
      11, QMetaType::QObjectStar, 0x00095001,
      12, QMetaType::QStringList, 0x00095001,
      13, 0x80000000 | 14, 0x00095009,
      15, 0x80000000 | 14, 0x00095009,
      16, 0x80000000 | 17, 0x0009510f,
      18, QMetaType::Bool, 0x00095103,
      19, QMetaType::Bool, 0x00095001,
      20, QMetaType::Bool, 0x00095001,
      21, QMetaType::Bool, 0x00095001,
      22, QMetaType::Bool, 0x00095001,
      23, QMetaType::QVariant, 0x00095001,
      24, QMetaType::QString, 0x00095001,
      25, QMetaType::QString, 0x00095001,
      26, 0x80000000 | 27, 0x00095009,
      28, QMetaType::QStringList, 0x00095001,

       0        // eod
};

void QQuickDropEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickDropEvent *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->getDataAsString((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 1: _t->getDataAsArrayBuffer((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 2: _t->acceptProposedAction((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 3: _t->accept((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 15:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QUrl> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickDropEvent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->x(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->y(); break;
        case 2: *reinterpret_cast< QObject**>(_v) = _t->source(); break;
        case 3: *reinterpret_cast< QStringList*>(_v) = _t->keys(); break;
        case 4: *reinterpret_cast< Qt::DropActions*>(_v) = _t->supportedActions(); break;
        case 5: *reinterpret_cast< Qt::DropActions*>(_v) = _t->proposedAction(); break;
        case 6: *reinterpret_cast< Qt::DropAction*>(_v) = _t->action(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->accepted(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->hasColor(); break;
        case 9: *reinterpret_cast< bool*>(_v) = _t->hasHtml(); break;
        case 10: *reinterpret_cast< bool*>(_v) = _t->hasText(); break;
        case 11: *reinterpret_cast< bool*>(_v) = _t->hasUrls(); break;
        case 12: *reinterpret_cast< QVariant*>(_v) = _t->colorData(); break;
        case 13: *reinterpret_cast< QString*>(_v) = _t->html(); break;
        case 14: *reinterpret_cast< QString*>(_v) = _t->text(); break;
        case 15: *reinterpret_cast< QList<QUrl>*>(_v) = _t->urls(); break;
        case 16: *reinterpret_cast< QStringList*>(_v) = _t->formats(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickDropEvent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 6: _t->setAction(*reinterpret_cast< Qt::DropAction*>(_v)); break;
        case 7: _t->setAccepted(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickDropEvent *_t = static_cast<QQuickDropEvent *>(_o);
        (void)_t;
        switch (_id) {
        case 6: _t->resetAction(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickDropEvent::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickDropEvent.data,
    qt_meta_data_QQuickDropEvent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickDropEvent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickDropEvent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickDropEvent.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickDropEvent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 17;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 17;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QQuickDropAreaDrag_t {
    QByteArrayData data[9];
    char stringdata0[83];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickDropAreaDrag_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickDropAreaDrag_t qt_meta_stringdata_QQuickDropAreaDrag = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QQuickDropAreaDrag"
QT_MOC_LITERAL(1, 19, 11), // "QML.Element"
QT_MOC_LITERAL(2, 31, 9), // "anonymous"
QT_MOC_LITERAL(3, 41, 15), // "positionChanged"
QT_MOC_LITERAL(4, 57, 0), // ""
QT_MOC_LITERAL(5, 58, 13), // "sourceChanged"
QT_MOC_LITERAL(6, 72, 1), // "x"
QT_MOC_LITERAL(7, 74, 1), // "y"
QT_MOC_LITERAL(8, 76, 6) // "source"

    },
    "QQuickDropAreaDrag\0QML.Element\0anonymous\0"
    "positionChanged\0\0sourceChanged\0x\0y\0"
    "source"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickDropAreaDrag[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       2,   16, // methods
       3,   28, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   26,    4, 0x06 /* Public */,
       5,    0,   27,    4, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       6, QMetaType::QReal, 0x00495001,
       7, QMetaType::QReal, 0x00495001,
       8, QMetaType::QObjectStar, 0x00495001,

 // properties: notify_signal_id
       0,
       0,
       1,

       0        // eod
};

void QQuickDropAreaDrag::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickDropAreaDrag *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->positionChanged(); break;
        case 1: _t->sourceChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickDropAreaDrag::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDropAreaDrag::positionChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickDropAreaDrag::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDropAreaDrag::sourceChanged)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickDropAreaDrag *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->x(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->y(); break;
        case 2: *reinterpret_cast< QObject**>(_v) = _t->source(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickDropAreaDrag::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickDropAreaDrag.data,
    qt_meta_data_QQuickDropAreaDrag,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickDropAreaDrag::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickDropAreaDrag::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickDropAreaDrag.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickDropAreaDrag::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickDropAreaDrag::positionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickDropAreaDrag::sourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_QQuickDropArea_t {
    QByteArrayData data[17];
    char stringdata0[187];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickDropArea_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickDropArea_t qt_meta_stringdata_QQuickDropArea = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QQuickDropArea"
QT_MOC_LITERAL(1, 15, 11), // "QML.Element"
QT_MOC_LITERAL(2, 27, 8), // "DropArea"
QT_MOC_LITERAL(3, 36, 19), // "containsDragChanged"
QT_MOC_LITERAL(4, 56, 0), // ""
QT_MOC_LITERAL(5, 57, 11), // "keysChanged"
QT_MOC_LITERAL(6, 69, 13), // "sourceChanged"
QT_MOC_LITERAL(7, 83, 7), // "entered"
QT_MOC_LITERAL(8, 91, 16), // "QQuickDropEvent*"
QT_MOC_LITERAL(9, 108, 4), // "drag"
QT_MOC_LITERAL(10, 113, 6), // "exited"
QT_MOC_LITERAL(11, 120, 15), // "positionChanged"
QT_MOC_LITERAL(12, 136, 7), // "dropped"
QT_MOC_LITERAL(13, 144, 4), // "drop"
QT_MOC_LITERAL(14, 149, 12), // "containsDrag"
QT_MOC_LITERAL(15, 162, 4), // "keys"
QT_MOC_LITERAL(16, 167, 19) // "QQuickDropAreaDrag*"

    },
    "QQuickDropArea\0QML.Element\0DropArea\0"
    "containsDragChanged\0\0keysChanged\0"
    "sourceChanged\0entered\0QQuickDropEvent*\0"
    "drag\0exited\0positionChanged\0dropped\0"
    "drop\0containsDrag\0keys\0QQuickDropAreaDrag*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickDropArea[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       7,   16, // methods
       3,   64, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   51,    4, 0x06 /* Public */,
       5,    0,   52,    4, 0x06 /* Public */,
       6,    0,   53,    4, 0x06 /* Public */,
       7,    1,   54,    4, 0x06 /* Public */,
      10,    0,   57,    4, 0x06 /* Public */,
      11,    1,   58,    4, 0x06 /* Public */,
      12,    1,   61,    4, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, 0x80000000 | 8,   13,

 // properties: name, type, flags
      14, QMetaType::Bool, 0x00495001,
      15, QMetaType::QStringList, 0x00495103,
       9, 0x80000000 | 16, 0x00095409,

 // properties: notify_signal_id
       0,
       1,
       0,

       0        // eod
};

void QQuickDropArea::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickDropArea *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->containsDragChanged(); break;
        case 1: _t->keysChanged(); break;
        case 2: _t->sourceChanged(); break;
        case 3: _t->entered((*reinterpret_cast< QQuickDropEvent*(*)>(_a[1]))); break;
        case 4: _t->exited(); break;
        case 5: _t->positionChanged((*reinterpret_cast< QQuickDropEvent*(*)>(_a[1]))); break;
        case 6: _t->dropped((*reinterpret_cast< QQuickDropEvent*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDropEvent* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDropEvent* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDropEvent* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickDropArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDropArea::containsDragChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickDropArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDropArea::keysChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickDropArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDropArea::sourceChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickDropArea::*)(QQuickDropEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDropArea::entered)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickDropArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDropArea::exited)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickDropArea::*)(QQuickDropEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDropArea::positionChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickDropArea::*)(QQuickDropEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDropArea::dropped)) {
                *result = 6;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDropAreaDrag* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickDropArea *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->containsDrag(); break;
        case 1: *reinterpret_cast< QStringList*>(_v) = _t->keys(); break;
        case 2: *reinterpret_cast< QQuickDropAreaDrag**>(_v) = _t->drag(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickDropArea *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setKeys(*reinterpret_cast< QStringList*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickDropArea::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickDropArea.data,
    qt_meta_data_QQuickDropArea,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickDropArea::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickDropArea::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickDropArea.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QQuickDropArea::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickDropArea::containsDragChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickDropArea::keysChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickDropArea::sourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickDropArea::entered(QQuickDropEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickDropArea::exited()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickDropArea::positionChanged(QQuickDropEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QQuickDropArea::dropped(QQuickDropEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
