/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <private/quicktest_p.h>

void qml_register_types_Qt_test_qtestroot()
{
    qmlRegisterTypesAndRevisions<QTestRootObject>("Qt.test.qtestroot", 1);
    qmlRegisterModule("Qt.test.qtestroot", 1, 0);
}

static const QQmlModuleRegistration registration("Qt.test.qtestroot", 1, qml_register_types_Qt_test_qtestroot);
