/****************************************************************************
** Meta object code from reading C++ file 'qquickparticlepainter_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/particles/qquickparticlepainter_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickparticlepainter_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickParticlePainter_t {
    QByteArrayData data[20];
    char stringdata0[302];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickParticlePainter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickParticlePainter_t qt_meta_stringdata_QQuickParticlePainter = {
    {
QT_MOC_LITERAL(0, 0, 21), // "QQuickParticlePainter"
QT_MOC_LITERAL(1, 22, 11), // "QML.Element"
QT_MOC_LITERAL(2, 34, 15), // "ParticlePainter"
QT_MOC_LITERAL(3, 50, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 64, 5), // "false"
QT_MOC_LITERAL(5, 70, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 92, 55), // "Abstract type. Use one of the..."
QT_MOC_LITERAL(7, 148, 12), // "countChanged"
QT_MOC_LITERAL(8, 161, 0), // ""
QT_MOC_LITERAL(9, 162, 13), // "systemChanged"
QT_MOC_LITERAL(10, 176, 21), // "QQuickParticleSystem*"
QT_MOC_LITERAL(11, 198, 3), // "arg"
QT_MOC_LITERAL(12, 202, 13), // "groupsChanged"
QT_MOC_LITERAL(13, 216, 9), // "setSystem"
QT_MOC_LITERAL(14, 226, 9), // "setGroups"
QT_MOC_LITERAL(15, 236, 16), // "calcSystemOffset"
QT_MOC_LITERAL(16, 253, 12), // "resetPending"
QT_MOC_LITERAL(17, 266, 21), // "sceneGraphInvalidated"
QT_MOC_LITERAL(18, 288, 6), // "system"
QT_MOC_LITERAL(19, 295, 6) // "groups"

    },
    "QQuickParticlePainter\0QML.Element\0"
    "ParticlePainter\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "Abstract type. Use one of the inheriting types instead.\0"
    "countChanged\0\0systemChanged\0"
    "QQuickParticleSystem*\0arg\0groupsChanged\0"
    "setSystem\0setGroups\0calcSystemOffset\0"
    "resetPending\0sceneGraphInvalidated\0"
    "system\0groups"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickParticlePainter[] = {

 // content:
       8,       // revision
       0,       // classname
       3,   14, // classinfo
       8,   20, // methods
       2,   78, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,

 // signals: name, argc, parameters, tag, flags
       7,    0,   60,    8, 0x06 /* Public */,
       9,    1,   61,    8, 0x06 /* Public */,
      12,    1,   64,    8, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      13,    1,   67,    8, 0x0a /* Public */,
      14,    1,   70,    8, 0x0a /* Public */,
      15,    1,   73,    8, 0x0a /* Public */,
      15,    0,   76,    8, 0x2a /* Public | MethodCloned */,
      17,    0,   77,    8, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, QMetaType::QStringList,   11,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, QMetaType::QStringList,   11,
    QMetaType::Void, QMetaType::Bool,   16,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      18, 0x80000000 | 10, 0x0049510b,
      19, QMetaType::QStringList, 0x00495103,

 // properties: notify_signal_id
       1,
       2,

       0        // eod
};

void QQuickParticlePainter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickParticlePainter *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->countChanged(); break;
        case 1: _t->systemChanged((*reinterpret_cast< QQuickParticleSystem*(*)>(_a[1]))); break;
        case 2: _t->groupsChanged((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 3: _t->setSystem((*reinterpret_cast< QQuickParticleSystem*(*)>(_a[1]))); break;
        case 4: _t->setGroups((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 5: _t->calcSystemOffset((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->calcSystemOffset(); break;
        case 7: _t->sceneGraphInvalidated(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleSystem* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleSystem* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickParticlePainter::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticlePainter::countChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickParticlePainter::*)(QQuickParticleSystem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticlePainter::systemChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickParticlePainter::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticlePainter::groupsChanged)) {
                *result = 2;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleSystem* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickParticlePainter *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQuickParticleSystem**>(_v) = _t->system(); break;
        case 1: *reinterpret_cast< QStringList*>(_v) = _t->groups(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickParticlePainter *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSystem(*reinterpret_cast< QQuickParticleSystem**>(_v)); break;
        case 1: _t->setGroups(*reinterpret_cast< QStringList*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickParticlePainter::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickParticlePainter.data,
    qt_meta_data_QQuickParticlePainter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickParticlePainter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickParticlePainter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickParticlePainter.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QQuickParticlePainter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickParticlePainter::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickParticlePainter::systemChanged(QQuickParticleSystem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickParticlePainter::groupsChanged(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
