/****************************************************************************
** Meta object code from reading C++ file 'qquickpositioners_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquickpositioners_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickpositioners_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickPositionerAttached_t {
    QByteArrayData data[8];
    char stringdata0[105];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPositionerAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPositionerAttached_t qt_meta_stringdata_QQuickPositionerAttached = {
    {
QT_MOC_LITERAL(0, 0, 24), // "QQuickPositionerAttached"
QT_MOC_LITERAL(1, 25, 12), // "indexChanged"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 18), // "isFirstItemChanged"
QT_MOC_LITERAL(4, 58, 17), // "isLastItemChanged"
QT_MOC_LITERAL(5, 76, 5), // "index"
QT_MOC_LITERAL(6, 82, 11), // "isFirstItem"
QT_MOC_LITERAL(7, 94, 10) // "isLastItem"

    },
    "QQuickPositionerAttached\0indexChanged\0"
    "\0isFirstItemChanged\0isLastItemChanged\0"
    "index\0isFirstItem\0isLastItem"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPositionerAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       3,   32, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   29,    2, 0x06 /* Public */,
       3,    0,   30,    2, 0x06 /* Public */,
       4,    0,   31,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       5, QMetaType::Int, 0x00495001,
       6, QMetaType::Bool, 0x00495001,
       7, QMetaType::Bool, 0x00495001,

 // properties: notify_signal_id
       0,
       1,
       2,

       0        // eod
};

void QQuickPositionerAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickPositionerAttached *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->indexChanged(); break;
        case 1: _t->isFirstItemChanged(); break;
        case 2: _t->isLastItemChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickPositionerAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPositionerAttached::indexChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickPositionerAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPositionerAttached::isFirstItemChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickPositionerAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPositionerAttached::isLastItemChanged)) {
                *result = 2;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickPositionerAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->index(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->isFirstItem(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->isLastItem(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickPositionerAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickPositionerAttached.data,
    qt_meta_data_QQuickPositionerAttached,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPositionerAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPositionerAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPositionerAttached.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickPositionerAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickPositionerAttached::indexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickPositionerAttached::isFirstItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickPositionerAttached::isLastItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
struct qt_meta_stringdata_QQuickBasePositioner_t {
    QByteArrayData data[32];
    char stringdata0[500];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickBasePositioner_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickBasePositioner_t qt_meta_stringdata_QQuickBasePositioner = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QQuickBasePositioner"
QT_MOC_LITERAL(1, 21, 11), // "QML.Element"
QT_MOC_LITERAL(2, 33, 10), // "Positioner"
QT_MOC_LITERAL(3, 44, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 58, 5), // "false"
QT_MOC_LITERAL(5, 64, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 86, 78), // "Positioner is an abstract typ..."
QT_MOC_LITERAL(7, 165, 12), // "QML.Attached"
QT_MOC_LITERAL(8, 178, 24), // "QQuickPositionerAttached"
QT_MOC_LITERAL(9, 203, 14), // "spacingChanged"
QT_MOC_LITERAL(10, 218, 0), // ""
QT_MOC_LITERAL(11, 219, 15), // "populateChanged"
QT_MOC_LITERAL(12, 235, 11), // "moveChanged"
QT_MOC_LITERAL(13, 247, 10), // "addChanged"
QT_MOC_LITERAL(14, 258, 14), // "paddingChanged"
QT_MOC_LITERAL(15, 273, 17), // "topPaddingChanged"
QT_MOC_LITERAL(16, 291, 18), // "leftPaddingChanged"
QT_MOC_LITERAL(17, 310, 19), // "rightPaddingChanged"
QT_MOC_LITERAL(18, 330, 20), // "bottomPaddingChanged"
QT_MOC_LITERAL(19, 351, 19), // "positioningComplete"
QT_MOC_LITERAL(20, 371, 14), // "prePositioning"
QT_MOC_LITERAL(21, 386, 11), // "forceLayout"
QT_MOC_LITERAL(22, 398, 7), // "spacing"
QT_MOC_LITERAL(23, 406, 8), // "populate"
QT_MOC_LITERAL(24, 415, 17), // "QQuickTransition*"
QT_MOC_LITERAL(25, 433, 4), // "move"
QT_MOC_LITERAL(26, 438, 3), // "add"
QT_MOC_LITERAL(27, 442, 7), // "padding"
QT_MOC_LITERAL(28, 450, 10), // "topPadding"
QT_MOC_LITERAL(29, 461, 11), // "leftPadding"
QT_MOC_LITERAL(30, 473, 12), // "rightPadding"
QT_MOC_LITERAL(31, 486, 13) // "bottomPadding"

    },
    "QQuickBasePositioner\0QML.Element\0"
    "Positioner\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "Positioner is an abstract type that is only available as an attached p"
    "roperty.\0"
    "QML.Attached\0QQuickPositionerAttached\0"
    "spacingChanged\0\0populateChanged\0"
    "moveChanged\0addChanged\0paddingChanged\0"
    "topPaddingChanged\0leftPaddingChanged\0"
    "rightPaddingChanged\0bottomPaddingChanged\0"
    "positioningComplete\0prePositioning\0"
    "forceLayout\0spacing\0populate\0"
    "QQuickTransition*\0move\0add\0padding\0"
    "topPadding\0leftPadding\0rightPadding\0"
    "bottomPadding"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickBasePositioner[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
      12,   22, // methods
       9,  106, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

 // signals: name, argc, parameters, tag, flags
       9,    0,   94,   10, 0x06 /* Public */,
      11,    0,   95,   10, 0x06 /* Public */,
      12,    0,   96,   10, 0x06 /* Public */,
      13,    0,   97,   10, 0x06 /* Public */,
      14,    0,   98,   10, 0x86 /* Public | MethodRevisioned */,
      15,    0,   99,   10, 0x86 /* Public | MethodRevisioned */,
      16,    0,  100,   10, 0x86 /* Public | MethodRevisioned */,
      17,    0,  101,   10, 0x86 /* Public | MethodRevisioned */,
      18,    0,  102,   10, 0x86 /* Public | MethodRevisioned */,
      19,    0,  103,   10, 0x86 /* Public | MethodRevisioned */,

 // slots: name, argc, parameters, tag, flags
      20,    0,  104,   10, 0x09 /* Protected */,

 // methods: name, argc, parameters, tag, flags
      21,    0,  105,   10, 0x82 /* Public | MethodRevisioned */,

 // signals: revision
       0,
       0,
       0,
       0,
       6,
       6,
       6,
       6,
       6,
       9,

 // slots: revision
       0,

 // methods: revision
       9,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void,

 // properties: name, type, flags
      22, QMetaType::QReal, 0x00495103,
      23, 0x80000000 | 24, 0x0049510b,
      25, 0x80000000 | 24, 0x0049510b,
      26, 0x80000000 | 24, 0x0049510b,
      27, QMetaType::QReal, 0x00c95107,
      28, QMetaType::QReal, 0x00c95107,
      29, QMetaType::QReal, 0x00c95107,
      30, QMetaType::QReal, 0x00c95107,
      31, QMetaType::QReal, 0x00c95107,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,

 // properties: revision
       0,
       0,
       0,
       0,
       6,
       6,
       6,
       6,
       6,

       0        // eod
};

void QQuickBasePositioner::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickBasePositioner *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->spacingChanged(); break;
        case 1: _t->populateChanged(); break;
        case 2: _t->moveChanged(); break;
        case 3: _t->addChanged(); break;
        case 4: _t->paddingChanged(); break;
        case 5: _t->topPaddingChanged(); break;
        case 6: _t->leftPaddingChanged(); break;
        case 7: _t->rightPaddingChanged(); break;
        case 8: _t->bottomPaddingChanged(); break;
        case 9: _t->positioningComplete(); break;
        case 10: _t->prePositioning(); break;
        case 11: _t->forceLayout(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickBasePositioner::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBasePositioner::spacingChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickBasePositioner::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBasePositioner::populateChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickBasePositioner::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBasePositioner::moveChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickBasePositioner::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBasePositioner::addChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickBasePositioner::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBasePositioner::paddingChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickBasePositioner::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBasePositioner::topPaddingChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickBasePositioner::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBasePositioner::leftPaddingChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickBasePositioner::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBasePositioner::rightPaddingChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickBasePositioner::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBasePositioner::bottomPaddingChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickBasePositioner::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBasePositioner::positioningComplete)) {
                *result = 9;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
        case 2:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickTransition* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickBasePositioner *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->spacing(); break;
        case 1: *reinterpret_cast< QQuickTransition**>(_v) = _t->populate(); break;
        case 2: *reinterpret_cast< QQuickTransition**>(_v) = _t->move(); break;
        case 3: *reinterpret_cast< QQuickTransition**>(_v) = _t->add(); break;
        case 4: *reinterpret_cast< qreal*>(_v) = _t->padding(); break;
        case 5: *reinterpret_cast< qreal*>(_v) = _t->topPadding(); break;
        case 6: *reinterpret_cast< qreal*>(_v) = _t->leftPadding(); break;
        case 7: *reinterpret_cast< qreal*>(_v) = _t->rightPadding(); break;
        case 8: *reinterpret_cast< qreal*>(_v) = _t->bottomPadding(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickBasePositioner *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSpacing(*reinterpret_cast< qreal*>(_v)); break;
        case 1: _t->setPopulate(*reinterpret_cast< QQuickTransition**>(_v)); break;
        case 2: _t->setMove(*reinterpret_cast< QQuickTransition**>(_v)); break;
        case 3: _t->setAdd(*reinterpret_cast< QQuickTransition**>(_v)); break;
        case 4: _t->setPadding(*reinterpret_cast< qreal*>(_v)); break;
        case 5: _t->setTopPadding(*reinterpret_cast< qreal*>(_v)); break;
        case 6: _t->setLeftPadding(*reinterpret_cast< qreal*>(_v)); break;
        case 7: _t->setRightPadding(*reinterpret_cast< qreal*>(_v)); break;
        case 8: _t->setBottomPadding(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickBasePositioner *_t = static_cast<QQuickBasePositioner *>(_o);
        (void)_t;
        switch (_id) {
        case 4: _t->resetPadding(); break;
        case 5: _t->resetTopPadding(); break;
        case 6: _t->resetLeftPadding(); break;
        case 7: _t->resetRightPadding(); break;
        case 8: _t->resetBottomPadding(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickBasePositioner::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickImplicitSizeItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickBasePositioner.data,
    qt_meta_data_QQuickBasePositioner,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickBasePositioner::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickBasePositioner::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickBasePositioner.stringdata0))
        return static_cast<void*>(this);
    return QQuickImplicitSizeItem::qt_metacast(_clname);
}

int QQuickBasePositioner::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickImplicitSizeItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 9;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickBasePositioner::spacingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickBasePositioner::populateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickBasePositioner::moveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickBasePositioner::addChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickBasePositioner::paddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickBasePositioner::topPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickBasePositioner::leftPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickBasePositioner::rightPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickBasePositioner::bottomPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickBasePositioner::positioningComplete()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
struct qt_meta_stringdata_QQuickColumn_t {
    QByteArrayData data[3];
    char stringdata0[32];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickColumn_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickColumn_t qt_meta_stringdata_QQuickColumn = {
    {
QT_MOC_LITERAL(0, 0, 12), // "QQuickColumn"
QT_MOC_LITERAL(1, 13, 11), // "QML.Element"
QT_MOC_LITERAL(2, 25, 6) // "Column"

    },
    "QQuickColumn\0QML.Element\0Column"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickColumn[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

       0        // eod
};

void QQuickColumn::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickColumn::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickBasePositioner::staticMetaObject>(),
    qt_meta_stringdata_QQuickColumn.data,
    qt_meta_data_QQuickColumn,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickColumn::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickColumn::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickColumn.stringdata0))
        return static_cast<void*>(this);
    return QQuickBasePositioner::qt_metacast(_clname);
}

int QQuickColumn::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickBasePositioner::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQuickRow_t {
    QByteArrayData data[9];
    char stringdata0[143];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickRow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickRow_t qt_meta_stringdata_QQuickRow = {
    {
QT_MOC_LITERAL(0, 0, 9), // "QQuickRow"
QT_MOC_LITERAL(1, 10, 11), // "QML.Element"
QT_MOC_LITERAL(2, 22, 3), // "Row"
QT_MOC_LITERAL(3, 26, 22), // "layoutDirectionChanged"
QT_MOC_LITERAL(4, 49, 0), // ""
QT_MOC_LITERAL(5, 50, 31), // "effectiveLayoutDirectionChanged"
QT_MOC_LITERAL(6, 82, 15), // "layoutDirection"
QT_MOC_LITERAL(7, 98, 19), // "Qt::LayoutDirection"
QT_MOC_LITERAL(8, 118, 24) // "effectiveLayoutDirection"

    },
    "QQuickRow\0QML.Element\0Row\0"
    "layoutDirectionChanged\0\0"
    "effectiveLayoutDirectionChanged\0"
    "layoutDirection\0Qt::LayoutDirection\0"
    "effectiveLayoutDirection"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickRow[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       2,   16, // methods
       2,   28, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   26,    4, 0x06 /* Public */,
       5,    0,   27,    4, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       6, 0x80000000 | 7, 0x0049510b,
       8, 0x80000000 | 7, 0x00495009,

 // properties: notify_signal_id
       0,
       1,

       0        // eod
};

void QQuickRow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickRow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->layoutDirectionChanged(); break;
        case 1: _t->effectiveLayoutDirectionChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickRow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickRow::layoutDirectionChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickRow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickRow::effectiveLayoutDirectionChanged)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickRow *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< Qt::LayoutDirection*>(_v) = _t->layoutDirection(); break;
        case 1: *reinterpret_cast< Qt::LayoutDirection*>(_v) = _t->effectiveLayoutDirection(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickRow *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLayoutDirection(*reinterpret_cast< Qt::LayoutDirection*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickRow::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickBasePositioner::staticMetaObject>(),
    qt_meta_stringdata_QQuickRow.data,
    qt_meta_data_QQuickRow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickRow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickRow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickRow.stringdata0))
        return static_cast<void*>(this);
    return QQuickBasePositioner::qt_metacast(_clname);
}

int QQuickRow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickBasePositioner::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickRow::layoutDirectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickRow::effectiveLayoutDirectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_QQuickGrid_t {
    QByteArrayData data[37];
    char stringdata0[562];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickGrid_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickGrid_t qt_meta_stringdata_QQuickGrid = {
    {
QT_MOC_LITERAL(0, 0, 10), // "QQuickGrid"
QT_MOC_LITERAL(1, 11, 11), // "QML.Element"
QT_MOC_LITERAL(2, 23, 4), // "Grid"
QT_MOC_LITERAL(3, 28, 11), // "rowsChanged"
QT_MOC_LITERAL(4, 40, 0), // ""
QT_MOC_LITERAL(5, 41, 14), // "columnsChanged"
QT_MOC_LITERAL(6, 56, 11), // "flowChanged"
QT_MOC_LITERAL(7, 68, 22), // "layoutDirectionChanged"
QT_MOC_LITERAL(8, 91, 31), // "effectiveLayoutDirectionChanged"
QT_MOC_LITERAL(9, 123, 17), // "rowSpacingChanged"
QT_MOC_LITERAL(10, 141, 20), // "columnSpacingChanged"
QT_MOC_LITERAL(11, 162, 26), // "horizontalAlignmentChanged"
QT_MOC_LITERAL(12, 189, 10), // "HAlignment"
QT_MOC_LITERAL(13, 200, 9), // "alignment"
QT_MOC_LITERAL(14, 210, 35), // "effectiveHorizontalAlignmentC..."
QT_MOC_LITERAL(15, 246, 24), // "verticalAlignmentChanged"
QT_MOC_LITERAL(16, 271, 10), // "VAlignment"
QT_MOC_LITERAL(17, 282, 4), // "rows"
QT_MOC_LITERAL(18, 287, 7), // "columns"
QT_MOC_LITERAL(19, 295, 10), // "rowSpacing"
QT_MOC_LITERAL(20, 306, 13), // "columnSpacing"
QT_MOC_LITERAL(21, 320, 4), // "flow"
QT_MOC_LITERAL(22, 325, 4), // "Flow"
QT_MOC_LITERAL(23, 330, 15), // "layoutDirection"
QT_MOC_LITERAL(24, 346, 19), // "Qt::LayoutDirection"
QT_MOC_LITERAL(25, 366, 24), // "effectiveLayoutDirection"
QT_MOC_LITERAL(26, 391, 23), // "horizontalItemAlignment"
QT_MOC_LITERAL(27, 415, 32), // "effectiveHorizontalItemAlignment"
QT_MOC_LITERAL(28, 448, 21), // "verticalItemAlignment"
QT_MOC_LITERAL(29, 470, 11), // "LeftToRight"
QT_MOC_LITERAL(30, 482, 11), // "TopToBottom"
QT_MOC_LITERAL(31, 494, 9), // "AlignLeft"
QT_MOC_LITERAL(32, 504, 10), // "AlignRight"
QT_MOC_LITERAL(33, 515, 12), // "AlignHCenter"
QT_MOC_LITERAL(34, 528, 8), // "AlignTop"
QT_MOC_LITERAL(35, 537, 11), // "AlignBottom"
QT_MOC_LITERAL(36, 549, 12) // "AlignVCenter"

    },
    "QQuickGrid\0QML.Element\0Grid\0rowsChanged\0"
    "\0columnsChanged\0flowChanged\0"
    "layoutDirectionChanged\0"
    "effectiveLayoutDirectionChanged\0"
    "rowSpacingChanged\0columnSpacingChanged\0"
    "horizontalAlignmentChanged\0HAlignment\0"
    "alignment\0effectiveHorizontalAlignmentChanged\0"
    "verticalAlignmentChanged\0VAlignment\0"
    "rows\0columns\0rowSpacing\0columnSpacing\0"
    "flow\0Flow\0layoutDirection\0Qt::LayoutDirection\0"
    "effectiveLayoutDirection\0"
    "horizontalItemAlignment\0"
    "effectiveHorizontalItemAlignment\0"
    "verticalItemAlignment\0LeftToRight\0"
    "TopToBottom\0AlignLeft\0AlignRight\0"
    "AlignHCenter\0AlignTop\0AlignBottom\0"
    "AlignVCenter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickGrid[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      10,   16, // methods
      10,   92, // properties
       3,  142, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   76,    4, 0x06 /* Public */,
       5,    0,   77,    4, 0x06 /* Public */,
       6,    0,   78,    4, 0x06 /* Public */,
       7,    0,   79,    4, 0x06 /* Public */,
       8,    0,   80,    4, 0x06 /* Public */,
       9,    0,   81,    4, 0x06 /* Public */,
      10,    0,   82,    4, 0x06 /* Public */,
      11,    1,   83,    4, 0x86 /* Public | MethodRevisioned */,
      14,    1,   86,    4, 0x86 /* Public | MethodRevisioned */,
      15,    1,   89,    4, 0x86 /* Public | MethodRevisioned */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       1,
       1,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 16,   13,

 // properties: name, type, flags
      17, QMetaType::Int, 0x00495103,
      18, QMetaType::Int, 0x00495103,
      19, QMetaType::QReal, 0x00495107,
      20, QMetaType::QReal, 0x00495107,
      21, 0x80000000 | 22, 0x0049510b,
      23, 0x80000000 | 24, 0x0049510b,
      25, 0x80000000 | 24, 0x00495009,
      26, 0x80000000 | 12, 0x00c9500b,
      27, 0x80000000 | 12, 0x00c95009,
      28, 0x80000000 | 16, 0x00c9500b,

 // properties: notify_signal_id
       0,
       1,
       5,
       6,
       2,
       3,
       4,
       7,
       8,
       9,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       1,
       1,

 // enums: name, alias, flags, count, data
      22,   22, 0x0,    2,  157,
      12,   12, 0x0,    3,  161,
      16,   16, 0x0,    3,  167,

 // enum data: key, value
      29, uint(QQuickGrid::LeftToRight),
      30, uint(QQuickGrid::TopToBottom),
      31, uint(QQuickGrid::AlignLeft),
      32, uint(QQuickGrid::AlignRight),
      33, uint(QQuickGrid::AlignHCenter),
      34, uint(QQuickGrid::AlignTop),
      35, uint(QQuickGrid::AlignBottom),
      36, uint(QQuickGrid::AlignVCenter),

       0        // eod
};

void QQuickGrid::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickGrid *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->rowsChanged(); break;
        case 1: _t->columnsChanged(); break;
        case 2: _t->flowChanged(); break;
        case 3: _t->layoutDirectionChanged(); break;
        case 4: _t->effectiveLayoutDirectionChanged(); break;
        case 5: _t->rowSpacingChanged(); break;
        case 6: _t->columnSpacingChanged(); break;
        case 7: _t->horizontalAlignmentChanged((*reinterpret_cast< HAlignment(*)>(_a[1]))); break;
        case 8: _t->effectiveHorizontalAlignmentChanged((*reinterpret_cast< HAlignment(*)>(_a[1]))); break;
        case 9: _t->verticalAlignmentChanged((*reinterpret_cast< VAlignment(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickGrid::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGrid::rowsChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickGrid::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGrid::columnsChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickGrid::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGrid::flowChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickGrid::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGrid::layoutDirectionChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickGrid::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGrid::effectiveLayoutDirectionChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickGrid::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGrid::rowSpacingChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickGrid::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGrid::columnSpacingChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickGrid::*)(HAlignment );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGrid::horizontalAlignmentChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickGrid::*)(HAlignment );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGrid::effectiveHorizontalAlignmentChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickGrid::*)(VAlignment );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGrid::verticalAlignmentChanged)) {
                *result = 9;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickGrid *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->rows(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->columns(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->rowSpacing(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->columnSpacing(); break;
        case 4: *reinterpret_cast< Flow*>(_v) = _t->flow(); break;
        case 5: *reinterpret_cast< Qt::LayoutDirection*>(_v) = _t->layoutDirection(); break;
        case 6: *reinterpret_cast< Qt::LayoutDirection*>(_v) = _t->effectiveLayoutDirection(); break;
        case 7: *reinterpret_cast< HAlignment*>(_v) = _t->hItemAlign(); break;
        case 8: *reinterpret_cast< HAlignment*>(_v) = _t->effectiveHAlign(); break;
        case 9: *reinterpret_cast< VAlignment*>(_v) = _t->vItemAlign(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickGrid *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRows(*reinterpret_cast< int*>(_v)); break;
        case 1: _t->setColumns(*reinterpret_cast< int*>(_v)); break;
        case 2: _t->setRowSpacing(*reinterpret_cast< qreal*>(_v)); break;
        case 3: _t->setColumnSpacing(*reinterpret_cast< qreal*>(_v)); break;
        case 4: _t->setFlow(*reinterpret_cast< Flow*>(_v)); break;
        case 5: _t->setLayoutDirection(*reinterpret_cast< Qt::LayoutDirection*>(_v)); break;
        case 7: _t->setHItemAlign(*reinterpret_cast< HAlignment*>(_v)); break;
        case 9: _t->setVItemAlign(*reinterpret_cast< VAlignment*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickGrid *_t = static_cast<QQuickGrid *>(_o);
        (void)_t;
        switch (_id) {
        case 2: _t->resetRowSpacing(); break;
        case 3: _t->resetColumnSpacing(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickGrid::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickBasePositioner::staticMetaObject>(),
    qt_meta_stringdata_QQuickGrid.data,
    qt_meta_data_QQuickGrid,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickGrid::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickGrid::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickGrid.stringdata0))
        return static_cast<void*>(this);
    return QQuickBasePositioner::qt_metacast(_clname);
}

int QQuickGrid::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickBasePositioner::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 10;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickGrid::rowsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickGrid::columnsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickGrid::flowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickGrid::layoutDirectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickGrid::effectiveLayoutDirectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickGrid::rowSpacingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickGrid::columnSpacingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickGrid::horizontalAlignmentChanged(HAlignment _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QQuickGrid::effectiveHorizontalAlignmentChanged(HAlignment _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QQuickGrid::verticalAlignmentChanged(VAlignment _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}
struct qt_meta_stringdata_QQuickFlow_t {
    QByteArrayData data[13];
    char stringdata0[186];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickFlow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickFlow_t qt_meta_stringdata_QQuickFlow = {
    {
QT_MOC_LITERAL(0, 0, 10), // "QQuickFlow"
QT_MOC_LITERAL(1, 11, 11), // "QML.Element"
QT_MOC_LITERAL(2, 23, 4), // "Flow"
QT_MOC_LITERAL(3, 28, 11), // "flowChanged"
QT_MOC_LITERAL(4, 40, 0), // ""
QT_MOC_LITERAL(5, 41, 22), // "layoutDirectionChanged"
QT_MOC_LITERAL(6, 64, 31), // "effectiveLayoutDirectionChanged"
QT_MOC_LITERAL(7, 96, 4), // "flow"
QT_MOC_LITERAL(8, 101, 15), // "layoutDirection"
QT_MOC_LITERAL(9, 117, 19), // "Qt::LayoutDirection"
QT_MOC_LITERAL(10, 137, 24), // "effectiveLayoutDirection"
QT_MOC_LITERAL(11, 162, 11), // "LeftToRight"
QT_MOC_LITERAL(12, 174, 11) // "TopToBottom"

    },
    "QQuickFlow\0QML.Element\0Flow\0flowChanged\0"
    "\0layoutDirectionChanged\0"
    "effectiveLayoutDirectionChanged\0flow\0"
    "layoutDirection\0Qt::LayoutDirection\0"
    "effectiveLayoutDirection\0LeftToRight\0"
    "TopToBottom"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickFlow[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       3,   16, // methods
       3,   34, // properties
       1,   46, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   31,    4, 0x06 /* Public */,
       5,    0,   32,    4, 0x06 /* Public */,
       6,    0,   33,    4, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       7, 0x80000000 | 2, 0x0049510b,
       8, 0x80000000 | 9, 0x0049510b,
      10, 0x80000000 | 9, 0x00495009,

 // properties: notify_signal_id
       0,
       1,
       2,

 // enums: name, alias, flags, count, data
       2,    2, 0x0,    2,   51,

 // enum data: key, value
      11, uint(QQuickFlow::LeftToRight),
      12, uint(QQuickFlow::TopToBottom),

       0        // eod
};

void QQuickFlow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickFlow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->flowChanged(); break;
        case 1: _t->layoutDirectionChanged(); break;
        case 2: _t->effectiveLayoutDirectionChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickFlow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickFlow::flowChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickFlow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickFlow::layoutDirectionChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickFlow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickFlow::effectiveLayoutDirectionChanged)) {
                *result = 2;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickFlow *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< Flow*>(_v) = _t->flow(); break;
        case 1: *reinterpret_cast< Qt::LayoutDirection*>(_v) = _t->layoutDirection(); break;
        case 2: *reinterpret_cast< Qt::LayoutDirection*>(_v) = _t->effectiveLayoutDirection(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickFlow *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFlow(*reinterpret_cast< Flow*>(_v)); break;
        case 1: _t->setLayoutDirection(*reinterpret_cast< Qt::LayoutDirection*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickFlow::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickBasePositioner::staticMetaObject>(),
    qt_meta_stringdata_QQuickFlow.data,
    qt_meta_data_QQuickFlow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickFlow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickFlow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickFlow.stringdata0))
        return static_cast<void*>(this);
    return QQuickBasePositioner::qt_metacast(_clname);
}

int QQuickFlow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickBasePositioner::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickFlow::flowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickFlow::layoutDirectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickFlow::effectiveLayoutDirectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
