/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <private/qqmlbind_p.h>
#include <private/qqmlcomponentattached_p.h>
#include <private/qqmlconnections_p.h>
#include <private/qqmlengine_p.h>
#include <private/qqmllocale_p.h>
#include <private/qqmlloggingcategory_p.h>
#include <private/qqmltimer_p.h>
#include <private/qqmltypenotavailable_p.h>
#include <private/qqmlvaluetype_p.h>
#include <qqmlcomponent.h>

void qml_register_types_QtQml()
{
    qmlRegisterTypesAndRevisions<QObjectForeign>("QtQml", 2);
    qmlRegisterTypesAndRevisions<QQmlBind>("QtQml", 2);
    qmlRegisterTypesAndRevisions<QQmlComponent>("QtQml", 2);
    qmlRegisterTypesAndRevisions<QQmlComponentAttached>("QtQml", 2);
    qmlRegisterTypesAndRevisions<QQmlConnections>("QtQml", 2);
    qmlRegisterTypesAndRevisions<QQmlEasingValueType>("QtQml", 2);
    qmlRegisterTypesAndRevisions<QQmlLocale>("QtQml", 2);
    qmlRegisterTypesAndRevisions<QQmlLoggingCategory>("QtQml", 2);
    qmlRegisterTypesAndRevisions<QQmlTimer>("QtQml", 2);
    qmlRegisterTypesAndRevisions<QQmlTypeNotAvailable>("QtQml", 2);
    qmlRegisterModule("QtQml", 2, 15);
}

static const QQmlModuleRegistration registration("QtQml", 2, qml_register_types_QtQml);
