/****************************************************************************
** Meta object code from reading C++ file 'qquicktextedit_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquicktextedit_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquicktextedit_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickTextEdit_t {
    QByteArrayData data[183];
    char stringdata0[2629];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickTextEdit_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickTextEdit_t qt_meta_stringdata_QQuickTextEdit = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QQuickTextEdit"
QT_MOC_LITERAL(1, 15, 11), // "QML.Element"
QT_MOC_LITERAL(2, 27, 8), // "TextEdit"
QT_MOC_LITERAL(3, 36, 11), // "textChanged"
QT_MOC_LITERAL(4, 48, 0), // ""
QT_MOC_LITERAL(5, 49, 18), // "preeditTextChanged"
QT_MOC_LITERAL(6, 68, 18), // "contentSizeChanged"
QT_MOC_LITERAL(7, 87, 21), // "cursorPositionChanged"
QT_MOC_LITERAL(8, 109, 22), // "cursorRectangleChanged"
QT_MOC_LITERAL(9, 132, 21), // "selectionStartChanged"
QT_MOC_LITERAL(10, 154, 19), // "selectionEndChanged"
QT_MOC_LITERAL(11, 174, 19), // "selectedTextChanged"
QT_MOC_LITERAL(12, 194, 12), // "colorChanged"
QT_MOC_LITERAL(13, 207, 5), // "color"
QT_MOC_LITERAL(14, 213, 21), // "selectionColorChanged"
QT_MOC_LITERAL(15, 235, 24), // "selectedTextColorChanged"
QT_MOC_LITERAL(16, 260, 11), // "fontChanged"
QT_MOC_LITERAL(17, 272, 4), // "font"
QT_MOC_LITERAL(18, 277, 26), // "horizontalAlignmentChanged"
QT_MOC_LITERAL(19, 304, 26), // "QQuickTextEdit::HAlignment"
QT_MOC_LITERAL(20, 331, 9), // "alignment"
QT_MOC_LITERAL(21, 341, 24), // "verticalAlignmentChanged"
QT_MOC_LITERAL(22, 366, 26), // "QQuickTextEdit::VAlignment"
QT_MOC_LITERAL(23, 393, 15), // "wrapModeChanged"
QT_MOC_LITERAL(24, 409, 16), // "lineCountChanged"
QT_MOC_LITERAL(25, 426, 17), // "textFormatChanged"
QT_MOC_LITERAL(26, 444, 26), // "QQuickTextEdit::TextFormat"
QT_MOC_LITERAL(27, 471, 10), // "textFormat"
QT_MOC_LITERAL(28, 482, 15), // "readOnlyChanged"
QT_MOC_LITERAL(29, 498, 10), // "isReadOnly"
QT_MOC_LITERAL(30, 509, 20), // "cursorVisibleChanged"
QT_MOC_LITERAL(31, 530, 15), // "isCursorVisible"
QT_MOC_LITERAL(32, 546, 21), // "cursorDelegateChanged"
QT_MOC_LITERAL(33, 568, 20), // "overwriteModeChanged"
QT_MOC_LITERAL(34, 589, 13), // "overwriteMode"
QT_MOC_LITERAL(35, 603, 25), // "activeFocusOnPressChanged"
QT_MOC_LITERAL(36, 629, 20), // "activeFocusOnPressed"
QT_MOC_LITERAL(37, 650, 26), // "persistentSelectionChanged"
QT_MOC_LITERAL(38, 677, 21), // "isPersistentSelection"
QT_MOC_LITERAL(39, 699, 17), // "textMarginChanged"
QT_MOC_LITERAL(40, 717, 10), // "textMargin"
QT_MOC_LITERAL(41, 728, 23), // "selectByKeyboardChanged"
QT_MOC_LITERAL(42, 752, 16), // "selectByKeyboard"
QT_MOC_LITERAL(43, 769, 20), // "selectByMouseChanged"
QT_MOC_LITERAL(44, 790, 13), // "selectByMouse"
QT_MOC_LITERAL(45, 804, 25), // "mouseSelectionModeChanged"
QT_MOC_LITERAL(46, 830, 29), // "QQuickTextEdit::SelectionMode"
QT_MOC_LITERAL(47, 860, 4), // "mode"
QT_MOC_LITERAL(48, 865, 13), // "linkActivated"
QT_MOC_LITERAL(49, 879, 4), // "link"
QT_MOC_LITERAL(50, 884, 11), // "linkHovered"
QT_MOC_LITERAL(51, 896, 15), // "canPasteChanged"
QT_MOC_LITERAL(52, 912, 14), // "canUndoChanged"
QT_MOC_LITERAL(53, 927, 14), // "canRedoChanged"
QT_MOC_LITERAL(54, 942, 27), // "inputMethodComposingChanged"
QT_MOC_LITERAL(55, 970, 35), // "effectiveHorizontalAlignmentC..."
QT_MOC_LITERAL(56, 1006, 14), // "baseUrlChanged"
QT_MOC_LITERAL(57, 1021, 23), // "inputMethodHintsChanged"
QT_MOC_LITERAL(58, 1045, 17), // "renderTypeChanged"
QT_MOC_LITERAL(59, 1063, 15), // "editingFinished"
QT_MOC_LITERAL(60, 1079, 14), // "paddingChanged"
QT_MOC_LITERAL(61, 1094, 17), // "topPaddingChanged"
QT_MOC_LITERAL(62, 1112, 18), // "leftPaddingChanged"
QT_MOC_LITERAL(63, 1131, 19), // "rightPaddingChanged"
QT_MOC_LITERAL(64, 1151, 20), // "bottomPaddingChanged"
QT_MOC_LITERAL(65, 1172, 22), // "tabStopDistanceChanged"
QT_MOC_LITERAL(66, 1195, 8), // "distance"
QT_MOC_LITERAL(67, 1204, 9), // "selectAll"
QT_MOC_LITERAL(68, 1214, 10), // "selectWord"
QT_MOC_LITERAL(69, 1225, 6), // "select"
QT_MOC_LITERAL(70, 1232, 5), // "start"
QT_MOC_LITERAL(71, 1238, 3), // "end"
QT_MOC_LITERAL(72, 1242, 8), // "deselect"
QT_MOC_LITERAL(73, 1251, 13), // "isRightToLeft"
QT_MOC_LITERAL(74, 1265, 3), // "cut"
QT_MOC_LITERAL(75, 1269, 4), // "copy"
QT_MOC_LITERAL(76, 1274, 5), // "paste"
QT_MOC_LITERAL(77, 1280, 4), // "undo"
QT_MOC_LITERAL(78, 1285, 4), // "redo"
QT_MOC_LITERAL(79, 1290, 6), // "insert"
QT_MOC_LITERAL(80, 1297, 8), // "position"
QT_MOC_LITERAL(81, 1306, 4), // "text"
QT_MOC_LITERAL(82, 1311, 6), // "remove"
QT_MOC_LITERAL(83, 1318, 6), // "append"
QT_MOC_LITERAL(84, 1325, 5), // "clear"
QT_MOC_LITERAL(85, 1331, 13), // "q_textChanged"
QT_MOC_LITERAL(86, 1345, 16), // "q_contentsChange"
QT_MOC_LITERAL(87, 1362, 15), // "updateSelection"
QT_MOC_LITERAL(88, 1378, 18), // "moveCursorDelegate"
QT_MOC_LITERAL(89, 1397, 12), // "createCursor"
QT_MOC_LITERAL(90, 1410, 17), // "q_canPasteChanged"
QT_MOC_LITERAL(91, 1428, 19), // "updateWholeDocument"
QT_MOC_LITERAL(92, 1448, 15), // "invalidateBlock"
QT_MOC_LITERAL(93, 1464, 10), // "QTextBlock"
QT_MOC_LITERAL(94, 1475, 5), // "block"
QT_MOC_LITERAL(95, 1481, 12), // "updateCursor"
QT_MOC_LITERAL(96, 1494, 13), // "q_linkHovered"
QT_MOC_LITERAL(97, 1508, 15), // "q_markerHovered"
QT_MOC_LITERAL(98, 1524, 7), // "hovered"
QT_MOC_LITERAL(99, 1532, 17), // "q_updateAlignment"
QT_MOC_LITERAL(100, 1550, 10), // "updateSize"
QT_MOC_LITERAL(101, 1561, 17), // "triggerPreprocess"
QT_MOC_LITERAL(102, 1579, 16), // "inputMethodQuery"
QT_MOC_LITERAL(103, 1596, 20), // "Qt::InputMethodQuery"
QT_MOC_LITERAL(104, 1617, 5), // "query"
QT_MOC_LITERAL(105, 1623, 8), // "argument"
QT_MOC_LITERAL(106, 1632, 19), // "positionToRectangle"
QT_MOC_LITERAL(107, 1652, 10), // "positionAt"
QT_MOC_LITERAL(108, 1663, 1), // "x"
QT_MOC_LITERAL(109, 1665, 1), // "y"
QT_MOC_LITERAL(110, 1667, 19), // "moveCursorSelection"
QT_MOC_LITERAL(111, 1687, 3), // "pos"
QT_MOC_LITERAL(112, 1691, 13), // "SelectionMode"
QT_MOC_LITERAL(113, 1705, 7), // "getText"
QT_MOC_LITERAL(114, 1713, 16), // "getFormattedText"
QT_MOC_LITERAL(115, 1730, 6), // "linkAt"
QT_MOC_LITERAL(116, 1737, 14), // "selectionColor"
QT_MOC_LITERAL(117, 1752, 17), // "selectedTextColor"
QT_MOC_LITERAL(118, 1770, 19), // "horizontalAlignment"
QT_MOC_LITERAL(119, 1790, 10), // "HAlignment"
QT_MOC_LITERAL(120, 1801, 28), // "effectiveHorizontalAlignment"
QT_MOC_LITERAL(121, 1830, 17), // "verticalAlignment"
QT_MOC_LITERAL(122, 1848, 10), // "VAlignment"
QT_MOC_LITERAL(123, 1859, 8), // "wrapMode"
QT_MOC_LITERAL(124, 1868, 8), // "WrapMode"
QT_MOC_LITERAL(125, 1877, 9), // "lineCount"
QT_MOC_LITERAL(126, 1887, 6), // "length"
QT_MOC_LITERAL(127, 1894, 12), // "contentWidth"
QT_MOC_LITERAL(128, 1907, 13), // "contentHeight"
QT_MOC_LITERAL(129, 1921, 12), // "paintedWidth"
QT_MOC_LITERAL(130, 1934, 13), // "paintedHeight"
QT_MOC_LITERAL(131, 1948, 10), // "TextFormat"
QT_MOC_LITERAL(132, 1959, 8), // "readOnly"
QT_MOC_LITERAL(133, 1968, 13), // "cursorVisible"
QT_MOC_LITERAL(134, 1982, 14), // "cursorPosition"
QT_MOC_LITERAL(135, 1997, 15), // "cursorRectangle"
QT_MOC_LITERAL(136, 2013, 14), // "cursorDelegate"
QT_MOC_LITERAL(137, 2028, 14), // "QQmlComponent*"
QT_MOC_LITERAL(138, 2043, 14), // "selectionStart"
QT_MOC_LITERAL(139, 2058, 12), // "selectionEnd"
QT_MOC_LITERAL(140, 2071, 12), // "selectedText"
QT_MOC_LITERAL(141, 2084, 18), // "activeFocusOnPress"
QT_MOC_LITERAL(142, 2103, 19), // "persistentSelection"
QT_MOC_LITERAL(143, 2123, 16), // "inputMethodHints"
QT_MOC_LITERAL(144, 2140, 20), // "Qt::InputMethodHints"
QT_MOC_LITERAL(145, 2161, 18), // "mouseSelectionMode"
QT_MOC_LITERAL(146, 2180, 8), // "canPaste"
QT_MOC_LITERAL(147, 2189, 7), // "canUndo"
QT_MOC_LITERAL(148, 2197, 7), // "canRedo"
QT_MOC_LITERAL(149, 2205, 20), // "inputMethodComposing"
QT_MOC_LITERAL(150, 2226, 7), // "baseUrl"
QT_MOC_LITERAL(151, 2234, 10), // "renderType"
QT_MOC_LITERAL(152, 2245, 10), // "RenderType"
QT_MOC_LITERAL(153, 2256, 12), // "textDocument"
QT_MOC_LITERAL(154, 2269, 19), // "QQuickTextDocument*"
QT_MOC_LITERAL(155, 2289, 11), // "hoveredLink"
QT_MOC_LITERAL(156, 2301, 7), // "padding"
QT_MOC_LITERAL(157, 2309, 10), // "topPadding"
QT_MOC_LITERAL(158, 2320, 11), // "leftPadding"
QT_MOC_LITERAL(159, 2332, 12), // "rightPadding"
QT_MOC_LITERAL(160, 2345, 13), // "bottomPadding"
QT_MOC_LITERAL(161, 2359, 11), // "preeditText"
QT_MOC_LITERAL(162, 2371, 15), // "tabStopDistance"
QT_MOC_LITERAL(163, 2387, 9), // "AlignLeft"
QT_MOC_LITERAL(164, 2397, 10), // "AlignRight"
QT_MOC_LITERAL(165, 2408, 12), // "AlignHCenter"
QT_MOC_LITERAL(166, 2421, 12), // "AlignJustify"
QT_MOC_LITERAL(167, 2434, 8), // "AlignTop"
QT_MOC_LITERAL(168, 2443, 11), // "AlignBottom"
QT_MOC_LITERAL(169, 2455, 12), // "AlignVCenter"
QT_MOC_LITERAL(170, 2468, 9), // "PlainText"
QT_MOC_LITERAL(171, 2478, 8), // "RichText"
QT_MOC_LITERAL(172, 2487, 8), // "AutoText"
QT_MOC_LITERAL(173, 2496, 12), // "MarkdownText"
QT_MOC_LITERAL(174, 2509, 6), // "NoWrap"
QT_MOC_LITERAL(175, 2516, 8), // "WordWrap"
QT_MOC_LITERAL(176, 2525, 12), // "WrapAnywhere"
QT_MOC_LITERAL(177, 2538, 28), // "WrapAtWordBoundaryOrAnywhere"
QT_MOC_LITERAL(178, 2567, 4), // "Wrap"
QT_MOC_LITERAL(179, 2572, 16), // "SelectCharacters"
QT_MOC_LITERAL(180, 2589, 11), // "SelectWords"
QT_MOC_LITERAL(181, 2601, 11), // "QtRendering"
QT_MOC_LITERAL(182, 2613, 15) // "NativeRendering"

    },
    "QQuickTextEdit\0QML.Element\0TextEdit\0"
    "textChanged\0\0preeditTextChanged\0"
    "contentSizeChanged\0cursorPositionChanged\0"
    "cursorRectangleChanged\0selectionStartChanged\0"
    "selectionEndChanged\0selectedTextChanged\0"
    "colorChanged\0color\0selectionColorChanged\0"
    "selectedTextColorChanged\0fontChanged\0"
    "font\0horizontalAlignmentChanged\0"
    "QQuickTextEdit::HAlignment\0alignment\0"
    "verticalAlignmentChanged\0"
    "QQuickTextEdit::VAlignment\0wrapModeChanged\0"
    "lineCountChanged\0textFormatChanged\0"
    "QQuickTextEdit::TextFormat\0textFormat\0"
    "readOnlyChanged\0isReadOnly\0"
    "cursorVisibleChanged\0isCursorVisible\0"
    "cursorDelegateChanged\0overwriteModeChanged\0"
    "overwriteMode\0activeFocusOnPressChanged\0"
    "activeFocusOnPressed\0persistentSelectionChanged\0"
    "isPersistentSelection\0textMarginChanged\0"
    "textMargin\0selectByKeyboardChanged\0"
    "selectByKeyboard\0selectByMouseChanged\0"
    "selectByMouse\0mouseSelectionModeChanged\0"
    "QQuickTextEdit::SelectionMode\0mode\0"
    "linkActivated\0link\0linkHovered\0"
    "canPasteChanged\0canUndoChanged\0"
    "canRedoChanged\0inputMethodComposingChanged\0"
    "effectiveHorizontalAlignmentChanged\0"
    "baseUrlChanged\0inputMethodHintsChanged\0"
    "renderTypeChanged\0editingFinished\0"
    "paddingChanged\0topPaddingChanged\0"
    "leftPaddingChanged\0rightPaddingChanged\0"
    "bottomPaddingChanged\0tabStopDistanceChanged\0"
    "distance\0selectAll\0selectWord\0select\0"
    "start\0end\0deselect\0isRightToLeft\0cut\0"
    "copy\0paste\0undo\0redo\0insert\0position\0"
    "text\0remove\0append\0clear\0q_textChanged\0"
    "q_contentsChange\0updateSelection\0"
    "moveCursorDelegate\0createCursor\0"
    "q_canPasteChanged\0updateWholeDocument\0"
    "invalidateBlock\0QTextBlock\0block\0"
    "updateCursor\0q_linkHovered\0q_markerHovered\0"
    "hovered\0q_updateAlignment\0updateSize\0"
    "triggerPreprocess\0inputMethodQuery\0"
    "Qt::InputMethodQuery\0query\0argument\0"
    "positionToRectangle\0positionAt\0x\0y\0"
    "moveCursorSelection\0pos\0SelectionMode\0"
    "getText\0getFormattedText\0linkAt\0"
    "selectionColor\0selectedTextColor\0"
    "horizontalAlignment\0HAlignment\0"
    "effectiveHorizontalAlignment\0"
    "verticalAlignment\0VAlignment\0wrapMode\0"
    "WrapMode\0lineCount\0length\0contentWidth\0"
    "contentHeight\0paintedWidth\0paintedHeight\0"
    "TextFormat\0readOnly\0cursorVisible\0"
    "cursorPosition\0cursorRectangle\0"
    "cursorDelegate\0QQmlComponent*\0"
    "selectionStart\0selectionEnd\0selectedText\0"
    "activeFocusOnPress\0persistentSelection\0"
    "inputMethodHints\0Qt::InputMethodHints\0"
    "mouseSelectionMode\0canPaste\0canUndo\0"
    "canRedo\0inputMethodComposing\0baseUrl\0"
    "renderType\0RenderType\0textDocument\0"
    "QQuickTextDocument*\0hoveredLink\0padding\0"
    "topPadding\0leftPadding\0rightPadding\0"
    "bottomPadding\0preeditText\0tabStopDistance\0"
    "AlignLeft\0AlignRight\0AlignHCenter\0"
    "AlignJustify\0AlignTop\0AlignBottom\0"
    "AlignVCenter\0PlainText\0RichText\0"
    "AutoText\0MarkdownText\0NoWrap\0WordWrap\0"
    "WrapAnywhere\0WrapAtWordBoundaryOrAnywhere\0"
    "Wrap\0SelectCharacters\0SelectWords\0"
    "QtRendering\0NativeRendering"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickTextEdit[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      80,   16, // methods
      47,  672, // properties
       6,  907, // enums/sets
       0,    0, // constructors
       0,       // flags
      44,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,  496,    4, 0x06 /* Public */,
       5,    0,  497,    4, 0x86 /* Public | MethodRevisioned */,
       6,    0,  498,    4, 0x06 /* Public */,
       7,    0,  499,    4, 0x06 /* Public */,
       8,    0,  500,    4, 0x06 /* Public */,
       9,    0,  501,    4, 0x06 /* Public */,
      10,    0,  502,    4, 0x06 /* Public */,
      11,    0,  503,    4, 0x06 /* Public */,
      12,    1,  504,    4, 0x06 /* Public */,
      14,    1,  507,    4, 0x06 /* Public */,
      15,    1,  510,    4, 0x06 /* Public */,
      16,    1,  513,    4, 0x06 /* Public */,
      18,    1,  516,    4, 0x06 /* Public */,
      21,    1,  519,    4, 0x06 /* Public */,
      23,    0,  522,    4, 0x06 /* Public */,
      24,    0,  523,    4, 0x06 /* Public */,
      25,    1,  524,    4, 0x06 /* Public */,
      28,    1,  527,    4, 0x06 /* Public */,
      30,    1,  530,    4, 0x06 /* Public */,
      32,    0,  533,    4, 0x06 /* Public */,
      33,    1,  534,    4, 0x06 /* Public */,
      35,    1,  537,    4, 0x06 /* Public */,
      37,    1,  540,    4, 0x06 /* Public */,
      39,    1,  543,    4, 0x06 /* Public */,
      41,    1,  546,    4, 0x86 /* Public | MethodRevisioned */,
      43,    1,  549,    4, 0x06 /* Public */,
      45,    1,  552,    4, 0x06 /* Public */,
      48,    1,  555,    4, 0x06 /* Public */,
      50,    1,  558,    4, 0x86 /* Public | MethodRevisioned */,
      51,    0,  561,    4, 0x06 /* Public */,
      52,    0,  562,    4, 0x06 /* Public */,
      53,    0,  563,    4, 0x06 /* Public */,
      54,    0,  564,    4, 0x06 /* Public */,
      55,    0,  565,    4, 0x06 /* Public */,
      56,    0,  566,    4, 0x06 /* Public */,
      57,    0,  567,    4, 0x06 /* Public */,
      58,    0,  568,    4, 0x06 /* Public */,
      59,    0,  569,    4, 0x86 /* Public | MethodRevisioned */,
      60,    0,  570,    4, 0x86 /* Public | MethodRevisioned */,
      61,    0,  571,    4, 0x86 /* Public | MethodRevisioned */,
      62,    0,  572,    4, 0x86 /* Public | MethodRevisioned */,
      63,    0,  573,    4, 0x86 /* Public | MethodRevisioned */,
      64,    0,  574,    4, 0x86 /* Public | MethodRevisioned */,
      65,    1,  575,    4, 0x86 /* Public | MethodRevisioned */,

 // slots: name, argc, parameters, tag, flags
      67,    0,  578,    4, 0x0a /* Public */,
      68,    0,  579,    4, 0x0a /* Public */,
      69,    2,  580,    4, 0x0a /* Public */,
      72,    0,  585,    4, 0x0a /* Public */,
      73,    2,  586,    4, 0x0a /* Public */,
      74,    0,  591,    4, 0x0a /* Public */,
      75,    0,  592,    4, 0x0a /* Public */,
      76,    0,  593,    4, 0x0a /* Public */,
      77,    0,  594,    4, 0x0a /* Public */,
      78,    0,  595,    4, 0x0a /* Public */,
      79,    2,  596,    4, 0x0a /* Public */,
      82,    2,  601,    4, 0x0a /* Public */,
      83,    1,  606,    4, 0x8a /* Public | MethodRevisioned */,
      84,    0,  609,    4, 0x8a /* Public | MethodRevisioned */,
      85,    0,  610,    4, 0x08 /* Private */,
      86,    3,  611,    4, 0x08 /* Private */,
      87,    0,  618,    4, 0x08 /* Private */,
      88,    0,  619,    4, 0x08 /* Private */,
      89,    0,  620,    4, 0x08 /* Private */,
      90,    0,  621,    4, 0x08 /* Private */,
      91,    0,  622,    4, 0x08 /* Private */,
      92,    1,  623,    4, 0x08 /* Private */,
      95,    0,  626,    4, 0x08 /* Private */,
      96,    1,  627,    4, 0x08 /* Private */,
      97,    1,  630,    4, 0x08 /* Private */,
      99,    0,  633,    4, 0x08 /* Private */,
     100,    0,  634,    4, 0x08 /* Private */,
     101,    0,  635,    4, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
     102,    2,  636,    4, 0x82 /* Public | MethodRevisioned */,
     106,    1,  641,    4, 0x02 /* Public */,
     107,    2,  644,    4, 0x02 /* Public */,
     110,    1,  649,    4, 0x02 /* Public */,
     110,    2,  652,    4, 0x02 /* Public */,
     113,    2,  657,    4, 0x02 /* Public */,
     114,    2,  662,    4, 0x02 /* Public */,
     115,    2,  667,    4, 0x82 /* Public | MethodRevisioned */,

 // signals: revision
       0,
       7,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       0,
       0,
       0,
       2,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       6,
       6,
       6,
       6,
       6,
       6,
      10,

 // slots: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       2,
       7,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,

 // methods: revision
       4,
       0,
       0,
       0,
       0,
       0,
       0,
       3,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QColor,   13,
    QMetaType::Void, QMetaType::QColor,   13,
    QMetaType::Void, QMetaType::QColor,   13,
    QMetaType::Void, QMetaType::QFont,   17,
    QMetaType::Void, 0x80000000 | 19,   20,
    QMetaType::Void, 0x80000000 | 22,   20,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 26,   27,
    QMetaType::Void, QMetaType::Bool,   29,
    QMetaType::Void, QMetaType::Bool,   31,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   34,
    QMetaType::Void, QMetaType::Bool,   36,
    QMetaType::Void, QMetaType::Bool,   38,
    QMetaType::Void, QMetaType::QReal,   40,
    QMetaType::Void, QMetaType::Bool,   42,
    QMetaType::Void, QMetaType::Bool,   44,
    QMetaType::Void, 0x80000000 | 46,   47,
    QMetaType::Void, QMetaType::QString,   49,
    QMetaType::Void, QMetaType::QString,   49,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QReal,   66,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   70,   71,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::Int, QMetaType::Int,   70,   71,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   80,   81,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   70,   71,
    QMetaType::Void, QMetaType::QString,   81,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,    4,    4,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 93,   94,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   49,
    QMetaType::Void, QMetaType::Bool,   98,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::QVariant, 0x80000000 | 103, QMetaType::QVariant,  104,  105,
    QMetaType::QRectF, QMetaType::Int,    4,
    QMetaType::Int, QMetaType::QReal, QMetaType::QReal,  108,  109,
    QMetaType::Void, QMetaType::Int,  111,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 112,  111,   47,
    QMetaType::QString, QMetaType::Int, QMetaType::Int,   70,   71,
    QMetaType::QString, QMetaType::Int, QMetaType::Int,   70,   71,
    QMetaType::QString, QMetaType::QReal, QMetaType::QReal,  108,  109,

 // properties: name, type, flags
      81, QMetaType::QString, 0x00495103,
      13, QMetaType::QColor, 0x00495103,
     116, QMetaType::QColor, 0x00495103,
     117, QMetaType::QColor, 0x00495103,
      17, QMetaType::QFont, 0x00495103,
     118, 0x80000000 | 119, 0x0049500f,
     120, 0x80000000 | 119, 0x00495009,
     121, 0x80000000 | 122, 0x0049500b,
     123, 0x80000000 | 124, 0x0049510b,
     125, QMetaType::Int, 0x00495001,
     126, QMetaType::Int, 0x00495001,
     127, QMetaType::QReal, 0x00495001,
     128, QMetaType::QReal, 0x00495001,
     129, QMetaType::QReal, 0x00495001,
     130, QMetaType::QReal, 0x00495001,
      27, 0x80000000 | 131, 0x0049510b,
     132, QMetaType::Bool, 0x00495103,
     133, QMetaType::Bool, 0x00495103,
     134, QMetaType::Int, 0x00495103,
     135, QMetaType::QRectF, 0x00495001,
     136, 0x80000000 | 137, 0x0049510b,
      34, QMetaType::Bool, 0x00495103,
     138, QMetaType::Int, 0x00495001,
     139, QMetaType::Int, 0x00495001,
     140, QMetaType::QString, 0x00495001,
     141, QMetaType::Bool, 0x00495003,
     142, QMetaType::Bool, 0x00495103,
      40, QMetaType::QReal, 0x00495103,
     143, 0x80000000 | 144, 0x0049510b,
      42, QMetaType::Bool, 0x00c95103,
      44, QMetaType::Bool, 0x00495103,
     145, 0x80000000 | 112, 0x0049510b,
     146, QMetaType::Bool, 0x00495001,
     147, QMetaType::Bool, 0x00495001,
     148, QMetaType::Bool, 0x00495001,
     149, QMetaType::Bool, 0x00495001,
     150, QMetaType::QUrl, 0x00495107,
     151, 0x80000000 | 152, 0x0049510b,
     153, 0x80000000 | 154, 0x00895c09,
     155, QMetaType::QString, 0x00c95001,
     156, QMetaType::QReal, 0x00c95107,
     157, QMetaType::QReal, 0x00c95107,
     158, QMetaType::QReal, 0x00c95107,
     159, QMetaType::QReal, 0x00c95107,
     160, QMetaType::QReal, 0x00c95107,
     161, QMetaType::QString, 0x00c95001,
     162, QMetaType::QReal, 0x00c95103,

 // properties: notify_signal_id
       0,
       8,
       9,
      10,
      11,
      12,
      33,
      13,
      14,
      15,
       0,
       2,
       2,
       2,
       2,
      16,
      17,
      18,
       3,
       4,
      19,
      20,
       5,
       6,
       7,
      21,
      22,
      23,
      35,
      24,
      25,
      26,
      29,
      30,
      31,
      32,
      34,
      36,
       0,
      28,
      38,
      39,
      40,
      41,
      42,
       1,
      43,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       2,
       6,
       6,
       6,
       6,
       6,
       7,
      10,

 // enums: name, alias, flags, count, data
     119,  119, 0x0,    4,  937,
     122,  122, 0x0,    3,  945,
     131,  131, 0x0,    4,  951,
     124,  124, 0x0,    5,  959,
     112,  112, 0x0,    2,  969,
     152,  152, 0x0,    2,  973,

 // enum data: key, value
     163, uint(QQuickTextEdit::AlignLeft),
     164, uint(QQuickTextEdit::AlignRight),
     165, uint(QQuickTextEdit::AlignHCenter),
     166, uint(QQuickTextEdit::AlignJustify),
     167, uint(QQuickTextEdit::AlignTop),
     168, uint(QQuickTextEdit::AlignBottom),
     169, uint(QQuickTextEdit::AlignVCenter),
     170, uint(QQuickTextEdit::PlainText),
     171, uint(QQuickTextEdit::RichText),
     172, uint(QQuickTextEdit::AutoText),
     173, uint(QQuickTextEdit::MarkdownText),
     174, uint(QQuickTextEdit::NoWrap),
     175, uint(QQuickTextEdit::WordWrap),
     176, uint(QQuickTextEdit::WrapAnywhere),
     177, uint(QQuickTextEdit::WrapAtWordBoundaryOrAnywhere),
     178, uint(QQuickTextEdit::Wrap),
     179, uint(QQuickTextEdit::SelectCharacters),
     180, uint(QQuickTextEdit::SelectWords),
     181, uint(QQuickTextEdit::QtRendering),
     182, uint(QQuickTextEdit::NativeRendering),

       0        // eod
};

void QQuickTextEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickTextEdit *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->textChanged(); break;
        case 1: _t->preeditTextChanged(); break;
        case 2: _t->contentSizeChanged(); break;
        case 3: _t->cursorPositionChanged(); break;
        case 4: _t->cursorRectangleChanged(); break;
        case 5: _t->selectionStartChanged(); break;
        case 6: _t->selectionEndChanged(); break;
        case 7: _t->selectedTextChanged(); break;
        case 8: _t->colorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 9: _t->selectionColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 10: _t->selectedTextColorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 11: _t->fontChanged((*reinterpret_cast< const QFont(*)>(_a[1]))); break;
        case 12: _t->horizontalAlignmentChanged((*reinterpret_cast< QQuickTextEdit::HAlignment(*)>(_a[1]))); break;
        case 13: _t->verticalAlignmentChanged((*reinterpret_cast< QQuickTextEdit::VAlignment(*)>(_a[1]))); break;
        case 14: _t->wrapModeChanged(); break;
        case 15: _t->lineCountChanged(); break;
        case 16: _t->textFormatChanged((*reinterpret_cast< QQuickTextEdit::TextFormat(*)>(_a[1]))); break;
        case 17: _t->readOnlyChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 18: _t->cursorVisibleChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 19: _t->cursorDelegateChanged(); break;
        case 20: _t->overwriteModeChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->activeFocusOnPressChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 22: _t->persistentSelectionChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 23: _t->textMarginChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 24: _t->selectByKeyboardChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 25: _t->selectByMouseChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 26: _t->mouseSelectionModeChanged((*reinterpret_cast< QQuickTextEdit::SelectionMode(*)>(_a[1]))); break;
        case 27: _t->linkActivated((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 28: _t->linkHovered((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 29: _t->canPasteChanged(); break;
        case 30: _t->canUndoChanged(); break;
        case 31: _t->canRedoChanged(); break;
        case 32: _t->inputMethodComposingChanged(); break;
        case 33: _t->effectiveHorizontalAlignmentChanged(); break;
        case 34: _t->baseUrlChanged(); break;
        case 35: _t->inputMethodHintsChanged(); break;
        case 36: _t->renderTypeChanged(); break;
        case 37: _t->editingFinished(); break;
        case 38: _t->paddingChanged(); break;
        case 39: _t->topPaddingChanged(); break;
        case 40: _t->leftPaddingChanged(); break;
        case 41: _t->rightPaddingChanged(); break;
        case 42: _t->bottomPaddingChanged(); break;
        case 43: _t->tabStopDistanceChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 44: _t->selectAll(); break;
        case 45: _t->selectWord(); break;
        case 46: _t->select((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 47: _t->deselect(); break;
        case 48: { bool _r = _t->isRightToLeft((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 49: _t->cut(); break;
        case 50: _t->copy(); break;
        case 51: _t->paste(); break;
        case 52: _t->undo(); break;
        case 53: _t->redo(); break;
        case 54: _t->insert((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 55: _t->remove((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 56: _t->append((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 57: _t->clear(); break;
        case 58: _t->q_textChanged(); break;
        case 59: _t->q_contentsChange((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 60: _t->updateSelection(); break;
        case 61: _t->moveCursorDelegate(); break;
        case 62: _t->createCursor(); break;
        case 63: _t->q_canPasteChanged(); break;
        case 64: _t->updateWholeDocument(); break;
        case 65: _t->invalidateBlock((*reinterpret_cast< const QTextBlock(*)>(_a[1]))); break;
        case 66: _t->updateCursor(); break;
        case 67: _t->q_linkHovered((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 68: _t->q_markerHovered((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 69: _t->q_updateAlignment(); break;
        case 70: _t->updateSize(); break;
        case 71: _t->triggerPreprocess(); break;
        case 72: { QVariant _r = _t->inputMethodQuery((*reinterpret_cast< Qt::InputMethodQuery(*)>(_a[1])),(*reinterpret_cast< QVariant(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QVariant*>(_a[0]) = std::move(_r); }  break;
        case 73: { QRectF _r = _t->positionToRectangle((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QRectF*>(_a[0]) = std::move(_r); }  break;
        case 74: { int _r = _t->positionAt((*reinterpret_cast< qreal(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 75: _t->moveCursorSelection((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 76: _t->moveCursorSelection((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< SelectionMode(*)>(_a[2]))); break;
        case 77: { QString _r = _t->getText((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 78: { QString _r = _t->getFormattedText((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 79: { QString _r = _t->linkAt((*reinterpret_cast< qreal(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::textChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::preeditTextChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::contentSizeChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::cursorPositionChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::cursorRectangleChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::selectionStartChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::selectionEndChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::selectedTextChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::colorChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::selectionColorChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::selectedTextColorChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(const QFont & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::fontChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(QQuickTextEdit::HAlignment );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::horizontalAlignmentChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(QQuickTextEdit::VAlignment );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::verticalAlignmentChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::wrapModeChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::lineCountChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(QQuickTextEdit::TextFormat );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::textFormatChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::readOnlyChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::cursorVisibleChanged)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::cursorDelegateChanged)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::overwriteModeChanged)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::activeFocusOnPressChanged)) {
                *result = 21;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::persistentSelectionChanged)) {
                *result = 22;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::textMarginChanged)) {
                *result = 23;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::selectByKeyboardChanged)) {
                *result = 24;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::selectByMouseChanged)) {
                *result = 25;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(QQuickTextEdit::SelectionMode );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::mouseSelectionModeChanged)) {
                *result = 26;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::linkActivated)) {
                *result = 27;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::linkHovered)) {
                *result = 28;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::canPasteChanged)) {
                *result = 29;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::canUndoChanged)) {
                *result = 30;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::canRedoChanged)) {
                *result = 31;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::inputMethodComposingChanged)) {
                *result = 32;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::effectiveHorizontalAlignmentChanged)) {
                *result = 33;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::baseUrlChanged)) {
                *result = 34;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::inputMethodHintsChanged)) {
                *result = 35;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::renderTypeChanged)) {
                *result = 36;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::editingFinished)) {
                *result = 37;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::paddingChanged)) {
                *result = 38;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::topPaddingChanged)) {
                *result = 39;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::leftPaddingChanged)) {
                *result = 40;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::rightPaddingChanged)) {
                *result = 41;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::bottomPaddingChanged)) {
                *result = 42;
                return;
            }
        }
        {
            using _t = void (QQuickTextEdit::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextEdit::tabStopDistanceChanged)) {
                *result = 43;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 20:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickTextEdit *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->text(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->color(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->selectionColor(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->selectedTextColor(); break;
        case 4: *reinterpret_cast< QFont*>(_v) = _t->font(); break;
        case 5: *reinterpret_cast< HAlignment*>(_v) = _t->hAlign(); break;
        case 6: *reinterpret_cast< HAlignment*>(_v) = _t->effectiveHAlign(); break;
        case 7: *reinterpret_cast< VAlignment*>(_v) = _t->vAlign(); break;
        case 8: *reinterpret_cast< WrapMode*>(_v) = _t->wrapMode(); break;
        case 9: *reinterpret_cast< int*>(_v) = _t->lineCount(); break;
        case 10: *reinterpret_cast< int*>(_v) = _t->length(); break;
        case 11: *reinterpret_cast< qreal*>(_v) = _t->contentWidth(); break;
        case 12: *reinterpret_cast< qreal*>(_v) = _t->contentHeight(); break;
        case 13: *reinterpret_cast< qreal*>(_v) = _t->contentWidth(); break;
        case 14: *reinterpret_cast< qreal*>(_v) = _t->contentHeight(); break;
        case 15: *reinterpret_cast< TextFormat*>(_v) = _t->textFormat(); break;
        case 16: *reinterpret_cast< bool*>(_v) = _t->isReadOnly(); break;
        case 17: *reinterpret_cast< bool*>(_v) = _t->isCursorVisible(); break;
        case 18: *reinterpret_cast< int*>(_v) = _t->cursorPosition(); break;
        case 19: *reinterpret_cast< QRectF*>(_v) = _t->cursorRectangle(); break;
        case 20: *reinterpret_cast< QQmlComponent**>(_v) = _t->cursorDelegate(); break;
        case 21: *reinterpret_cast< bool*>(_v) = _t->overwriteMode(); break;
        case 22: *reinterpret_cast< int*>(_v) = _t->selectionStart(); break;
        case 23: *reinterpret_cast< int*>(_v) = _t->selectionEnd(); break;
        case 24: *reinterpret_cast< QString*>(_v) = _t->selectedText(); break;
        case 25: *reinterpret_cast< bool*>(_v) = _t->focusOnPress(); break;
        case 26: *reinterpret_cast< bool*>(_v) = _t->persistentSelection(); break;
        case 27: *reinterpret_cast< qreal*>(_v) = _t->textMargin(); break;
        case 28: *reinterpret_cast< Qt::InputMethodHints*>(_v) = _t->inputMethodHints(); break;
        case 29: *reinterpret_cast< bool*>(_v) = _t->selectByKeyboard(); break;
        case 30: *reinterpret_cast< bool*>(_v) = _t->selectByMouse(); break;
        case 31: *reinterpret_cast< SelectionMode*>(_v) = _t->mouseSelectionMode(); break;
        case 32: *reinterpret_cast< bool*>(_v) = _t->canPaste(); break;
        case 33: *reinterpret_cast< bool*>(_v) = _t->canUndo(); break;
        case 34: *reinterpret_cast< bool*>(_v) = _t->canRedo(); break;
        case 35: *reinterpret_cast< bool*>(_v) = _t->isInputMethodComposing(); break;
        case 36: *reinterpret_cast< QUrl*>(_v) = _t->baseUrl(); break;
        case 37: *reinterpret_cast< RenderType*>(_v) = _t->renderType(); break;
        case 38: *reinterpret_cast< QQuickTextDocument**>(_v) = _t->textDocument(); break;
        case 39: *reinterpret_cast< QString*>(_v) = _t->hoveredLink(); break;
        case 40: *reinterpret_cast< qreal*>(_v) = _t->padding(); break;
        case 41: *reinterpret_cast< qreal*>(_v) = _t->topPadding(); break;
        case 42: *reinterpret_cast< qreal*>(_v) = _t->leftPadding(); break;
        case 43: *reinterpret_cast< qreal*>(_v) = _t->rightPadding(); break;
        case 44: *reinterpret_cast< qreal*>(_v) = _t->bottomPadding(); break;
        case 45: *reinterpret_cast< QString*>(_v) = _t->preeditText(); break;
        case 46: *reinterpret_cast< qreal*>(_v) = _t->tabStopDistance(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickTextEdit *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setText(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setColor(*reinterpret_cast< QColor*>(_v)); break;
        case 2: _t->setSelectionColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setSelectedTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 4: _t->setFont(*reinterpret_cast< QFont*>(_v)); break;
        case 5: _t->setHAlign(*reinterpret_cast< HAlignment*>(_v)); break;
        case 7: _t->setVAlign(*reinterpret_cast< VAlignment*>(_v)); break;
        case 8: _t->setWrapMode(*reinterpret_cast< WrapMode*>(_v)); break;
        case 15: _t->setTextFormat(*reinterpret_cast< TextFormat*>(_v)); break;
        case 16: _t->setReadOnly(*reinterpret_cast< bool*>(_v)); break;
        case 17: _t->setCursorVisible(*reinterpret_cast< bool*>(_v)); break;
        case 18: _t->setCursorPosition(*reinterpret_cast< int*>(_v)); break;
        case 20: _t->setCursorDelegate(*reinterpret_cast< QQmlComponent**>(_v)); break;
        case 21: _t->setOverwriteMode(*reinterpret_cast< bool*>(_v)); break;
        case 25: _t->setFocusOnPress(*reinterpret_cast< bool*>(_v)); break;
        case 26: _t->setPersistentSelection(*reinterpret_cast< bool*>(_v)); break;
        case 27: _t->setTextMargin(*reinterpret_cast< qreal*>(_v)); break;
        case 28: _t->setInputMethodHints(*reinterpret_cast< Qt::InputMethodHints*>(_v)); break;
        case 29: _t->setSelectByKeyboard(*reinterpret_cast< bool*>(_v)); break;
        case 30: _t->setSelectByMouse(*reinterpret_cast< bool*>(_v)); break;
        case 31: _t->setMouseSelectionMode(*reinterpret_cast< SelectionMode*>(_v)); break;
        case 36: _t->setBaseUrl(*reinterpret_cast< QUrl*>(_v)); break;
        case 37: _t->setRenderType(*reinterpret_cast< RenderType*>(_v)); break;
        case 40: _t->setPadding(*reinterpret_cast< qreal*>(_v)); break;
        case 41: _t->setTopPadding(*reinterpret_cast< qreal*>(_v)); break;
        case 42: _t->setLeftPadding(*reinterpret_cast< qreal*>(_v)); break;
        case 43: _t->setRightPadding(*reinterpret_cast< qreal*>(_v)); break;
        case 44: _t->setBottomPadding(*reinterpret_cast< qreal*>(_v)); break;
        case 46: _t->setTabStopDistance(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickTextEdit *_t = static_cast<QQuickTextEdit *>(_o);
        (void)_t;
        switch (_id) {
        case 5: _t->resetHAlign(); break;
        case 36: _t->resetBaseUrl(); break;
        case 40: _t->resetPadding(); break;
        case 41: _t->resetTopPadding(); break;
        case 42: _t->resetLeftPadding(); break;
        case 43: _t->resetRightPadding(); break;
        case 44: _t->resetBottomPadding(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickTextEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickImplicitSizeItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickTextEdit.data,
    qt_meta_data_QQuickTextEdit,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickTextEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickTextEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickTextEdit.stringdata0))
        return static_cast<void*>(this);
    return QQuickImplicitSizeItem::qt_metacast(_clname);
}

int QQuickTextEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickImplicitSizeItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 80)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 80;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 80)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 80;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 47;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 47;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 47;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 47;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 47;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 47;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickTextEdit::textChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickTextEdit::preeditTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickTextEdit::contentSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickTextEdit::cursorPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickTextEdit::cursorRectangleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickTextEdit::selectionStartChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickTextEdit::selectionEndChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickTextEdit::selectedTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickTextEdit::colorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QQuickTextEdit::selectionColorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QQuickTextEdit::selectedTextColorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QQuickTextEdit::fontChanged(const QFont & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QQuickTextEdit::horizontalAlignmentChanged(QQuickTextEdit::HAlignment _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QQuickTextEdit::verticalAlignmentChanged(QQuickTextEdit::VAlignment _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QQuickTextEdit::wrapModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QQuickTextEdit::lineCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QQuickTextEdit::textFormatChanged(QQuickTextEdit::TextFormat _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void QQuickTextEdit::readOnlyChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void QQuickTextEdit::cursorVisibleChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void QQuickTextEdit::cursorDelegateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void QQuickTextEdit::overwriteModeChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 20, _a);
}

// SIGNAL 21
void QQuickTextEdit::activeFocusOnPressChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 21, _a);
}

// SIGNAL 22
void QQuickTextEdit::persistentSelectionChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 22, _a);
}

// SIGNAL 23
void QQuickTextEdit::textMarginChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 23, _a);
}

// SIGNAL 24
void QQuickTextEdit::selectByKeyboardChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 24, _a);
}

// SIGNAL 25
void QQuickTextEdit::selectByMouseChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 25, _a);
}

// SIGNAL 26
void QQuickTextEdit::mouseSelectionModeChanged(QQuickTextEdit::SelectionMode _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 26, _a);
}

// SIGNAL 27
void QQuickTextEdit::linkActivated(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 27, _a);
}

// SIGNAL 28
void QQuickTextEdit::linkHovered(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 28, _a);
}

// SIGNAL 29
void QQuickTextEdit::canPasteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 29, nullptr);
}

// SIGNAL 30
void QQuickTextEdit::canUndoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 30, nullptr);
}

// SIGNAL 31
void QQuickTextEdit::canRedoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 31, nullptr);
}

// SIGNAL 32
void QQuickTextEdit::inputMethodComposingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 32, nullptr);
}

// SIGNAL 33
void QQuickTextEdit::effectiveHorizontalAlignmentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 33, nullptr);
}

// SIGNAL 34
void QQuickTextEdit::baseUrlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 34, nullptr);
}

// SIGNAL 35
void QQuickTextEdit::inputMethodHintsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 35, nullptr);
}

// SIGNAL 36
void QQuickTextEdit::renderTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 36, nullptr);
}

// SIGNAL 37
void QQuickTextEdit::editingFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 37, nullptr);
}

// SIGNAL 38
void QQuickTextEdit::paddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 38, nullptr);
}

// SIGNAL 39
void QQuickTextEdit::topPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 39, nullptr);
}

// SIGNAL 40
void QQuickTextEdit::leftPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 40, nullptr);
}

// SIGNAL 41
void QQuickTextEdit::rightPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 41, nullptr);
}

// SIGNAL 42
void QQuickTextEdit::bottomPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 42, nullptr);
}

// SIGNAL 43
void QQuickTextEdit::tabStopDistanceChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 43, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
