/****************************************************************************
** Meta object code from reading C++ file 'qquickdrag_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquickdrag_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickdrag_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickDragMimeData_t {
    QByteArrayData data[1];
    char stringdata0[19];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickDragMimeData_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickDragMimeData_t qt_meta_stringdata_QQuickDragMimeData = {
    {
QT_MOC_LITERAL(0, 0, 18) // "QQuickDragMimeData"

    },
    "QQuickDragMimeData"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickDragMimeData[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QQuickDragMimeData::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickDragMimeData::staticMetaObject = { {
    QMetaObject::SuperData::link<QMimeData::staticMetaObject>(),
    qt_meta_stringdata_QQuickDragMimeData.data,
    qt_meta_data_QQuickDragMimeData,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickDragMimeData::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickDragMimeData::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickDragMimeData.stringdata0))
        return static_cast<void*>(this);
    return QMimeData::qt_metacast(_clname);
}

int QQuickDragMimeData::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMimeData::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQuickDrag_t {
    QByteArrayData data[40];
    char stringdata0[481];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickDrag_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickDrag_t qt_meta_stringdata_QQuickDrag = {
    {
QT_MOC_LITERAL(0, 0, 10), // "QQuickDrag"
QT_MOC_LITERAL(1, 11, 11), // "QML.Element"
QT_MOC_LITERAL(2, 23, 4), // "Drag"
QT_MOC_LITERAL(3, 28, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 42, 5), // "false"
QT_MOC_LITERAL(5, 48, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 70, 47), // "Drag is only available via at..."
QT_MOC_LITERAL(7, 118, 12), // "QML.Attached"
QT_MOC_LITERAL(8, 131, 18), // "QQuickDragAttached"
QT_MOC_LITERAL(9, 150, 13), // "targetChanged"
QT_MOC_LITERAL(10, 164, 0), // ""
QT_MOC_LITERAL(11, 165, 11), // "axisChanged"
QT_MOC_LITERAL(12, 177, 15), // "minimumXChanged"
QT_MOC_LITERAL(13, 193, 15), // "maximumXChanged"
QT_MOC_LITERAL(14, 209, 15), // "minimumYChanged"
QT_MOC_LITERAL(15, 225, 15), // "maximumYChanged"
QT_MOC_LITERAL(16, 241, 13), // "activeChanged"
QT_MOC_LITERAL(17, 255, 21), // "filterChildrenChanged"
QT_MOC_LITERAL(18, 277, 15), // "smoothedChanged"
QT_MOC_LITERAL(19, 293, 16), // "thresholdChanged"
QT_MOC_LITERAL(20, 310, 6), // "target"
QT_MOC_LITERAL(21, 317, 11), // "QQuickItem*"
QT_MOC_LITERAL(22, 329, 4), // "axis"
QT_MOC_LITERAL(23, 334, 4), // "Axis"
QT_MOC_LITERAL(24, 339, 8), // "minimumX"
QT_MOC_LITERAL(25, 348, 8), // "maximumX"
QT_MOC_LITERAL(26, 357, 8), // "minimumY"
QT_MOC_LITERAL(27, 366, 8), // "maximumY"
QT_MOC_LITERAL(28, 375, 6), // "active"
QT_MOC_LITERAL(29, 382, 14), // "filterChildren"
QT_MOC_LITERAL(30, 397, 8), // "smoothed"
QT_MOC_LITERAL(31, 406, 9), // "threshold"
QT_MOC_LITERAL(32, 416, 8), // "DragType"
QT_MOC_LITERAL(33, 425, 4), // "None"
QT_MOC_LITERAL(34, 430, 9), // "Automatic"
QT_MOC_LITERAL(35, 440, 8), // "Internal"
QT_MOC_LITERAL(36, 449, 5), // "XAxis"
QT_MOC_LITERAL(37, 455, 5), // "YAxis"
QT_MOC_LITERAL(38, 461, 9), // "XAndYAxis"
QT_MOC_LITERAL(39, 471, 9) // "XandYAxis"

    },
    "QQuickDrag\0QML.Element\0Drag\0QML.Creatable\0"
    "false\0QML.UncreatableReason\0"
    "Drag is only available via attached properties.\0"
    "QML.Attached\0QQuickDragAttached\0"
    "targetChanged\0\0axisChanged\0minimumXChanged\0"
    "maximumXChanged\0minimumYChanged\0"
    "maximumYChanged\0activeChanged\0"
    "filterChildrenChanged\0smoothedChanged\0"
    "thresholdChanged\0target\0QQuickItem*\0"
    "axis\0Axis\0minimumX\0maximumX\0minimumY\0"
    "maximumY\0active\0filterChildren\0smoothed\0"
    "threshold\0DragType\0None\0Automatic\0"
    "Internal\0XAxis\0YAxis\0XAndYAxis\0XandYAxis"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickDrag[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
      10,   22, // methods
      10,   82, // properties
       2,  122, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

 // signals: name, argc, parameters, tag, flags
       9,    0,   72,   10, 0x06 /* Public */,
      11,    0,   73,   10, 0x06 /* Public */,
      12,    0,   74,   10, 0x06 /* Public */,
      13,    0,   75,   10, 0x06 /* Public */,
      14,    0,   76,   10, 0x06 /* Public */,
      15,    0,   77,   10, 0x06 /* Public */,
      16,    0,   78,   10, 0x06 /* Public */,
      17,    0,   79,   10, 0x06 /* Public */,
      18,    0,   80,   10, 0x06 /* Public */,
      19,    0,   81,   10, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      20, 0x80000000 | 21, 0x0049510f,
      22, 0x80000000 | 23, 0x0049510b,
      24, QMetaType::QReal, 0x00495003,
      25, QMetaType::QReal, 0x00495003,
      26, QMetaType::QReal, 0x00495003,
      27, QMetaType::QReal, 0x00495003,
      28, QMetaType::Bool, 0x00495001,
      29, QMetaType::Bool, 0x00495103,
      30, QMetaType::Bool, 0x00495103,
      31, QMetaType::QReal, 0x00495107,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,
       9,

 // enums: name, alias, flags, count, data
      32,   32, 0x0,    3,  132,
      23,   23, 0x0,    4,  138,

 // enum data: key, value
      33, uint(QQuickDrag::None),
      34, uint(QQuickDrag::Automatic),
      35, uint(QQuickDrag::Internal),
      36, uint(QQuickDrag::XAxis),
      37, uint(QQuickDrag::YAxis),
      38, uint(QQuickDrag::XAndYAxis),
      39, uint(QQuickDrag::XandYAxis),

       0        // eod
};

void QQuickDrag::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickDrag *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->targetChanged(); break;
        case 1: _t->axisChanged(); break;
        case 2: _t->minimumXChanged(); break;
        case 3: _t->maximumXChanged(); break;
        case 4: _t->minimumYChanged(); break;
        case 5: _t->maximumYChanged(); break;
        case 6: _t->activeChanged(); break;
        case 7: _t->filterChildrenChanged(); break;
        case 8: _t->smoothedChanged(); break;
        case 9: _t->thresholdChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickDrag::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDrag::targetChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickDrag::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDrag::axisChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickDrag::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDrag::minimumXChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickDrag::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDrag::maximumXChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickDrag::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDrag::minimumYChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickDrag::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDrag::maximumYChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickDrag::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDrag::activeChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickDrag::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDrag::filterChildrenChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickDrag::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDrag::smoothedChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickDrag::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDrag::thresholdChanged)) {
                *result = 9;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickDrag *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQuickItem**>(_v) = _t->target(); break;
        case 1: *reinterpret_cast< Axis*>(_v) = _t->axis(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->xmin(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->xmax(); break;
        case 4: *reinterpret_cast< qreal*>(_v) = _t->ymin(); break;
        case 5: *reinterpret_cast< qreal*>(_v) = _t->ymax(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->active(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->filterChildren(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->smoothed(); break;
        case 9: *reinterpret_cast< qreal*>(_v) = _t->threshold(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickDrag *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTarget(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 1: _t->setAxis(*reinterpret_cast< Axis*>(_v)); break;
        case 2: _t->setXmin(*reinterpret_cast< qreal*>(_v)); break;
        case 3: _t->setXmax(*reinterpret_cast< qreal*>(_v)); break;
        case 4: _t->setYmin(*reinterpret_cast< qreal*>(_v)); break;
        case 5: _t->setYmax(*reinterpret_cast< qreal*>(_v)); break;
        case 7: _t->setFilterChildren(*reinterpret_cast< bool*>(_v)); break;
        case 8: _t->setSmoothed(*reinterpret_cast< bool*>(_v)); break;
        case 9: _t->setThreshold(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickDrag *_t = static_cast<QQuickDrag *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->resetTarget(); break;
        case 9: _t->resetThreshold(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickDrag::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickDrag.data,
    qt_meta_data_QQuickDrag,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickDrag::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickDrag::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickDrag.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickDrag::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 10;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickDrag::targetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickDrag::axisChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickDrag::minimumXChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickDrag::maximumXChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickDrag::minimumYChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickDrag::maximumYChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickDrag::activeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickDrag::filterChildrenChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickDrag::smoothedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickDrag::thresholdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
struct qt_meta_stringdata_QQuickDragAttached_t {
    QByteArrayData data[35];
    char stringdata0[436];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickDragAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickDragAttached_t qt_meta_stringdata_QQuickDragAttached = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QQuickDragAttached"
QT_MOC_LITERAL(1, 19, 11), // "QML.Element"
QT_MOC_LITERAL(2, 31, 9), // "anonymous"
QT_MOC_LITERAL(3, 41, 11), // "dragStarted"
QT_MOC_LITERAL(4, 53, 0), // ""
QT_MOC_LITERAL(5, 54, 12), // "dragFinished"
QT_MOC_LITERAL(6, 67, 14), // "Qt::DropAction"
QT_MOC_LITERAL(7, 82, 10), // "dropAction"
QT_MOC_LITERAL(8, 93, 13), // "activeChanged"
QT_MOC_LITERAL(9, 107, 13), // "sourceChanged"
QT_MOC_LITERAL(10, 121, 13), // "targetChanged"
QT_MOC_LITERAL(11, 135, 14), // "hotSpotChanged"
QT_MOC_LITERAL(12, 150, 18), // "imageSourceChanged"
QT_MOC_LITERAL(13, 169, 11), // "keysChanged"
QT_MOC_LITERAL(14, 181, 15), // "mimeDataChanged"
QT_MOC_LITERAL(15, 197, 23), // "supportedActionsChanged"
QT_MOC_LITERAL(16, 221, 21), // "proposedActionChanged"
QT_MOC_LITERAL(17, 243, 15), // "dragTypeChanged"
QT_MOC_LITERAL(18, 259, 5), // "start"
QT_MOC_LITERAL(19, 265, 15), // "QQmlV4Function*"
QT_MOC_LITERAL(20, 281, 9), // "startDrag"
QT_MOC_LITERAL(21, 291, 6), // "cancel"
QT_MOC_LITERAL(22, 298, 4), // "drop"
QT_MOC_LITERAL(23, 303, 6), // "active"
QT_MOC_LITERAL(24, 310, 6), // "source"
QT_MOC_LITERAL(25, 317, 6), // "target"
QT_MOC_LITERAL(26, 324, 7), // "hotSpot"
QT_MOC_LITERAL(27, 332, 11), // "imageSource"
QT_MOC_LITERAL(28, 344, 4), // "keys"
QT_MOC_LITERAL(29, 349, 8), // "mimeData"
QT_MOC_LITERAL(30, 358, 16), // "supportedActions"
QT_MOC_LITERAL(31, 375, 15), // "Qt::DropActions"
QT_MOC_LITERAL(32, 391, 14), // "proposedAction"
QT_MOC_LITERAL(33, 406, 8), // "dragType"
QT_MOC_LITERAL(34, 415, 20) // "QQuickDrag::DragType"

    },
    "QQuickDragAttached\0QML.Element\0anonymous\0"
    "dragStarted\0\0dragFinished\0Qt::DropAction\0"
    "dropAction\0activeChanged\0sourceChanged\0"
    "targetChanged\0hotSpotChanged\0"
    "imageSourceChanged\0keysChanged\0"
    "mimeDataChanged\0supportedActionsChanged\0"
    "proposedActionChanged\0dragTypeChanged\0"
    "start\0QQmlV4Function*\0startDrag\0cancel\0"
    "drop\0active\0source\0target\0hotSpot\0"
    "imageSource\0keys\0mimeData\0supportedActions\0"
    "Qt::DropActions\0proposedAction\0dragType\0"
    "QQuickDrag::DragType"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickDragAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      16,   16, // methods
      10,  118, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      12,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   96,    4, 0x06 /* Public */,
       5,    1,   97,    4, 0x06 /* Public */,
       8,    0,  100,    4, 0x06 /* Public */,
       9,    0,  101,    4, 0x06 /* Public */,
      10,    0,  102,    4, 0x06 /* Public */,
      11,    0,  103,    4, 0x06 /* Public */,
      12,    0,  104,    4, 0x06 /* Public */,
      13,    0,  105,    4, 0x06 /* Public */,
      14,    0,  106,    4, 0x06 /* Public */,
      15,    0,  107,    4, 0x06 /* Public */,
      16,    0,  108,    4, 0x06 /* Public */,
      17,    0,  109,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      18,    1,  110,    4, 0x0a /* Public */,
      20,    1,  113,    4, 0x0a /* Public */,
      21,    0,  116,    4, 0x0a /* Public */,

 // methods: name, argc, parameters, tag, flags
      22,    0,  117,    4, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 19,    4,
    QMetaType::Void, 0x80000000 | 19,    4,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Int,

 // properties: name, type, flags
      23, QMetaType::Bool, 0x00495103,
      24, QMetaType::QObjectStar, 0x00495107,
      25, QMetaType::QObjectStar, 0x00495001,
      26, QMetaType::QPointF, 0x00495103,
      27, QMetaType::QUrl, 0x00495103,
      28, QMetaType::QStringList, 0x00495103,
      29, QMetaType::QVariantMap, 0x00495103,
      30, 0x80000000 | 31, 0x0049510b,
      32, 0x80000000 | 6, 0x0049510b,
      33, 0x80000000 | 34, 0x0049510b,

 // properties: notify_signal_id
       2,
       3,
       4,
       5,
       6,
       7,
       8,
       9,
      10,
      11,

       0        // eod
};

void QQuickDragAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickDragAttached *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->dragStarted(); break;
        case 1: _t->dragFinished((*reinterpret_cast< Qt::DropAction(*)>(_a[1]))); break;
        case 2: _t->activeChanged(); break;
        case 3: _t->sourceChanged(); break;
        case 4: _t->targetChanged(); break;
        case 5: _t->hotSpotChanged(); break;
        case 6: _t->imageSourceChanged(); break;
        case 7: _t->keysChanged(); break;
        case 8: _t->mimeDataChanged(); break;
        case 9: _t->supportedActionsChanged(); break;
        case 10: _t->proposedActionChanged(); break;
        case 11: _t->dragTypeChanged(); break;
        case 12: _t->start((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 13: _t->startDrag((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 14: _t->cancel(); break;
        case 15: { int _r = _t->drop();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickDragAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDragAttached::dragStarted)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickDragAttached::*)(Qt::DropAction );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDragAttached::dragFinished)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickDragAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDragAttached::activeChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickDragAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDragAttached::sourceChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickDragAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDragAttached::targetChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickDragAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDragAttached::hotSpotChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickDragAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDragAttached::imageSourceChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickDragAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDragAttached::keysChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickDragAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDragAttached::mimeDataChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickDragAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDragAttached::supportedActionsChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickDragAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDragAttached::proposedActionChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickDragAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDragAttached::dragTypeChanged)) {
                *result = 11;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickDragAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->isActive(); break;
        case 1: *reinterpret_cast< QObject**>(_v) = _t->source(); break;
        case 2: *reinterpret_cast< QObject**>(_v) = _t->target(); break;
        case 3: *reinterpret_cast< QPointF*>(_v) = _t->hotSpot(); break;
        case 4: *reinterpret_cast< QUrl*>(_v) = _t->imageSource(); break;
        case 5: *reinterpret_cast< QStringList*>(_v) = _t->keys(); break;
        case 6: *reinterpret_cast< QVariantMap*>(_v) = _t->mimeData(); break;
        case 7: *reinterpret_cast< Qt::DropActions*>(_v) = _t->supportedActions(); break;
        case 8: *reinterpret_cast< Qt::DropAction*>(_v) = _t->proposedAction(); break;
        case 9: *reinterpret_cast< QQuickDrag::DragType*>(_v) = _t->dragType(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickDragAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setActive(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setSource(*reinterpret_cast< QObject**>(_v)); break;
        case 3: _t->setHotSpot(*reinterpret_cast< QPointF*>(_v)); break;
        case 4: _t->setImageSource(*reinterpret_cast< QUrl*>(_v)); break;
        case 5: _t->setKeys(*reinterpret_cast< QStringList*>(_v)); break;
        case 6: _t->setMimeData(*reinterpret_cast< QVariantMap*>(_v)); break;
        case 7: _t->setSupportedActions(*reinterpret_cast< Qt::DropActions*>(_v)); break;
        case 8: _t->setProposedAction(*reinterpret_cast< Qt::DropAction*>(_v)); break;
        case 9: _t->setDragType(*reinterpret_cast< QQuickDrag::DragType*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickDragAttached *_t = static_cast<QQuickDragAttached *>(_o);
        (void)_t;
        switch (_id) {
        case 1: _t->resetSource(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

static const QMetaObject::SuperData qt_meta_extradata_QQuickDragAttached[] = {
    QMetaObject::SuperData::link<QQuickDrag::staticMetaObject>(),
    nullptr
};

QT_INIT_METAOBJECT const QMetaObject QQuickDragAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickDragAttached.data,
    qt_meta_data_QQuickDragAttached,
    qt_static_metacall,
    qt_meta_extradata_QQuickDragAttached,
    nullptr
} };


const QMetaObject *QQuickDragAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickDragAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickDragAttached.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickDragAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 16;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 10;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickDragAttached::dragStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickDragAttached::dragFinished(Qt::DropAction _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickDragAttached::activeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickDragAttached::sourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickDragAttached::targetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickDragAttached::hotSpotChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickDragAttached::imageSourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickDragAttached::keysChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickDragAttached::mimeDataChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickDragAttached::supportedActionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QQuickDragAttached::proposedActionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QQuickDragAttached::dragTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
