/****************************************************************************
** Meta object code from reading C++ file 'qquickcustomaffector_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/particles/qquickcustomaffector_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickcustomaffector_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickCustomAffector_t {
    QByteArrayData data[22];
    char stringdata0[262];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickCustomAffector_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickCustomAffector_t qt_meta_stringdata_QQuickCustomAffector = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QQuickCustomAffector"
QT_MOC_LITERAL(1, 21, 11), // "QML.Element"
QT_MOC_LITERAL(2, 33, 8), // "Affector"
QT_MOC_LITERAL(3, 42, 15), // "affectParticles"
QT_MOC_LITERAL(4, 58, 0), // ""
QT_MOC_LITERAL(5, 59, 8), // "QJSValue"
QT_MOC_LITERAL(6, 68, 9), // "particles"
QT_MOC_LITERAL(7, 78, 2), // "dt"
QT_MOC_LITERAL(8, 81, 15), // "positionChanged"
QT_MOC_LITERAL(9, 97, 16), // "QQuickDirection*"
QT_MOC_LITERAL(10, 114, 3), // "arg"
QT_MOC_LITERAL(11, 118, 15), // "velocityChanged"
QT_MOC_LITERAL(12, 134, 19), // "accelerationChanged"
QT_MOC_LITERAL(13, 154, 15), // "relativeChanged"
QT_MOC_LITERAL(14, 170, 11), // "setPosition"
QT_MOC_LITERAL(15, 182, 11), // "setVelocity"
QT_MOC_LITERAL(16, 194, 15), // "setAcceleration"
QT_MOC_LITERAL(17, 210, 11), // "setRelative"
QT_MOC_LITERAL(18, 222, 8), // "relative"
QT_MOC_LITERAL(19, 231, 8), // "position"
QT_MOC_LITERAL(20, 240, 8), // "velocity"
QT_MOC_LITERAL(21, 249, 12) // "acceleration"

    },
    "QQuickCustomAffector\0QML.Element\0"
    "Affector\0affectParticles\0\0QJSValue\0"
    "particles\0dt\0positionChanged\0"
    "QQuickDirection*\0arg\0velocityChanged\0"
    "accelerationChanged\0relativeChanged\0"
    "setPosition\0setVelocity\0setAcceleration\0"
    "setRelative\0relative\0position\0velocity\0"
    "acceleration"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickCustomAffector[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       9,   16, // methods
       4,   90, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    2,   61,    4, 0x06 /* Public */,
       8,    1,   66,    4, 0x06 /* Public */,
      11,    1,   69,    4, 0x06 /* Public */,
      12,    1,   72,    4, 0x06 /* Public */,
      13,    1,   75,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      14,    1,   78,    4, 0x0a /* Public */,
      15,    1,   81,    4, 0x0a /* Public */,
      16,    1,   84,    4, 0x0a /* Public */,
      17,    1,   87,    4, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 5, QMetaType::QReal,    6,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, QMetaType::Bool,   10,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, QMetaType::Bool,   10,

 // properties: name, type, flags
      18, QMetaType::Bool, 0x00495103,
      19, 0x80000000 | 9, 0x0049510f,
      20, 0x80000000 | 9, 0x0049510f,
      21, 0x80000000 | 9, 0x0049510f,

 // properties: notify_signal_id
       4,
       1,
       2,
       3,

       0        // eod
};

void QQuickCustomAffector::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickCustomAffector *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->affectParticles((*reinterpret_cast< const QJSValue(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2]))); break;
        case 1: _t->positionChanged((*reinterpret_cast< QQuickDirection*(*)>(_a[1]))); break;
        case 2: _t->velocityChanged((*reinterpret_cast< QQuickDirection*(*)>(_a[1]))); break;
        case 3: _t->accelerationChanged((*reinterpret_cast< QQuickDirection*(*)>(_a[1]))); break;
        case 4: _t->relativeChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->setPosition((*reinterpret_cast< QQuickDirection*(*)>(_a[1]))); break;
        case 6: _t->setVelocity((*reinterpret_cast< QQuickDirection*(*)>(_a[1]))); break;
        case 7: _t->setAcceleration((*reinterpret_cast< QQuickDirection*(*)>(_a[1]))); break;
        case 8: _t->setRelative((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDirection* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDirection* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDirection* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDirection* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDirection* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDirection* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickCustomAffector::*)(const QJSValue & , qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickCustomAffector::affectParticles)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickCustomAffector::*)(QQuickDirection * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickCustomAffector::positionChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickCustomAffector::*)(QQuickDirection * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickCustomAffector::velocityChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickCustomAffector::*)(QQuickDirection * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickCustomAffector::accelerationChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickCustomAffector::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickCustomAffector::relativeChanged)) {
                *result = 4;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
        case 2:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDirection* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickCustomAffector *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->relative(); break;
        case 1: *reinterpret_cast< QQuickDirection**>(_v) = _t->position(); break;
        case 2: *reinterpret_cast< QQuickDirection**>(_v) = _t->velocity(); break;
        case 3: *reinterpret_cast< QQuickDirection**>(_v) = _t->acceleration(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickCustomAffector *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRelative(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setPosition(*reinterpret_cast< QQuickDirection**>(_v)); break;
        case 2: _t->setVelocity(*reinterpret_cast< QQuickDirection**>(_v)); break;
        case 3: _t->setAcceleration(*reinterpret_cast< QQuickDirection**>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickCustomAffector *_t = static_cast<QQuickCustomAffector *>(_o);
        (void)_t;
        switch (_id) {
        case 1: _t->positionReset(); break;
        case 2: _t->velocityReset(); break;
        case 3: _t->accelerationReset(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickCustomAffector::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickParticleAffector::staticMetaObject>(),
    qt_meta_stringdata_QQuickCustomAffector.data,
    qt_meta_data_QQuickCustomAffector,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickCustomAffector::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickCustomAffector::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickCustomAffector.stringdata0))
        return static_cast<void*>(this);
    return QQuickParticleAffector::qt_metacast(_clname);
}

int QQuickCustomAffector::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickParticleAffector::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickCustomAffector::affectParticles(const QJSValue & _t1, qreal _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickCustomAffector::positionChanged(QQuickDirection * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickCustomAffector::velocityChanged(QQuickDirection * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickCustomAffector::accelerationChanged(QQuickDirection * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickCustomAffector::relativeChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
