/****************************************************************************
** Meta object code from reading C++ file 'qquickanimator_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/util/qquickanimator_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickanimator_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickAnimator_t {
    QByteArrayData data[23];
    char stringdata0[267];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickAnimator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickAnimator_t qt_meta_stringdata_QQuickAnimator = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QQuickAnimator"
QT_MOC_LITERAL(1, 15, 11), // "QML.Element"
QT_MOC_LITERAL(2, 27, 8), // "Animator"
QT_MOC_LITERAL(3, 36, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 60, 1), // "2"
QT_MOC_LITERAL(5, 62, 13), // "QML.Creatable"
QT_MOC_LITERAL(6, 76, 5), // "false"
QT_MOC_LITERAL(7, 82, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(8, 104, 29), // "Animator is an abstract class"
QT_MOC_LITERAL(9, 134, 17), // "targetItemChanged"
QT_MOC_LITERAL(10, 152, 0), // ""
QT_MOC_LITERAL(11, 153, 11), // "QQuickItem*"
QT_MOC_LITERAL(12, 165, 15), // "durationChanged"
QT_MOC_LITERAL(13, 181, 8), // "duration"
QT_MOC_LITERAL(14, 190, 13), // "easingChanged"
QT_MOC_LITERAL(15, 204, 12), // "QEasingCurve"
QT_MOC_LITERAL(16, 217, 5), // "curve"
QT_MOC_LITERAL(17, 223, 9), // "toChanged"
QT_MOC_LITERAL(18, 233, 2), // "to"
QT_MOC_LITERAL(19, 236, 11), // "fromChanged"
QT_MOC_LITERAL(20, 248, 4), // "from"
QT_MOC_LITERAL(21, 253, 6), // "target"
QT_MOC_LITERAL(22, 260, 6) // "easing"

    },
    "QQuickAnimator\0QML.Element\0Animator\0"
    "QML.AddedInMinorVersion\0""2\0QML.Creatable\0"
    "false\0QML.UncreatableReason\0"
    "Animator is an abstract class\0"
    "targetItemChanged\0\0QQuickItem*\0"
    "durationChanged\0duration\0easingChanged\0"
    "QEasingCurve\0curve\0toChanged\0to\0"
    "fromChanged\0from\0target\0easing"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickAnimator[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       5,   22, // methods
       5,   62, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

 // signals: name, argc, parameters, tag, flags
       9,    1,   47,   10, 0x06 /* Public */,
      12,    1,   50,   10, 0x06 /* Public */,
      14,    1,   53,   10, 0x06 /* Public */,
      17,    1,   56,   10, 0x06 /* Public */,
      19,    1,   59,   10, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 11,   10,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void, QMetaType::QReal,   18,
    QMetaType::Void, QMetaType::QReal,   20,

 // properties: name, type, flags
      21, 0x80000000 | 11, 0x0049500b,
      22, 0x80000000 | 15, 0x0049510b,
      13, QMetaType::Int, 0x00495103,
      18, QMetaType::QReal, 0x00495103,
      20, QMetaType::QReal, 0x00495103,

 // properties: notify_signal_id
       0,
       2,
       1,
       3,
       4,

       0        // eod
};

void QQuickAnimator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickAnimator *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->targetItemChanged((*reinterpret_cast< QQuickItem*(*)>(_a[1]))); break;
        case 1: _t->durationChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->easingChanged((*reinterpret_cast< const QEasingCurve(*)>(_a[1]))); break;
        case 3: _t->toChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 4: _t->fromChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickAnimator::*)(QQuickItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimator::targetItemChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickAnimator::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimator::durationChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickAnimator::*)(const QEasingCurve & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimator::easingChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickAnimator::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimator::toChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickAnimator::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAnimator::fromChanged)) {
                *result = 4;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickAnimator *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQuickItem**>(_v) = _t->targetItem(); break;
        case 1: *reinterpret_cast< QEasingCurve*>(_v) = _t->easing(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->duration(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->to(); break;
        case 4: *reinterpret_cast< qreal*>(_v) = _t->from(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickAnimator *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTargetItem(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 1: _t->setEasing(*reinterpret_cast< QEasingCurve*>(_v)); break;
        case 2: _t->setDuration(*reinterpret_cast< int*>(_v)); break;
        case 3: _t->setTo(*reinterpret_cast< qreal*>(_v)); break;
        case 4: _t->setFrom(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickAnimator::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickAbstractAnimation::staticMetaObject>(),
    qt_meta_stringdata_QQuickAnimator.data,
    qt_meta_data_QQuickAnimator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickAnimator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickAnimator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickAnimator.stringdata0))
        return static_cast<void*>(this);
    return QQuickAbstractAnimation::qt_metacast(_clname);
}

int QQuickAnimator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickAbstractAnimation::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickAnimator::targetItemChanged(QQuickItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickAnimator::durationChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickAnimator::easingChanged(const QEasingCurve & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickAnimator::toChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickAnimator::fromChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
struct qt_meta_stringdata_QQuickScaleAnimator_t {
    QByteArrayData data[5];
    char stringdata0[72];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickScaleAnimator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickScaleAnimator_t qt_meta_stringdata_QQuickScaleAnimator = {
    {
QT_MOC_LITERAL(0, 0, 19), // "QQuickScaleAnimator"
QT_MOC_LITERAL(1, 20, 11), // "QML.Element"
QT_MOC_LITERAL(2, 32, 13), // "ScaleAnimator"
QT_MOC_LITERAL(3, 46, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 70, 1) // "2"

    },
    "QQuickScaleAnimator\0QML.Element\0"
    "ScaleAnimator\0QML.AddedInMinorVersion\0"
    "2"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickScaleAnimator[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

       0        // eod
};

void QQuickScaleAnimator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickScaleAnimator::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickAnimator::staticMetaObject>(),
    qt_meta_stringdata_QQuickScaleAnimator.data,
    qt_meta_data_QQuickScaleAnimator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickScaleAnimator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickScaleAnimator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickScaleAnimator.stringdata0))
        return static_cast<void*>(this);
    return QQuickAnimator::qt_metacast(_clname);
}

int QQuickScaleAnimator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickAnimator::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQuickXAnimator_t {
    QByteArrayData data[5];
    char stringdata0[64];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickXAnimator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickXAnimator_t qt_meta_stringdata_QQuickXAnimator = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QQuickXAnimator"
QT_MOC_LITERAL(1, 16, 11), // "QML.Element"
QT_MOC_LITERAL(2, 28, 9), // "XAnimator"
QT_MOC_LITERAL(3, 38, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 62, 1) // "2"

    },
    "QQuickXAnimator\0QML.Element\0XAnimator\0"
    "QML.AddedInMinorVersion\0""2"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickXAnimator[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

       0        // eod
};

void QQuickXAnimator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickXAnimator::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickAnimator::staticMetaObject>(),
    qt_meta_stringdata_QQuickXAnimator.data,
    qt_meta_data_QQuickXAnimator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickXAnimator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickXAnimator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickXAnimator.stringdata0))
        return static_cast<void*>(this);
    return QQuickAnimator::qt_metacast(_clname);
}

int QQuickXAnimator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickAnimator::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQuickYAnimator_t {
    QByteArrayData data[5];
    char stringdata0[64];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickYAnimator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickYAnimator_t qt_meta_stringdata_QQuickYAnimator = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QQuickYAnimator"
QT_MOC_LITERAL(1, 16, 11), // "QML.Element"
QT_MOC_LITERAL(2, 28, 9), // "YAnimator"
QT_MOC_LITERAL(3, 38, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 62, 1) // "2"

    },
    "QQuickYAnimator\0QML.Element\0YAnimator\0"
    "QML.AddedInMinorVersion\0""2"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickYAnimator[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

       0        // eod
};

void QQuickYAnimator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickYAnimator::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickAnimator::staticMetaObject>(),
    qt_meta_stringdata_QQuickYAnimator.data,
    qt_meta_data_QQuickYAnimator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickYAnimator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickYAnimator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickYAnimator.stringdata0))
        return static_cast<void*>(this);
    return QQuickAnimator::qt_metacast(_clname);
}

int QQuickYAnimator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickAnimator::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQuickOpacityAnimator_t {
    QByteArrayData data[5];
    char stringdata0[76];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickOpacityAnimator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickOpacityAnimator_t qt_meta_stringdata_QQuickOpacityAnimator = {
    {
QT_MOC_LITERAL(0, 0, 21), // "QQuickOpacityAnimator"
QT_MOC_LITERAL(1, 22, 11), // "QML.Element"
QT_MOC_LITERAL(2, 34, 15), // "OpacityAnimator"
QT_MOC_LITERAL(3, 50, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 74, 1) // "2"

    },
    "QQuickOpacityAnimator\0QML.Element\0"
    "OpacityAnimator\0QML.AddedInMinorVersion\0"
    "2"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickOpacityAnimator[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

       0        // eod
};

void QQuickOpacityAnimator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickOpacityAnimator::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickAnimator::staticMetaObject>(),
    qt_meta_stringdata_QQuickOpacityAnimator.data,
    qt_meta_data_QQuickOpacityAnimator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickOpacityAnimator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickOpacityAnimator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickOpacityAnimator.stringdata0))
        return static_cast<void*>(this);
    return QQuickAnimator::qt_metacast(_clname);
}

int QQuickOpacityAnimator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickAnimator::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQuickRotationAnimator_t {
    QByteArrayData data[14];
    char stringdata0[174];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickRotationAnimator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickRotationAnimator_t qt_meta_stringdata_QQuickRotationAnimator = {
    {
QT_MOC_LITERAL(0, 0, 22), // "QQuickRotationAnimator"
QT_MOC_LITERAL(1, 23, 11), // "QML.Element"
QT_MOC_LITERAL(2, 35, 16), // "RotationAnimator"
QT_MOC_LITERAL(3, 52, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 76, 1), // "2"
QT_MOC_LITERAL(5, 78, 16), // "directionChanged"
QT_MOC_LITERAL(6, 95, 0), // ""
QT_MOC_LITERAL(7, 96, 17), // "RotationDirection"
QT_MOC_LITERAL(8, 114, 3), // "dir"
QT_MOC_LITERAL(9, 118, 9), // "direction"
QT_MOC_LITERAL(10, 128, 9), // "Numerical"
QT_MOC_LITERAL(11, 138, 8), // "Shortest"
QT_MOC_LITERAL(12, 147, 9), // "Clockwise"
QT_MOC_LITERAL(13, 157, 16) // "Counterclockwise"

    },
    "QQuickRotationAnimator\0QML.Element\0"
    "RotationAnimator\0QML.AddedInMinorVersion\0"
    "2\0directionChanged\0\0RotationDirection\0"
    "dir\0direction\0Numerical\0Shortest\0"
    "Clockwise\0Counterclockwise"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickRotationAnimator[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       1,   18, // methods
       1,   26, // properties
       1,   30, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    1,   23,    6, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 7,    8,

 // properties: name, type, flags
       9, 0x80000000 | 7, 0x0049510b,

 // properties: notify_signal_id
       0,

 // enums: name, alias, flags, count, data
       7,    7, 0x0,    4,   35,

 // enum data: key, value
      10, uint(QQuickRotationAnimator::Numerical),
      11, uint(QQuickRotationAnimator::Shortest),
      12, uint(QQuickRotationAnimator::Clockwise),
      13, uint(QQuickRotationAnimator::Counterclockwise),

       0        // eod
};

void QQuickRotationAnimator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickRotationAnimator *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->directionChanged((*reinterpret_cast< RotationDirection(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickRotationAnimator::*)(RotationDirection );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickRotationAnimator::directionChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickRotationAnimator *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< RotationDirection*>(_v) = _t->direction(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickRotationAnimator *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDirection(*reinterpret_cast< RotationDirection*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickRotationAnimator::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickAnimator::staticMetaObject>(),
    qt_meta_stringdata_QQuickRotationAnimator.data,
    qt_meta_data_QQuickRotationAnimator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickRotationAnimator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickRotationAnimator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickRotationAnimator.stringdata0))
        return static_cast<void*>(this);
    return QQuickAnimator::qt_metacast(_clname);
}

int QQuickRotationAnimator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickAnimator::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickRotationAnimator::directionChanged(RotationDirection _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_QQuickUniformAnimator_t {
    QByteArrayData data[8];
    char stringdata0[100];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickUniformAnimator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickUniformAnimator_t qt_meta_stringdata_QQuickUniformAnimator = {
    {
QT_MOC_LITERAL(0, 0, 21), // "QQuickUniformAnimator"
QT_MOC_LITERAL(1, 22, 11), // "QML.Element"
QT_MOC_LITERAL(2, 34, 15), // "UniformAnimator"
QT_MOC_LITERAL(3, 50, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 74, 1), // "2"
QT_MOC_LITERAL(5, 76, 14), // "uniformChanged"
QT_MOC_LITERAL(6, 91, 0), // ""
QT_MOC_LITERAL(7, 92, 7) // "uniform"

    },
    "QQuickUniformAnimator\0QML.Element\0"
    "UniformAnimator\0QML.AddedInMinorVersion\0"
    "2\0uniformChanged\0\0uniform"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickUniformAnimator[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       1,   18, // methods
       1,   26, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    1,   23,    6, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    6,

 // properties: name, type, flags
       7, QMetaType::QString, 0x00495103,

 // properties: notify_signal_id
       0,

       0        // eod
};

void QQuickUniformAnimator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickUniformAnimator *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->uniformChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickUniformAnimator::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickUniformAnimator::uniformChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickUniformAnimator *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->uniform(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickUniformAnimator *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setUniform(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickUniformAnimator::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickAnimator::staticMetaObject>(),
    qt_meta_stringdata_QQuickUniformAnimator.data,
    qt_meta_data_QQuickUniformAnimator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickUniformAnimator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickUniformAnimator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickUniformAnimator.stringdata0))
        return static_cast<void*>(this);
    return QQuickAnimator::qt_metacast(_clname);
}

int QQuickUniformAnimator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickAnimator::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickUniformAnimator::uniformChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
