/****************************************************************************
** Meta object code from reading C++ file 'qqmlbind_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/qml/types/qqmlbind_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qqmlbind_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQmlBind_t {
    QByteArrayData data[18];
    char stringdata0[202];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlBind_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlBind_t qt_meta_stringdata_QQmlBind = {
    {
QT_MOC_LITERAL(0, 0, 8), // "QQmlBind"
QT_MOC_LITERAL(1, 9, 11), // "QML.Element"
QT_MOC_LITERAL(2, 21, 7), // "Binding"
QT_MOC_LITERAL(3, 29, 18), // "restoreModeChanged"
QT_MOC_LITERAL(4, 48, 0), // ""
QT_MOC_LITERAL(5, 49, 18), // "targetValueChanged"
QT_MOC_LITERAL(6, 68, 6), // "target"
QT_MOC_LITERAL(7, 75, 8), // "property"
QT_MOC_LITERAL(8, 84, 5), // "value"
QT_MOC_LITERAL(9, 90, 8), // "QJSValue"
QT_MOC_LITERAL(10, 99, 4), // "when"
QT_MOC_LITERAL(11, 104, 7), // "delayed"
QT_MOC_LITERAL(12, 112, 11), // "restoreMode"
QT_MOC_LITERAL(13, 124, 15), // "RestorationMode"
QT_MOC_LITERAL(14, 140, 11), // "RestoreNone"
QT_MOC_LITERAL(15, 152, 14), // "RestoreBinding"
QT_MOC_LITERAL(16, 167, 12), // "RestoreValue"
QT_MOC_LITERAL(17, 180, 21) // "RestoreBindingOrValue"

    },
    "QQmlBind\0QML.Element\0Binding\0"
    "restoreModeChanged\0\0targetValueChanged\0"
    "target\0property\0value\0QJSValue\0when\0"
    "delayed\0restoreMode\0RestorationMode\0"
    "RestoreNone\0RestoreBinding\0RestoreValue\0"
    "RestoreBindingOrValue"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlBind[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       2,   16, // methods
       6,   28, // properties
       1,   58, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   26,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   27,    4, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

 // properties: name, type, flags
       6, QMetaType::QObjectStar, 0x00095003,
       7, QMetaType::QString, 0x00095103,
       8, 0x80000000 | 9, 0x0009510b,
      10, QMetaType::Bool, 0x00095103,
      11, QMetaType::Bool, 0x00895103,
      12, 0x80000000 | 13, 0x00c9510b,

 // properties: notify_signal_id
       0,
       0,
       0,
       0,
       0,
       0,

 // properties: revision
       0,
       0,
       0,
       0,
       8,
      14,

 // enums: name, alias, flags, count, data
      13,   13, 0x0,    4,   63,

 // enum data: key, value
      14, uint(QQmlBind::RestoreNone),
      15, uint(QQmlBind::RestoreBinding),
      16, uint(QQmlBind::RestoreValue),
      17, uint(QQmlBind::RestoreBindingOrValue),

       0        // eod
};

void QQmlBind::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQmlBind *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->restoreModeChanged(); break;
        case 1: _t->targetValueChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQmlBind::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlBind::restoreModeChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQmlBind *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QObject**>(_v) = _t->object(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->property(); break;
        case 2: *reinterpret_cast< QJSValue*>(_v) = _t->value(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->when(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->delayed(); break;
        case 5: *reinterpret_cast< RestorationMode*>(_v) = _t->restoreMode(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQmlBind *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setObject(*reinterpret_cast< QObject**>(_v)); break;
        case 1: _t->setProperty(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setValue(*reinterpret_cast< QJSValue*>(_v)); break;
        case 3: _t->setWhen(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setDelayed(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setRestoreMode(*reinterpret_cast< RestorationMode*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQmlBind::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQmlBind.data,
    qt_meta_data_QQmlBind,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlBind::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlBind::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlBind.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlPropertyValueSource"))
        return static_cast< QQmlPropertyValueSource*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlPropertyValueSource"))
        return static_cast< QQmlPropertyValueSource*>(this);
    return QObject::qt_metacast(_clname);
}

int QQmlBind::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 6;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQmlBind::restoreModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
