/****************************************************************************
** Meta object code from reading C++ file 'qquickanimation_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/util/qquickanimation_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickanimation_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickAbstractAnimation_t {
    QByteArrayData data[30];
    char stringdata0[346];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickAbstractAnimation_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickAbstractAnimation_t qt_meta_stringdata_QQuickAbstractAnimation = {
    {
QT_MOC_LITERAL(0, 0, 23), // "QQuickAbstractAnimation"
QT_MOC_LITERAL(1, 24, 13), // "DefaultMethod"
QT_MOC_LITERAL(2, 38, 7), // "start()"
QT_MOC_LITERAL(3, 46, 11), // "QML.Element"
QT_MOC_LITERAL(4, 58, 9), // "Animation"
QT_MOC_LITERAL(5, 68, 13), // "QML.Creatable"
QT_MOC_LITERAL(6, 82, 5), // "false"
QT_MOC_LITERAL(7, 88, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(8, 110, 30), // "Animation is an abstract class"
QT_MOC_LITERAL(9, 141, 7), // "started"
QT_MOC_LITERAL(10, 149, 0), // ""
QT_MOC_LITERAL(11, 150, 7), // "stopped"
QT_MOC_LITERAL(12, 158, 14), // "runningChanged"
QT_MOC_LITERAL(13, 173, 13), // "pausedChanged"
QT_MOC_LITERAL(14, 187, 21), // "alwaysRunToEndChanged"
QT_MOC_LITERAL(15, 209, 16), // "loopCountChanged"
QT_MOC_LITERAL(16, 226, 8), // "finished"
QT_MOC_LITERAL(17, 235, 7), // "restart"
QT_MOC_LITERAL(18, 243, 5), // "start"
QT_MOC_LITERAL(19, 249, 5), // "pause"
QT_MOC_LITERAL(20, 255, 6), // "resume"
QT_MOC_LITERAL(21, 262, 4), // "stop"
QT_MOC_LITERAL(22, 267, 8), // "complete"
QT_MOC_LITERAL(23, 276, 18), // "componentFinalized"
QT_MOC_LITERAL(24, 295, 7), // "running"
QT_MOC_LITERAL(25, 303, 6), // "paused"
QT_MOC_LITERAL(26, 310, 14), // "alwaysRunToEnd"
QT_MOC_LITERAL(27, 325, 5), // "loops"
QT_MOC_LITERAL(28, 331, 5), // "Loops"
QT_MOC_LITERAL(29, 337, 8) // "Infinite"

    },
    "QQuickAbstractAnimation\0DefaultMethod\0"
    "start()\0QML.Element\0Animation\0"
    "QML.Creatable\0false\0QML.UncreatableReason\0"
    "Animation is an abstract class\0started\0"
    "\0stopped\0runningChanged\0pausedChanged\0"
    "alwaysRunToEndChanged\0loopCountChanged\0"
    "finished\0restart\0start\0pause\0resume\0"
    "stop\0complete\0componentFinalized\0"
    "running\0paused\0alwaysRunToEnd\0loops\0"
    "Loops\0Infinite"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickAbstractAnimation[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
      14,   22, // methods
       4,  128, // properties
       1,  144, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

 // signals: name, argc, parameters, tag, flags
       9,    0,  106,   10, 0x06 /* Public */,
      11,    0,  107,   10, 0x06 /* Public */,
      12,    1,  108,   10, 0x06 /* Public */,
      13,    1,  111,   10, 0x06 /* Public */,
      14,    1,  114,   10, 0x06 /* Public */,
      15,    1,  117,   10, 0x06 /* Public */,
      16,    0,  120,   10, 0x86 /* Public | MethodRevisioned */,

 // slots: name, argc, parameters, tag, flags
      17,    0,  121,   10, 0x0a /* Public */,
      18,    0,  122,   10, 0x0a /* Public */,
      19,    0,  123,   10, 0x0a /* Public */,
      20,    0,  124,   10, 0x0a /* Public */,
      21,    0,  125,   10, 0x0a /* Public */,
      22,    0,  126,   10, 0x0a /* Public */,
      23,    0,  127,   10, 0x08 /* Private */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
       0,
      12,

 // slots: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      24, QMetaType::Bool, 0x00495103,
      25, QMetaType::Bool, 0x00495103,
      26, QMetaType::Bool, 0x00495103,
      27, QMetaType::Int, 0x00495103,

 // properties: notify_signal_id
       2,
       3,
       4,
       5,

 // enums: name, alias, flags, count, data
      28,   28, 0x0,    1,  149,

 // enum data: key, value
      29, uint(QQuickAbstractAnimation::Infinite),

       0        // eod
};

void QQuickAbstractAnimation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickAbstractAnimation *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->started(); break;
        case 1: _t->stopped(); break;
        case 2: _t->runningChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->pausedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->alwaysRunToEndChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->loopCountChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->finished(); break;
        case 7: _t->restart(); break;
        case 8: _t->start(); break;
        case 9: _t->pause(); break;
        case 10: _t->resume(); break;
        case 11: _t->stop(); break;
        case 12: _t->complete(); break;
        case 13: _t->componentFinalized(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickAbstractAnimation::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAbstractAnimation::started)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickAbstractAnimation::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAbstractAnimation::stopped)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickAbstractAnimation::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAbstractAnimation::runningChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickAbstractAnimation::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAbstractAnimation::pausedChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickAbstractAnimation::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAbstractAnimation::alwaysRunToEndChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickAbstractAnimation::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAbstractAnimation::loopCountChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickAbstractAnimation::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAbstractAnimation::finished)) {
                *result = 6;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickAbstractAnimation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->isRunning(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->isPaused(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->alwaysRunToEnd(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->loops(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickAbstractAnimation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRunning(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setPaused(*reinterpret_cast< bool*>(_v)); break;
        case 2: _t->setAlwaysRunToEnd(*reinterpret_cast< bool*>(_v)); break;
        case 3: _t->setLoops(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickAbstractAnimation::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickAbstractAnimation.data,
    qt_meta_data_QQuickAbstractAnimation,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickAbstractAnimation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickAbstractAnimation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickAbstractAnimation.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlPropertyValueSource"))
        return static_cast< QQmlPropertyValueSource*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlPropertyValueSource"))
        return static_cast< QQmlPropertyValueSource*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickAbstractAnimation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickAbstractAnimation::started()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickAbstractAnimation::stopped()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickAbstractAnimation::runningChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickAbstractAnimation::pausedChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickAbstractAnimation::alwaysRunToEndChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QQuickAbstractAnimation::loopCountChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QQuickAbstractAnimation::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
struct qt_meta_stringdata_QQuickPauseAnimation_t {
    QByteArrayData data[6];
    char stringdata0[74];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPauseAnimation_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPauseAnimation_t qt_meta_stringdata_QQuickPauseAnimation = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QQuickPauseAnimation"
QT_MOC_LITERAL(1, 21, 11), // "QML.Element"
QT_MOC_LITERAL(2, 33, 14), // "PauseAnimation"
QT_MOC_LITERAL(3, 48, 15), // "durationChanged"
QT_MOC_LITERAL(4, 64, 0), // ""
QT_MOC_LITERAL(5, 65, 8) // "duration"

    },
    "QQuickPauseAnimation\0QML.Element\0"
    "PauseAnimation\0durationChanged\0\0"
    "duration"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPauseAnimation[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       1,   16, // methods
       1,   24, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,   21,    4, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    4,

 // properties: name, type, flags
       5, QMetaType::Int, 0x00495103,

 // properties: notify_signal_id
       0,

       0        // eod
};

void QQuickPauseAnimation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickPauseAnimation *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->durationChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickPauseAnimation::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPauseAnimation::durationChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickPauseAnimation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->duration(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickPauseAnimation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDuration(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickPauseAnimation::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickAbstractAnimation::staticMetaObject>(),
    qt_meta_stringdata_QQuickPauseAnimation.data,
    qt_meta_data_QQuickPauseAnimation,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPauseAnimation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPauseAnimation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPauseAnimation.stringdata0))
        return static_cast<void*>(this);
    return QQuickAbstractAnimation::qt_metacast(_clname);
}

int QQuickPauseAnimation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickAbstractAnimation::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickPauseAnimation::durationChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_QQuickScriptAction_t {
    QByteArrayData data[6];
    char stringdata0[79];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickScriptAction_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickScriptAction_t qt_meta_stringdata_QQuickScriptAction = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QQuickScriptAction"
QT_MOC_LITERAL(1, 19, 11), // "QML.Element"
QT_MOC_LITERAL(2, 31, 12), // "ScriptAction"
QT_MOC_LITERAL(3, 44, 6), // "script"
QT_MOC_LITERAL(4, 51, 16), // "QQmlScriptString"
QT_MOC_LITERAL(5, 68, 10) // "scriptName"

    },
    "QQuickScriptAction\0QML.Element\0"
    "ScriptAction\0script\0QQmlScriptString\0"
    "scriptName"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickScriptAction[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       0,    0, // methods
       2,   16, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // properties: name, type, flags
       3, 0x80000000 | 4, 0x0009510b,
       5, QMetaType::QString, 0x00095003,

       0        // eod
};

void QQuickScriptAction::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlScriptString >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickScriptAction *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQmlScriptString*>(_v) = _t->script(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->stateChangeScriptName(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickScriptAction *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setScript(*reinterpret_cast< QQmlScriptString*>(_v)); break;
        case 1: _t->setStateChangeScriptName(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
}

QT_INIT_METAOBJECT const QMetaObject QQuickScriptAction::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickAbstractAnimation::staticMetaObject>(),
    qt_meta_stringdata_QQuickScriptAction.data,
    qt_meta_data_QQuickScriptAction,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickScriptAction::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickScriptAction::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickScriptAction.stringdata0))
        return static_cast<void*>(this);
    return QQuickAbstractAnimation::qt_metacast(_clname);
}

int QQuickScriptAction::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickAbstractAnimation::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QQuickPropertyAction_t {
    QByteArrayData data[15];
    char stringdata0[185];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPropertyAction_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPropertyAction_t qt_meta_stringdata_QQuickPropertyAction = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QQuickPropertyAction"
QT_MOC_LITERAL(1, 21, 11), // "QML.Element"
QT_MOC_LITERAL(2, 33, 14), // "PropertyAction"
QT_MOC_LITERAL(3, 48, 12), // "valueChanged"
QT_MOC_LITERAL(4, 61, 0), // ""
QT_MOC_LITERAL(5, 62, 17), // "propertiesChanged"
QT_MOC_LITERAL(6, 80, 13), // "targetChanged"
QT_MOC_LITERAL(7, 94, 15), // "propertyChanged"
QT_MOC_LITERAL(8, 110, 6), // "target"
QT_MOC_LITERAL(9, 117, 8), // "property"
QT_MOC_LITERAL(10, 126, 10), // "properties"
QT_MOC_LITERAL(11, 137, 7), // "targets"
QT_MOC_LITERAL(12, 145, 25), // "QQmlListProperty<QObject>"
QT_MOC_LITERAL(13, 171, 7), // "exclude"
QT_MOC_LITERAL(14, 179, 5) // "value"

    },
    "QQuickPropertyAction\0QML.Element\0"
    "PropertyAction\0valueChanged\0\0"
    "propertiesChanged\0targetChanged\0"
    "propertyChanged\0target\0property\0"
    "properties\0targets\0QQmlListProperty<QObject>\0"
    "exclude\0value"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPropertyAction[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       4,   16, // methods
       6,   44, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,   36,    4, 0x06 /* Public */,
       5,    1,   39,    4, 0x06 /* Public */,
       6,    0,   42,    4, 0x06 /* Public */,
       7,    0,   43,    4, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QVariant,    4,
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       8, QMetaType::QObjectStar, 0x00495003,
       9, QMetaType::QString, 0x00495103,
      10, QMetaType::QString, 0x00495103,
      11, 0x80000000 | 12, 0x00095009,
      13, 0x80000000 | 12, 0x00095009,
      14, QMetaType::QVariant, 0x00495103,

 // properties: notify_signal_id
       2,
       3,
       1,
       0,
       0,
       0,

       0        // eod
};

void QQuickPropertyAction::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickPropertyAction *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->valueChanged((*reinterpret_cast< const QVariant(*)>(_a[1]))); break;
        case 1: _t->propertiesChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->targetChanged(); break;
        case 3: _t->propertyChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickPropertyAction::*)(const QVariant & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPropertyAction::valueChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickPropertyAction::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPropertyAction::propertiesChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickPropertyAction::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPropertyAction::targetChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickPropertyAction::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPropertyAction::propertyChanged)) {
                *result = 3;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QObject> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickPropertyAction *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QObject**>(_v) = _t->target(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->property(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->properties(); break;
        case 3: *reinterpret_cast< QQmlListProperty<QObject>*>(_v) = _t->targets(); break;
        case 4: *reinterpret_cast< QQmlListProperty<QObject>*>(_v) = _t->exclude(); break;
        case 5: *reinterpret_cast< QVariant*>(_v) = _t->value(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickPropertyAction *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTargetObject(*reinterpret_cast< QObject**>(_v)); break;
        case 1: _t->setProperty(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setProperties(*reinterpret_cast< QString*>(_v)); break;
        case 5: _t->setValue(*reinterpret_cast< QVariant*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickPropertyAction::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickAbstractAnimation::staticMetaObject>(),
    qt_meta_stringdata_QQuickPropertyAction.data,
    qt_meta_data_QQuickPropertyAction,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPropertyAction::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPropertyAction::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPropertyAction.stringdata0))
        return static_cast<void*>(this);
    return QQuickAbstractAnimation::qt_metacast(_clname);
}

int QQuickPropertyAction::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickAbstractAnimation::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 6;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickPropertyAction::valueChanged(const QVariant & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickPropertyAction::propertiesChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickPropertyAction::targetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickPropertyAction::propertyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
struct qt_meta_stringdata_QQuickPropertyAnimation_t {
    QByteArrayData data[22];
    char stringdata0[261];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPropertyAnimation_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPropertyAnimation_t qt_meta_stringdata_QQuickPropertyAnimation = {
    {
QT_MOC_LITERAL(0, 0, 23), // "QQuickPropertyAnimation"
QT_MOC_LITERAL(1, 24, 11), // "QML.Element"
QT_MOC_LITERAL(2, 36, 17), // "PropertyAnimation"
QT_MOC_LITERAL(3, 54, 15), // "durationChanged"
QT_MOC_LITERAL(4, 70, 0), // ""
QT_MOC_LITERAL(5, 71, 11), // "fromChanged"
QT_MOC_LITERAL(6, 83, 9), // "toChanged"
QT_MOC_LITERAL(7, 93, 13), // "easingChanged"
QT_MOC_LITERAL(8, 107, 12), // "QEasingCurve"
QT_MOC_LITERAL(9, 120, 17), // "propertiesChanged"
QT_MOC_LITERAL(10, 138, 13), // "targetChanged"
QT_MOC_LITERAL(11, 152, 15), // "propertyChanged"
QT_MOC_LITERAL(12, 168, 8), // "duration"
QT_MOC_LITERAL(13, 177, 4), // "from"
QT_MOC_LITERAL(14, 182, 2), // "to"
QT_MOC_LITERAL(15, 185, 6), // "easing"
QT_MOC_LITERAL(16, 192, 6), // "target"
QT_MOC_LITERAL(17, 199, 8), // "property"
QT_MOC_LITERAL(18, 208, 10), // "properties"
QT_MOC_LITERAL(19, 219, 7), // "targets"
QT_MOC_LITERAL(20, 227, 25), // "QQmlListProperty<QObject>"
QT_MOC_LITERAL(21, 253, 7) // "exclude"

    },
    "QQuickPropertyAnimation\0QML.Element\0"
    "PropertyAnimation\0durationChanged\0\0"
    "fromChanged\0toChanged\0easingChanged\0"
    "QEasingCurve\0propertiesChanged\0"
    "targetChanged\0propertyChanged\0duration\0"
    "from\0to\0easing\0target\0property\0"
    "properties\0targets\0QQmlListProperty<QObject>\0"
    "exclude"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPropertyAnimation[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       7,   16, // methods
       9,   64, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,   51,    4, 0x06 /* Public */,
       5,    0,   54,    4, 0x06 /* Public */,
       6,    0,   55,    4, 0x06 /* Public */,
       7,    1,   56,    4, 0x06 /* Public */,
       9,    1,   59,    4, 0x06 /* Public */,
      10,    0,   62,    4, 0x06 /* Public */,
      11,    0,   63,    4, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    4,
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      12, QMetaType::Int, 0x00495103,
      13, QMetaType::QVariant, 0x00495103,
      14, QMetaType::QVariant, 0x00495103,
      15, 0x80000000 | 8, 0x0049510b,
      16, QMetaType::QObjectStar, 0x00495003,
      17, QMetaType::QString, 0x00495103,
      18, QMetaType::QString, 0x00495103,
      19, 0x80000000 | 20, 0x00095009,
      21, 0x80000000 | 20, 0x00095009,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       5,
       6,
       4,
       0,
       0,

       0        // eod
};

void QQuickPropertyAnimation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickPropertyAnimation *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->durationChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->fromChanged(); break;
        case 2: _t->toChanged(); break;
        case 3: _t->easingChanged((*reinterpret_cast< const QEasingCurve(*)>(_a[1]))); break;
        case 4: _t->propertiesChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->targetChanged(); break;
        case 6: _t->propertyChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickPropertyAnimation::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPropertyAnimation::durationChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickPropertyAnimation::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPropertyAnimation::fromChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickPropertyAnimation::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPropertyAnimation::toChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickPropertyAnimation::*)(const QEasingCurve & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPropertyAnimation::easingChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickPropertyAnimation::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPropertyAnimation::propertiesChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickPropertyAnimation::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPropertyAnimation::targetChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickPropertyAnimation::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPropertyAnimation::propertyChanged)) {
                *result = 6;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 8:
        case 7:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QObject> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickPropertyAnimation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->duration(); break;
        case 1: *reinterpret_cast< QVariant*>(_v) = _t->from(); break;
        case 2: *reinterpret_cast< QVariant*>(_v) = _t->to(); break;
        case 3: *reinterpret_cast< QEasingCurve*>(_v) = _t->easing(); break;
        case 4: *reinterpret_cast< QObject**>(_v) = _t->target(); break;
        case 5: *reinterpret_cast< QString*>(_v) = _t->property(); break;
        case 6: *reinterpret_cast< QString*>(_v) = _t->properties(); break;
        case 7: *reinterpret_cast< QQmlListProperty<QObject>*>(_v) = _t->targets(); break;
        case 8: *reinterpret_cast< QQmlListProperty<QObject>*>(_v) = _t->exclude(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickPropertyAnimation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDuration(*reinterpret_cast< int*>(_v)); break;
        case 1: _t->setFrom(*reinterpret_cast< QVariant*>(_v)); break;
        case 2: _t->setTo(*reinterpret_cast< QVariant*>(_v)); break;
        case 3: _t->setEasing(*reinterpret_cast< QEasingCurve*>(_v)); break;
        case 4: _t->setTargetObject(*reinterpret_cast< QObject**>(_v)); break;
        case 5: _t->setProperty(*reinterpret_cast< QString*>(_v)); break;
        case 6: _t->setProperties(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickPropertyAnimation::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickAbstractAnimation::staticMetaObject>(),
    qt_meta_stringdata_QQuickPropertyAnimation.data,
    qt_meta_data_QQuickPropertyAnimation,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPropertyAnimation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPropertyAnimation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPropertyAnimation.stringdata0))
        return static_cast<void*>(this);
    return QQuickAbstractAnimation::qt_metacast(_clname);
}

int QQuickPropertyAnimation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickAbstractAnimation::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 9;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 9;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickPropertyAnimation::durationChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickPropertyAnimation::fromChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickPropertyAnimation::toChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickPropertyAnimation::easingChanged(const QEasingCurve & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickPropertyAnimation::propertiesChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QQuickPropertyAnimation::targetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickPropertyAnimation::propertyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
struct qt_meta_stringdata_QQuickColorAnimation_t {
    QByteArrayData data[5];
    char stringdata0[56];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickColorAnimation_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickColorAnimation_t qt_meta_stringdata_QQuickColorAnimation = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QQuickColorAnimation"
QT_MOC_LITERAL(1, 21, 11), // "QML.Element"
QT_MOC_LITERAL(2, 33, 14), // "ColorAnimation"
QT_MOC_LITERAL(3, 48, 4), // "from"
QT_MOC_LITERAL(4, 53, 2) // "to"

    },
    "QQuickColorAnimation\0QML.Element\0"
    "ColorAnimation\0from\0to"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickColorAnimation[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       0,    0, // methods
       2,   16, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // properties: name, type, flags
       3, QMetaType::QColor, 0x00095103,
       4, QMetaType::QColor, 0x00095103,

       0        // eod
};

void QQuickColorAnimation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickColorAnimation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->from(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->to(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickColorAnimation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFrom(*reinterpret_cast< QColor*>(_v)); break;
        case 1: _t->setTo(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickColorAnimation::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickPropertyAnimation::staticMetaObject>(),
    qt_meta_stringdata_QQuickColorAnimation.data,
    qt_meta_data_QQuickColorAnimation,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickColorAnimation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickColorAnimation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickColorAnimation.stringdata0))
        return static_cast<void*>(this);
    return QQuickPropertyAnimation::qt_metacast(_clname);
}

int QQuickColorAnimation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickPropertyAnimation::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QQuickNumberAnimation_t {
    QByteArrayData data[7];
    char stringdata0[80];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickNumberAnimation_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickNumberAnimation_t qt_meta_stringdata_QQuickNumberAnimation = {
    {
QT_MOC_LITERAL(0, 0, 21), // "QQuickNumberAnimation"
QT_MOC_LITERAL(1, 22, 11), // "QML.Element"
QT_MOC_LITERAL(2, 34, 15), // "NumberAnimation"
QT_MOC_LITERAL(3, 50, 11), // "fromChanged"
QT_MOC_LITERAL(4, 62, 9), // "toChanged"
QT_MOC_LITERAL(5, 72, 4), // "from"
QT_MOC_LITERAL(6, 77, 2) // "to"

    },
    "QQuickNumberAnimation\0QML.Element\0"
    "NumberAnimation\0fromChanged\0toChanged\0"
    "from\0to"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickNumberAnimation[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       0,    0, // methods
       2,   16, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // properties: name, type, flags
       5, QMetaType::QReal, 0x00495103,
       6, QMetaType::QReal, 0x00495103,

 // properties: notify_signal_id
    1879048195,
    1879048196,

       0        // eod
};

void QQuickNumberAnimation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickNumberAnimation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->from(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->to(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickNumberAnimation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFrom(*reinterpret_cast< qreal*>(_v)); break;
        case 1: _t->setTo(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickNumberAnimation::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickPropertyAnimation::staticMetaObject>(),
    qt_meta_stringdata_QQuickNumberAnimation.data,
    qt_meta_data_QQuickNumberAnimation,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickNumberAnimation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickNumberAnimation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickNumberAnimation.stringdata0))
        return static_cast<void*>(this);
    return QQuickPropertyAnimation::qt_metacast(_clname);
}

int QQuickNumberAnimation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickPropertyAnimation::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
// If you get a compile error in this function it can be because either
//     a) You are using a NOTIFY signal that does not exist. Fix it.
//     b) You are using a NOTIFY signal that does exist (in a parent class) but has a non-empty parameter list. This is a moc limitation.
Q_DECL_UNUSED static void checkNotifySignalValidity_QQuickNumberAnimation(QQuickNumberAnimation *t) {
    t->fromChanged();
    t->toChanged();
}
struct qt_meta_stringdata_QQuickVector3dAnimation_t {
    QByteArrayData data[7];
    char stringdata0[84];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickVector3dAnimation_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickVector3dAnimation_t qt_meta_stringdata_QQuickVector3dAnimation = {
    {
QT_MOC_LITERAL(0, 0, 23), // "QQuickVector3dAnimation"
QT_MOC_LITERAL(1, 24, 11), // "QML.Element"
QT_MOC_LITERAL(2, 36, 17), // "Vector3dAnimation"
QT_MOC_LITERAL(3, 54, 11), // "fromChanged"
QT_MOC_LITERAL(4, 66, 9), // "toChanged"
QT_MOC_LITERAL(5, 76, 4), // "from"
QT_MOC_LITERAL(6, 81, 2) // "to"

    },
    "QQuickVector3dAnimation\0QML.Element\0"
    "Vector3dAnimation\0fromChanged\0toChanged\0"
    "from\0to"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickVector3dAnimation[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       0,    0, // methods
       2,   16, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // properties: name, type, flags
       5, QMetaType::QVector3D, 0x00495103,
       6, QMetaType::QVector3D, 0x00495103,

 // properties: notify_signal_id
    1879048195,
    1879048196,

       0        // eod
};

void QQuickVector3dAnimation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickVector3dAnimation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QVector3D*>(_v) = _t->from(); break;
        case 1: *reinterpret_cast< QVector3D*>(_v) = _t->to(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickVector3dAnimation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFrom(*reinterpret_cast< QVector3D*>(_v)); break;
        case 1: _t->setTo(*reinterpret_cast< QVector3D*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickVector3dAnimation::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickPropertyAnimation::staticMetaObject>(),
    qt_meta_stringdata_QQuickVector3dAnimation.data,
    qt_meta_data_QQuickVector3dAnimation,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickVector3dAnimation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickVector3dAnimation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickVector3dAnimation.stringdata0))
        return static_cast<void*>(this);
    return QQuickPropertyAnimation::qt_metacast(_clname);
}

int QQuickVector3dAnimation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickPropertyAnimation::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
// If you get a compile error in this function it can be because either
//     a) You are using a NOTIFY signal that does not exist. Fix it.
//     b) You are using a NOTIFY signal that does exist (in a parent class) but has a non-empty parameter list. This is a moc limitation.
Q_DECL_UNUSED static void checkNotifySignalValidity_QQuickVector3dAnimation(QQuickVector3dAnimation *t) {
    t->fromChanged();
    t->toChanged();
}
struct qt_meta_stringdata_QQuickRotationAnimation_t {
    QByteArrayData data[15];
    char stringdata0[176];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickRotationAnimation_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickRotationAnimation_t qt_meta_stringdata_QQuickRotationAnimation = {
    {
QT_MOC_LITERAL(0, 0, 23), // "QQuickRotationAnimation"
QT_MOC_LITERAL(1, 24, 11), // "QML.Element"
QT_MOC_LITERAL(2, 36, 17), // "RotationAnimation"
QT_MOC_LITERAL(3, 54, 16), // "directionChanged"
QT_MOC_LITERAL(4, 71, 0), // ""
QT_MOC_LITERAL(5, 72, 11), // "fromChanged"
QT_MOC_LITERAL(6, 84, 9), // "toChanged"
QT_MOC_LITERAL(7, 94, 4), // "from"
QT_MOC_LITERAL(8, 99, 2), // "to"
QT_MOC_LITERAL(9, 102, 9), // "direction"
QT_MOC_LITERAL(10, 112, 17), // "RotationDirection"
QT_MOC_LITERAL(11, 130, 9), // "Numerical"
QT_MOC_LITERAL(12, 140, 8), // "Shortest"
QT_MOC_LITERAL(13, 149, 9), // "Clockwise"
QT_MOC_LITERAL(14, 159, 16) // "Counterclockwise"

    },
    "QQuickRotationAnimation\0QML.Element\0"
    "RotationAnimation\0directionChanged\0\0"
    "fromChanged\0toChanged\0from\0to\0direction\0"
    "RotationDirection\0Numerical\0Shortest\0"
    "Clockwise\0Counterclockwise"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickRotationAnimation[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       1,   16, // methods
       3,   22, // properties
       1,   34, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   21,    4, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

 // properties: name, type, flags
       7, QMetaType::QReal, 0x00495103,
       8, QMetaType::QReal, 0x00495103,
       9, 0x80000000 | 10, 0x0049510b,

 // properties: notify_signal_id
    1879048197,
    1879048198,
       0,

 // enums: name, alias, flags, count, data
      10,   10, 0x0,    4,   39,

 // enum data: key, value
      11, uint(QQuickRotationAnimation::Numerical),
      12, uint(QQuickRotationAnimation::Shortest),
      13, uint(QQuickRotationAnimation::Clockwise),
      14, uint(QQuickRotationAnimation::Counterclockwise),

       0        // eod
};

void QQuickRotationAnimation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickRotationAnimation *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->directionChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickRotationAnimation::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickRotationAnimation::directionChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickRotationAnimation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->from(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->to(); break;
        case 2: *reinterpret_cast< RotationDirection*>(_v) = _t->direction(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickRotationAnimation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFrom(*reinterpret_cast< qreal*>(_v)); break;
        case 1: _t->setTo(*reinterpret_cast< qreal*>(_v)); break;
        case 2: _t->setDirection(*reinterpret_cast< RotationDirection*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickRotationAnimation::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickPropertyAnimation::staticMetaObject>(),
    qt_meta_stringdata_QQuickRotationAnimation.data,
    qt_meta_data_QQuickRotationAnimation,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickRotationAnimation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickRotationAnimation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickRotationAnimation.stringdata0))
        return static_cast<void*>(this);
    return QQuickPropertyAnimation::qt_metacast(_clname);
}

int QQuickRotationAnimation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickPropertyAnimation::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickRotationAnimation::directionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
// If you get a compile error in this function it can be because either
//     a) You are using a NOTIFY signal that does not exist. Fix it.
//     b) You are using a NOTIFY signal that does exist (in a parent class) but has a non-empty parameter list. This is a moc limitation.
Q_DECL_UNUSED static void checkNotifySignalValidity_QQuickRotationAnimation(QQuickRotationAnimation *t) {
    t->fromChanged();
    t->toChanged();
}
struct qt_meta_stringdata_QQuickAnimationGroup_t {
    QByteArrayData data[4];
    char stringdata0[90];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickAnimationGroup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickAnimationGroup_t qt_meta_stringdata_QQuickAnimationGroup = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QQuickAnimationGroup"
QT_MOC_LITERAL(1, 21, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 37, 10), // "animations"
QT_MOC_LITERAL(3, 48, 41) // "QQmlListProperty<QQuickAbstra..."

    },
    "QQuickAnimationGroup\0DefaultProperty\0"
    "animations\0QQmlListProperty<QQuickAbstractAnimation>"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickAnimationGroup[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       0,    0, // methods
       1,   16, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // properties: name, type, flags
       2, 0x80000000 | 3, 0x00095009,

       0        // eod
};

void QQuickAnimationGroup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QQuickAbstractAnimation> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickAnimationGroup *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQmlListProperty<QQuickAbstractAnimation>*>(_v) = _t->animations(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
}

QT_INIT_METAOBJECT const QMetaObject QQuickAnimationGroup::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickAbstractAnimation::staticMetaObject>(),
    qt_meta_stringdata_QQuickAnimationGroup.data,
    qt_meta_data_QQuickAnimationGroup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickAnimationGroup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickAnimationGroup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickAnimationGroup.stringdata0))
        return static_cast<void*>(this);
    return QQuickAbstractAnimation::qt_metacast(_clname);
}

int QQuickAnimationGroup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickAbstractAnimation::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QQuickSequentialAnimation_t {
    QByteArrayData data[3];
    char stringdata0[58];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickSequentialAnimation_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickSequentialAnimation_t qt_meta_stringdata_QQuickSequentialAnimation = {
    {
QT_MOC_LITERAL(0, 0, 25), // "QQuickSequentialAnimation"
QT_MOC_LITERAL(1, 26, 11), // "QML.Element"
QT_MOC_LITERAL(2, 38, 19) // "SequentialAnimation"

    },
    "QQuickSequentialAnimation\0QML.Element\0"
    "SequentialAnimation"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickSequentialAnimation[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

       0        // eod
};

void QQuickSequentialAnimation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickSequentialAnimation::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickAnimationGroup::staticMetaObject>(),
    qt_meta_stringdata_QQuickSequentialAnimation.data,
    qt_meta_data_QQuickSequentialAnimation,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickSequentialAnimation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickSequentialAnimation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickSequentialAnimation.stringdata0))
        return static_cast<void*>(this);
    return QQuickAnimationGroup::qt_metacast(_clname);
}

int QQuickSequentialAnimation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickAnimationGroup::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQuickParallelAnimation_t {
    QByteArrayData data[3];
    char stringdata0[54];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickParallelAnimation_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickParallelAnimation_t qt_meta_stringdata_QQuickParallelAnimation = {
    {
QT_MOC_LITERAL(0, 0, 23), // "QQuickParallelAnimation"
QT_MOC_LITERAL(1, 24, 11), // "QML.Element"
QT_MOC_LITERAL(2, 36, 17) // "ParallelAnimation"

    },
    "QQuickParallelAnimation\0QML.Element\0"
    "ParallelAnimation"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickParallelAnimation[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

       0        // eod
};

void QQuickParallelAnimation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickParallelAnimation::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickAnimationGroup::staticMetaObject>(),
    qt_meta_stringdata_QQuickParallelAnimation.data,
    qt_meta_data_QQuickParallelAnimation,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickParallelAnimation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickParallelAnimation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickParallelAnimation.stringdata0))
        return static_cast<void*>(this);
    return QQuickAnimationGroup::qt_metacast(_clname);
}

int QQuickParallelAnimation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickAnimationGroup::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
