/****************************************************************************
** Meta object code from reading C++ file 'qquicktextcontrol_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquicktextcontrol_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquicktextcontrol_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickTextControl_t {
    QByteArrayData data[48];
    char stringdata0[618];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickTextControl_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickTextControl_t qt_meta_stringdata_QQuickTextControl = {
    {
QT_MOC_LITERAL(0, 0, 17), // "QQuickTextControl"
QT_MOC_LITERAL(1, 18, 11), // "textChanged"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 18), // "preeditTextChanged"
QT_MOC_LITERAL(4, 50, 14), // "contentsChange"
QT_MOC_LITERAL(5, 65, 4), // "from"
QT_MOC_LITERAL(6, 70, 12), // "charsRemoved"
QT_MOC_LITERAL(7, 83, 10), // "charsAdded"
QT_MOC_LITERAL(8, 94, 13), // "undoAvailable"
QT_MOC_LITERAL(9, 108, 1), // "b"
QT_MOC_LITERAL(10, 110, 13), // "redoAvailable"
QT_MOC_LITERAL(11, 124, 24), // "currentCharFormatChanged"
QT_MOC_LITERAL(12, 149, 15), // "QTextCharFormat"
QT_MOC_LITERAL(13, 165, 6), // "format"
QT_MOC_LITERAL(14, 172, 13), // "copyAvailable"
QT_MOC_LITERAL(15, 186, 16), // "selectionChanged"
QT_MOC_LITERAL(16, 203, 21), // "cursorPositionChanged"
QT_MOC_LITERAL(17, 225, 20), // "overwriteModeChanged"
QT_MOC_LITERAL(18, 246, 13), // "overwriteMode"
QT_MOC_LITERAL(19, 260, 19), // "updateCursorRequest"
QT_MOC_LITERAL(20, 280, 13), // "updateRequest"
QT_MOC_LITERAL(21, 294, 22), // "cursorRectangleChanged"
QT_MOC_LITERAL(22, 317, 13), // "linkActivated"
QT_MOC_LITERAL(23, 331, 4), // "link"
QT_MOC_LITERAL(24, 336, 11), // "linkHovered"
QT_MOC_LITERAL(25, 348, 13), // "markerClicked"
QT_MOC_LITERAL(26, 362, 13), // "markerHovered"
QT_MOC_LITERAL(27, 376, 6), // "marker"
QT_MOC_LITERAL(28, 383, 12), // "setPlainText"
QT_MOC_LITERAL(29, 396, 4), // "text"
QT_MOC_LITERAL(30, 401, 15), // "setMarkdownText"
QT_MOC_LITERAL(31, 417, 7), // "setHtml"
QT_MOC_LITERAL(32, 425, 3), // "cut"
QT_MOC_LITERAL(33, 429, 4), // "copy"
QT_MOC_LITERAL(34, 434, 5), // "paste"
QT_MOC_LITERAL(35, 440, 16), // "QClipboard::Mode"
QT_MOC_LITERAL(36, 457, 4), // "mode"
QT_MOC_LITERAL(37, 462, 4), // "undo"
QT_MOC_LITERAL(38, 467, 4), // "redo"
QT_MOC_LITERAL(39, 472, 5), // "clear"
QT_MOC_LITERAL(40, 478, 9), // "selectAll"
QT_MOC_LITERAL(41, 488, 38), // "_q_updateCurrentCharFormatAnd..."
QT_MOC_LITERAL(42, 527, 25), // "_q_updateCursorPosChanged"
QT_MOC_LITERAL(43, 553, 11), // "QTextCursor"
QT_MOC_LITERAL(44, 565, 16), // "inputMethodQuery"
QT_MOC_LITERAL(45, 582, 20), // "Qt::InputMethodQuery"
QT_MOC_LITERAL(46, 603, 5), // "query"
QT_MOC_LITERAL(47, 609, 8) // "argument"

    },
    "QQuickTextControl\0textChanged\0\0"
    "preeditTextChanged\0contentsChange\0"
    "from\0charsRemoved\0charsAdded\0undoAvailable\0"
    "b\0redoAvailable\0currentCharFormatChanged\0"
    "QTextCharFormat\0format\0copyAvailable\0"
    "selectionChanged\0cursorPositionChanged\0"
    "overwriteModeChanged\0overwriteMode\0"
    "updateCursorRequest\0updateRequest\0"
    "cursorRectangleChanged\0linkActivated\0"
    "link\0linkHovered\0markerClicked\0"
    "markerHovered\0marker\0setPlainText\0"
    "text\0setMarkdownText\0setHtml\0cut\0copy\0"
    "paste\0QClipboard::Mode\0mode\0undo\0redo\0"
    "clear\0selectAll\0_q_updateCurrentCharFormatAndSelection\0"
    "_q_updateCursorPosChanged\0QTextCursor\0"
    "inputMethodQuery\0Qt::InputMethodQuery\0"
    "query\0argument"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickTextControl[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      31,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      17,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  169,    2, 0x06 /* Public */,
       3,    0,  170,    2, 0x06 /* Public */,
       4,    3,  171,    2, 0x06 /* Public */,
       8,    1,  178,    2, 0x06 /* Public */,
      10,    1,  181,    2, 0x06 /* Public */,
      11,    1,  184,    2, 0x06 /* Public */,
      14,    1,  187,    2, 0x06 /* Public */,
      15,    0,  190,    2, 0x06 /* Public */,
      16,    0,  191,    2, 0x06 /* Public */,
      17,    1,  192,    2, 0x06 /* Public */,
      19,    0,  195,    2, 0x06 /* Public */,
      20,    0,  196,    2, 0x06 /* Public */,
      21,    0,  197,    2, 0x06 /* Public */,
      22,    1,  198,    2, 0x06 /* Public */,
      24,    1,  201,    2, 0x06 /* Public */,
      25,    0,  204,    2, 0x06 /* Public */,
      26,    1,  205,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      28,    1,  208,    2, 0x0a /* Public */,
      30,    1,  211,    2, 0x0a /* Public */,
      31,    1,  214,    2, 0x0a /* Public */,
      32,    0,  217,    2, 0x0a /* Public */,
      33,    0,  218,    2, 0x0a /* Public */,
      34,    1,  219,    2, 0x0a /* Public */,
      34,    0,  222,    2, 0x2a /* Public | MethodCloned */,
      37,    0,  223,    2, 0x0a /* Public */,
      38,    0,  224,    2, 0x0a /* Public */,
      39,    0,  225,    2, 0x0a /* Public */,
      40,    0,  226,    2, 0x0a /* Public */,
      41,    0,  227,    2, 0x08 /* Private */,
      42,    1,  228,    2, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      44,    2,  231,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,    5,    6,    7,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   23,
    QMetaType::Void, QMetaType::QString,   23,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   27,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,   29,
    QMetaType::Void, QMetaType::QString,   29,
    QMetaType::Void, QMetaType::QString,   29,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 35,   36,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 43,    2,

 // methods: parameters
    QMetaType::QVariant, 0x80000000 | 45, QMetaType::QVariant,   46,   47,

       0        // eod
};

void QQuickTextControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickTextControl *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->textChanged(); break;
        case 1: _t->preeditTextChanged(); break;
        case 2: _t->contentsChange((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 3: _t->undoAvailable((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->redoAvailable((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->currentCharFormatChanged((*reinterpret_cast< const QTextCharFormat(*)>(_a[1]))); break;
        case 6: _t->copyAvailable((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->selectionChanged(); break;
        case 8: _t->cursorPositionChanged(); break;
        case 9: _t->overwriteModeChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->updateCursorRequest(); break;
        case 11: _t->updateRequest(); break;
        case 12: _t->cursorRectangleChanged(); break;
        case 13: _t->linkActivated((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 14: _t->linkHovered((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 15: _t->markerClicked(); break;
        case 16: _t->markerHovered((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 17: _t->setPlainText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 18: _t->setMarkdownText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 19: _t->setHtml((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 20: _t->cut(); break;
        case 21: _t->copy(); break;
        case 22: _t->paste((*reinterpret_cast< QClipboard::Mode(*)>(_a[1]))); break;
        case 23: _t->paste(); break;
        case 24: _t->undo(); break;
        case 25: _t->redo(); break;
        case 26: _t->clear(); break;
        case 27: _t->selectAll(); break;
        case 28: _t->d_func()->_q_updateCurrentCharFormatAndSelection(); break;
        case 29: _t->d_func()->_q_updateCursorPosChanged((*reinterpret_cast< const QTextCursor(*)>(_a[1]))); break;
        case 30: { QVariant _r = _t->inputMethodQuery((*reinterpret_cast< Qt::InputMethodQuery(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QVariant*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickTextControl::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextControl::textChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickTextControl::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextControl::preeditTextChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickTextControl::*)(int , int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextControl::contentsChange)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickTextControl::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextControl::undoAvailable)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickTextControl::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextControl::redoAvailable)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickTextControl::*)(const QTextCharFormat & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextControl::currentCharFormatChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickTextControl::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextControl::copyAvailable)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickTextControl::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextControl::selectionChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickTextControl::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextControl::cursorPositionChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickTextControl::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextControl::overwriteModeChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickTextControl::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextControl::updateCursorRequest)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickTextControl::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextControl::updateRequest)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQuickTextControl::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextControl::cursorRectangleChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQuickTextControl::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextControl::linkActivated)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QQuickTextControl::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextControl::linkHovered)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QQuickTextControl::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextControl::markerClicked)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QQuickTextControl::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextControl::markerHovered)) {
                *result = 16;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QQuickTextControl::staticMetaObject = { {
    QMetaObject::SuperData::link<QInputControl::staticMetaObject>(),
    qt_meta_stringdata_QQuickTextControl.data,
    qt_meta_data_QQuickTextControl,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickTextControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickTextControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickTextControl.stringdata0))
        return static_cast<void*>(this);
    return QInputControl::qt_metacast(_clname);
}

int QQuickTextControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QInputControl::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 31)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 31;
    }
    return _id;
}

// SIGNAL 0
void QQuickTextControl::textChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickTextControl::preeditTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickTextControl::contentsChange(int _t1, int _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickTextControl::undoAvailable(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickTextControl::redoAvailable(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QQuickTextControl::currentCharFormatChanged(const QTextCharFormat & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QQuickTextControl::copyAvailable(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QQuickTextControl::selectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickTextControl::cursorPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickTextControl::overwriteModeChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QQuickTextControl::updateCursorRequest()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QQuickTextControl::updateRequest()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QQuickTextControl::cursorRectangleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QQuickTextControl::linkActivated(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QQuickTextControl::linkHovered(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QQuickTextControl::markerClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QQuickTextControl::markerHovered(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
