/****************************************************************************
** Meta object code from reading C++ file 'qqmlabstractprofileradapter_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/qml/debugger/qqmlabstractprofileradapter_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qqmlabstractprofileradapter_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQmlAbstractProfilerAdapter_t {
    QByteArrayData data[11];
    char stringdata0[185];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlAbstractProfilerAdapter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlAbstractProfilerAdapter_t qt_meta_stringdata_QQmlAbstractProfilerAdapter = {
    {
QT_MOC_LITERAL(0, 0, 27), // "QQmlAbstractProfilerAdapter"
QT_MOC_LITERAL(1, 28, 16), // "profilingEnabled"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 8), // "features"
QT_MOC_LITERAL(4, 55, 28), // "profilingEnabledWhileWaiting"
QT_MOC_LITERAL(5, 84, 17), // "profilingDisabled"
QT_MOC_LITERAL(6, 102, 29), // "profilingDisabledWhileWaiting"
QT_MOC_LITERAL(7, 132, 13), // "dataRequested"
QT_MOC_LITERAL(8, 146, 18), // "referenceTimeKnown"
QT_MOC_LITERAL(9, 165, 13), // "QElapsedTimer"
QT_MOC_LITERAL(10, 179, 5) // "timer"

    },
    "QQmlAbstractProfilerAdapter\0"
    "profilingEnabled\0\0features\0"
    "profilingEnabledWhileWaiting\0"
    "profilingDisabled\0profilingDisabledWhileWaiting\0"
    "dataRequested\0referenceTimeKnown\0"
    "QElapsedTimer\0timer"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlAbstractProfilerAdapter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,
       4,    1,   47,    2, 0x06 /* Public */,
       5,    0,   50,    2, 0x06 /* Public */,
       6,    0,   51,    2, 0x06 /* Public */,
       7,    0,   52,    2, 0x06 /* Public */,
       8,    1,   53,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::ULongLong,    3,
    QMetaType::Void, QMetaType::ULongLong,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 9,   10,

       0        // eod
};

void QQmlAbstractProfilerAdapter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQmlAbstractProfilerAdapter *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->profilingEnabled((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        case 1: _t->profilingEnabledWhileWaiting((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        case 2: _t->profilingDisabled(); break;
        case 3: _t->profilingDisabledWhileWaiting(); break;
        case 4: _t->dataRequested(); break;
        case 5: _t->referenceTimeKnown((*reinterpret_cast< const QElapsedTimer(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQmlAbstractProfilerAdapter::*)(quint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlAbstractProfilerAdapter::profilingEnabled)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQmlAbstractProfilerAdapter::*)(quint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlAbstractProfilerAdapter::profilingEnabledWhileWaiting)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQmlAbstractProfilerAdapter::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlAbstractProfilerAdapter::profilingDisabled)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQmlAbstractProfilerAdapter::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlAbstractProfilerAdapter::profilingDisabledWhileWaiting)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQmlAbstractProfilerAdapter::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlAbstractProfilerAdapter::dataRequested)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQmlAbstractProfilerAdapter::*)(const QElapsedTimer & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlAbstractProfilerAdapter::referenceTimeKnown)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QQmlAbstractProfilerAdapter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQmlAbstractProfilerAdapter.data,
    qt_meta_data_QQmlAbstractProfilerAdapter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlAbstractProfilerAdapter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlAbstractProfilerAdapter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlAbstractProfilerAdapter.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlProfilerDefinitions"))
        return static_cast< QQmlProfilerDefinitions*>(this);
    return QObject::qt_metacast(_clname);
}

int QQmlAbstractProfilerAdapter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void QQmlAbstractProfilerAdapter::profilingEnabled(quint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQmlAbstractProfilerAdapter::profilingEnabledWhileWaiting(quint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQmlAbstractProfilerAdapter::profilingDisabled()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQmlAbstractProfilerAdapter::profilingDisabledWhileWaiting()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQmlAbstractProfilerAdapter::dataRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQmlAbstractProfilerAdapter::referenceTimeKnown(const QElapsedTimer & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
struct qt_meta_stringdata_QQmlAbstractProfilerAdapterFactory_t {
    QByteArrayData data[1];
    char stringdata0[35];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlAbstractProfilerAdapterFactory_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlAbstractProfilerAdapterFactory_t qt_meta_stringdata_QQmlAbstractProfilerAdapterFactory = {
    {
QT_MOC_LITERAL(0, 0, 34) // "QQmlAbstractProfilerAdapterFa..."

    },
    "QQmlAbstractProfilerAdapterFactory"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlAbstractProfilerAdapterFactory[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QQmlAbstractProfilerAdapterFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQmlAbstractProfilerAdapterFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQmlAbstractProfilerAdapterFactory.data,
    qt_meta_data_QQmlAbstractProfilerAdapterFactory,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlAbstractProfilerAdapterFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlAbstractProfilerAdapterFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlAbstractProfilerAdapterFactory.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQmlAbstractProfilerAdapterFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
