/****************************************************************************
** Meta object code from reading C++ file 'qquicksprite_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquicksprite_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquicksprite_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickSprite_t {
    QByteArrayData data[44];
    char stringdata0[609];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickSprite_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickSprite_t qt_meta_stringdata_QQuickSprite = {
    {
QT_MOC_LITERAL(0, 0, 12), // "QQuickSprite"
QT_MOC_LITERAL(1, 13, 11), // "QML.Element"
QT_MOC_LITERAL(2, 25, 6), // "Sprite"
QT_MOC_LITERAL(3, 32, 13), // "sourceChanged"
QT_MOC_LITERAL(4, 46, 0), // ""
QT_MOC_LITERAL(5, 47, 3), // "arg"
QT_MOC_LITERAL(6, 51, 18), // "frameHeightChanged"
QT_MOC_LITERAL(7, 70, 17), // "frameWidthChanged"
QT_MOC_LITERAL(8, 88, 14), // "reverseChanged"
QT_MOC_LITERAL(9, 103, 17), // "frameCountChanged"
QT_MOC_LITERAL(10, 121, 13), // "frameXChanged"
QT_MOC_LITERAL(11, 135, 13), // "frameYChanged"
QT_MOC_LITERAL(12, 149, 16), // "frameRateChanged"
QT_MOC_LITERAL(13, 166, 25), // "frameRateVariationChanged"
QT_MOC_LITERAL(14, 192, 20), // "frameDurationChanged"
QT_MOC_LITERAL(15, 213, 29), // "frameDurationVariationChanged"
QT_MOC_LITERAL(16, 243, 16), // "frameSyncChanged"
QT_MOC_LITERAL(17, 260, 9), // "setSource"
QT_MOC_LITERAL(18, 270, 14), // "setFrameHeight"
QT_MOC_LITERAL(19, 285, 13), // "setFrameWidth"
QT_MOC_LITERAL(20, 299, 10), // "setReverse"
QT_MOC_LITERAL(21, 310, 9), // "setFrames"
QT_MOC_LITERAL(22, 320, 13), // "setFrameCount"
QT_MOC_LITERAL(23, 334, 9), // "setFrameX"
QT_MOC_LITERAL(24, 344, 9), // "setFrameY"
QT_MOC_LITERAL(25, 354, 12), // "setFrameRate"
QT_MOC_LITERAL(26, 367, 21), // "setFrameRateVariation"
QT_MOC_LITERAL(27, 389, 16), // "setFrameDuration"
QT_MOC_LITERAL(28, 406, 25), // "setFrameDurationVariation"
QT_MOC_LITERAL(29, 432, 12), // "setFrameSync"
QT_MOC_LITERAL(30, 445, 17), // "startImageLoading"
QT_MOC_LITERAL(31, 463, 6), // "source"
QT_MOC_LITERAL(32, 470, 7), // "reverse"
QT_MOC_LITERAL(33, 478, 9), // "frameSync"
QT_MOC_LITERAL(34, 488, 6), // "frames"
QT_MOC_LITERAL(35, 495, 10), // "frameCount"
QT_MOC_LITERAL(36, 506, 11), // "frameHeight"
QT_MOC_LITERAL(37, 518, 10), // "frameWidth"
QT_MOC_LITERAL(38, 529, 6), // "frameX"
QT_MOC_LITERAL(39, 536, 6), // "frameY"
QT_MOC_LITERAL(40, 543, 9), // "frameRate"
QT_MOC_LITERAL(41, 553, 18), // "frameRateVariation"
QT_MOC_LITERAL(42, 572, 13), // "frameDuration"
QT_MOC_LITERAL(43, 586, 22) // "frameDurationVariation"

    },
    "QQuickSprite\0QML.Element\0Sprite\0"
    "sourceChanged\0\0arg\0frameHeightChanged\0"
    "frameWidthChanged\0reverseChanged\0"
    "frameCountChanged\0frameXChanged\0"
    "frameYChanged\0frameRateChanged\0"
    "frameRateVariationChanged\0"
    "frameDurationChanged\0frameDurationVariationChanged\0"
    "frameSyncChanged\0setSource\0setFrameHeight\0"
    "setFrameWidth\0setReverse\0setFrames\0"
    "setFrameCount\0setFrameX\0setFrameY\0"
    "setFrameRate\0setFrameRateVariation\0"
    "setFrameDuration\0setFrameDurationVariation\0"
    "setFrameSync\0startImageLoading\0source\0"
    "reverse\0frameSync\0frames\0frameCount\0"
    "frameHeight\0frameWidth\0frameX\0frameY\0"
    "frameRate\0frameRateVariation\0frameDuration\0"
    "frameDurationVariation"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickSprite[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      26,   16, // methods
      13,  222, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      12,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,  146,    4, 0x06 /* Public */,
       6,    1,  149,    4, 0x06 /* Public */,
       7,    1,  152,    4, 0x06 /* Public */,
       8,    1,  155,    4, 0x06 /* Public */,
       9,    1,  158,    4, 0x06 /* Public */,
      10,    1,  161,    4, 0x06 /* Public */,
      11,    1,  164,    4, 0x06 /* Public */,
      12,    1,  167,    4, 0x06 /* Public */,
      13,    1,  170,    4, 0x06 /* Public */,
      14,    1,  173,    4, 0x06 /* Public */,
      15,    1,  176,    4, 0x06 /* Public */,
      16,    1,  179,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      17,    1,  182,    4, 0x0a /* Public */,
      18,    1,  185,    4, 0x0a /* Public */,
      19,    1,  188,    4, 0x0a /* Public */,
      20,    1,  191,    4, 0x0a /* Public */,
      21,    1,  194,    4, 0x0a /* Public */,
      22,    1,  197,    4, 0x0a /* Public */,
      23,    1,  200,    4, 0x0a /* Public */,
      24,    1,  203,    4, 0x0a /* Public */,
      25,    1,  206,    4, 0x0a /* Public */,
      26,    1,  209,    4, 0x0a /* Public */,
      27,    1,  212,    4, 0x0a /* Public */,
      28,    1,  215,    4, 0x0a /* Public */,
      29,    1,  218,    4, 0x0a /* Public */,
      30,    0,  221,    4, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QUrl,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Bool,    5,

 // slots: parameters
    QMetaType::Void, QMetaType::QUrl,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void,

 // properties: name, type, flags
      31, QMetaType::QUrl, 0x00495103,
      32, QMetaType::Bool, 0x00495103,
      33, QMetaType::Bool, 0x00495103,
      34, QMetaType::Int, 0x00495103,
      35, QMetaType::Int, 0x00495103,
      36, QMetaType::Int, 0x00495103,
      37, QMetaType::Int, 0x00495103,
      38, QMetaType::Int, 0x00495103,
      39, QMetaType::Int, 0x00495103,
      40, QMetaType::QReal, 0x00495107,
      41, QMetaType::QReal, 0x00495103,
      42, QMetaType::Int, 0x00495107,
      43, QMetaType::Int, 0x00495103,

 // properties: notify_signal_id
       0,
       3,
      11,
       4,
       4,
       1,
       2,
       5,
       6,
       7,
       8,
       9,
      10,

       0        // eod
};

void QQuickSprite::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickSprite *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sourceChanged((*reinterpret_cast< QUrl(*)>(_a[1]))); break;
        case 1: _t->frameHeightChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->frameWidthChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->reverseChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->frameCountChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->frameXChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->frameYChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->frameRateChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 8: _t->frameRateVariationChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 9: _t->frameDurationChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->frameDurationVariationChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->frameSyncChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->setSource((*reinterpret_cast< QUrl(*)>(_a[1]))); break;
        case 13: _t->setFrameHeight((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->setFrameWidth((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->setReverse((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 16: _t->setFrames((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->setFrameCount((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->setFrameX((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->setFrameY((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 20: _t->setFrameRate((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 21: _t->setFrameRateVariation((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 22: _t->setFrameDuration((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 23: _t->setFrameDurationVariation((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 24: _t->setFrameSync((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 25: _t->startImageLoading(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickSprite::*)(QUrl );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickSprite::sourceChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickSprite::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickSprite::frameHeightChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickSprite::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickSprite::frameWidthChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickSprite::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickSprite::reverseChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickSprite::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickSprite::frameCountChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickSprite::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickSprite::frameXChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickSprite::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickSprite::frameYChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickSprite::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickSprite::frameRateChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickSprite::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickSprite::frameRateVariationChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickSprite::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickSprite::frameDurationChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickSprite::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickSprite::frameDurationVariationChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickSprite::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickSprite::frameSyncChanged)) {
                *result = 11;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickSprite *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QUrl*>(_v) = _t->source(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->reverse(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->frameSync(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->frames(); break;
        case 4: *reinterpret_cast< int*>(_v) = _t->frameCount(); break;
        case 5: *reinterpret_cast< int*>(_v) = _t->frameHeight(); break;
        case 6: *reinterpret_cast< int*>(_v) = _t->frameWidth(); break;
        case 7: *reinterpret_cast< int*>(_v) = _t->frameX(); break;
        case 8: *reinterpret_cast< int*>(_v) = _t->frameY(); break;
        case 9: *reinterpret_cast< qreal*>(_v) = _t->frameRate(); break;
        case 10: *reinterpret_cast< qreal*>(_v) = _t->frameRateVariation(); break;
        case 11: *reinterpret_cast< int*>(_v) = _t->frameDuration(); break;
        case 12: *reinterpret_cast< int*>(_v) = _t->frameDurationVariation(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickSprite *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSource(*reinterpret_cast< QUrl*>(_v)); break;
        case 1: _t->setReverse(*reinterpret_cast< bool*>(_v)); break;
        case 2: _t->setFrameSync(*reinterpret_cast< bool*>(_v)); break;
        case 3: _t->setFrames(*reinterpret_cast< int*>(_v)); break;
        case 4: _t->setFrameCount(*reinterpret_cast< int*>(_v)); break;
        case 5: _t->setFrameHeight(*reinterpret_cast< int*>(_v)); break;
        case 6: _t->setFrameWidth(*reinterpret_cast< int*>(_v)); break;
        case 7: _t->setFrameX(*reinterpret_cast< int*>(_v)); break;
        case 8: _t->setFrameY(*reinterpret_cast< int*>(_v)); break;
        case 9: _t->setFrameRate(*reinterpret_cast< qreal*>(_v)); break;
        case 10: _t->setFrameRateVariation(*reinterpret_cast< qreal*>(_v)); break;
        case 11: _t->setFrameDuration(*reinterpret_cast< int*>(_v)); break;
        case 12: _t->setFrameDurationVariation(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickSprite *_t = static_cast<QQuickSprite *>(_o);
        (void)_t;
        switch (_id) {
        case 9: _t->resetFrameRate(); break;
        case 11: _t->resetFrameDuration(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickSprite::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickStochasticState::staticMetaObject>(),
    qt_meta_stringdata_QQuickSprite.data,
    qt_meta_data_QQuickSprite,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickSprite::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickSprite::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickSprite.stringdata0))
        return static_cast<void*>(this);
    return QQuickStochasticState::qt_metacast(_clname);
}

int QQuickSprite::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickStochasticState::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 26;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 13;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickSprite::sourceChanged(QUrl _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickSprite::frameHeightChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickSprite::frameWidthChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickSprite::reverseChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickSprite::frameCountChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QQuickSprite::frameXChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QQuickSprite::frameYChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QQuickSprite::frameRateChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QQuickSprite::frameRateVariationChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QQuickSprite::frameDurationChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QQuickSprite::frameDurationVariationChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QQuickSprite::frameSyncChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
