/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 5.15.4
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

static const unsigned char qt_resource_data[] = {
  // qmldir
  0x0,0x0,0x0,0x36,
  0x6d,
  0x6f,0x64,0x75,0x6c,0x65,0x20,0x6d,0x6f,0x64,0x75,0x6c,0x65,0x57,0x69,0x74,0x68,
  0x57,0x61,0x69,0x74,0x69,0x6e,0x67,0x50,0x6c,0x75,0x67,0x69,0x6e,0xa,0x70,0x6c,
  0x75,0x67,0x69,0x6e,0x20,0x70,0x6c,0x75,0x67,0x69,0x6e,0x54,0x68,0x61,0x74,0x57,
  0x61,0x69,0x74,0x73,0xa,
  
};

static const unsigned char qt_resource_name[] = {
  // qt-project.org
  0x0,0xe,
  0x5,0x28,0x6c,0x87,
  0x0,0x71,
  0x0,0x74,0x0,0x2d,0x0,0x70,0x0,0x72,0x0,0x6f,0x0,0x6a,0x0,0x65,0x0,0x63,0x0,0x74,0x0,0x2e,0x0,0x6f,0x0,0x72,0x0,0x67,
    // imports
  0x0,0x7,
  0x0,0x47,0x69,0x53,
  0x0,0x69,
  0x0,0x6d,0x0,0x70,0x0,0x6f,0x0,0x72,0x0,0x74,0x0,0x73,
    // moduleWithWaitingPlugin
  0x0,0x17,
  0x8,0x68,0x10,0x1e,
  0x0,0x6d,
  0x0,0x6f,0x0,0x64,0x0,0x75,0x0,0x6c,0x0,0x65,0x0,0x57,0x0,0x69,0x0,0x74,0x0,0x68,0x0,0x57,0x0,0x61,0x0,0x69,0x0,0x74,0x0,0x69,0x0,0x6e,0x0,0x67,
  0x0,0x50,0x0,0x6c,0x0,0x75,0x0,0x67,0x0,0x69,0x0,0x6e,
    // qmldir
  0x0,0x6,
  0x7,0x84,0x2b,0x2,
  0x0,0x71,
  0x0,0x6d,0x0,0x6c,0x0,0x64,0x0,0x69,0x0,0x72,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x80,0x3,0xf9,0x76,0x10,
  // :/qt-project.org
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0x80,0x3,0xf9,0x76,0x10,
  // :/qt-project.org/imports
  0x0,0x0,0x0,0x22,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0x80,0x3,0xf9,0x76,0x10,
  // :/qt-project.org/imports/moduleWithWaitingPlugin
  0x0,0x0,0x0,0x36,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x4,
0x0,0x0,0x1,0x80,0x3,0xf9,0x76,0x10,
  // :/qt-project.org/imports/moduleWithWaitingPlugin/qmldir
  0x0,0x0,0x0,0x6a,0x0,0x0,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x80,0x3,0xf9,0x76,0x10,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#ifdef QT_NAMESPACE
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_waitingPlugin)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_waitingPlugin)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_waitingPlugin)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_waitingPlugin)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_waitingPlugin)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_waitingPlugin)(); }
   } dummy;
}
