/****************************************************************************
** Meta object code from reading C++ file 'debugutil_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../../git/tests/auto/qml/debugger/shared/debugutil_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'debugutil_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQmlDebugTest_t {
    QByteArrayData data[11];
    char stringdata0[139];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlDebugTest_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlDebugTest_t qt_meta_stringdata_QQmlDebugTest = {
    {
QT_MOC_LITERAL(0, 0, 13), // "QQmlDebugTest"
QT_MOC_LITERAL(1, 14, 7), // "cleanup"
QT_MOC_LITERAL(2, 22, 0), // ""
QT_MOC_LITERAL(3, 23, 13), // "ConnectResult"
QT_MOC_LITERAL(4, 37, 14), // "ConnectSuccess"
QT_MOC_LITERAL(5, 52, 13), // "ProcessFailed"
QT_MOC_LITERAL(6, 66, 13), // "SessionFailed"
QT_MOC_LITERAL(7, 80, 16), // "ConnectionFailed"
QT_MOC_LITERAL(8, 97, 13), // "ClientsFailed"
QT_MOC_LITERAL(9, 111, 12), // "EnableFailed"
QT_MOC_LITERAL(10, 124, 14) // "RestrictFailed"

    },
    "QQmlDebugTest\0cleanup\0\0ConnectResult\0"
    "ConnectSuccess\0ProcessFailed\0SessionFailed\0"
    "ConnectionFailed\0ClientsFailed\0"
    "EnableFailed\0RestrictFailed"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlDebugTest[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       1,   20, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,

 // enums: name, alias, flags, count, data
       3,    3, 0x0,    7,   25,

 // enum data: key, value
       4, uint(QQmlDebugTest::ConnectSuccess),
       5, uint(QQmlDebugTest::ProcessFailed),
       6, uint(QQmlDebugTest::SessionFailed),
       7, uint(QQmlDebugTest::ConnectionFailed),
       8, uint(QQmlDebugTest::ClientsFailed),
       9, uint(QQmlDebugTest::EnableFailed),
      10, uint(QQmlDebugTest::RestrictFailed),

       0        // eod
};

void QQmlDebugTest::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQmlDebugTest *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->cleanup(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQmlDebugTest::staticMetaObject = { {
    QMetaObject::SuperData::link<QQmlDataTest::staticMetaObject>(),
    qt_meta_stringdata_QQmlDebugTest.data,
    qt_meta_data_QQmlDebugTest,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlDebugTest::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlDebugTest::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlDebugTest.stringdata0))
        return static_cast<void*>(this);
    return QQmlDataTest::qt_metacast(_clname);
}

int QQmlDebugTest::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQmlDataTest::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_QQmlDebugTestClient_t {
    QByteArrayData data[3];
    char stringdata0[35];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlDebugTestClient_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlDebugTestClient_t qt_meta_stringdata_QQmlDebugTestClient = {
    {
QT_MOC_LITERAL(0, 0, 19), // "QQmlDebugTestClient"
QT_MOC_LITERAL(1, 20, 13), // "serverMessage"
QT_MOC_LITERAL(2, 34, 0) // ""

    },
    "QQmlDebugTestClient\0serverMessage\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlDebugTestClient[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QByteArray,    2,

       0        // eod
};

void QQmlDebugTestClient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQmlDebugTestClient *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->serverMessage((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQmlDebugTestClient::*)(const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDebugTestClient::serverMessage)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QQmlDebugTestClient::staticMetaObject = { {
    QMetaObject::SuperData::link<QQmlDebugClient::staticMetaObject>(),
    qt_meta_stringdata_QQmlDebugTestClient.data,
    qt_meta_data_QQmlDebugTestClient,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlDebugTestClient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlDebugTestClient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlDebugTestClient.stringdata0))
        return static_cast<void*>(this);
    return QQmlDebugClient::qt_metacast(_clname);
}

int QQmlDebugTestClient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQmlDebugClient::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void QQmlDebugTestClient::serverMessage(const QByteArray & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_QQmlInspectorResultRecipient_t {
    QByteArrayData data[1];
    char stringdata0[29];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlInspectorResultRecipient_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlInspectorResultRecipient_t qt_meta_stringdata_QQmlInspectorResultRecipient = {
    {
QT_MOC_LITERAL(0, 0, 28) // "QQmlInspectorResultRecipient"

    },
    "QQmlInspectorResultRecipient"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlInspectorResultRecipient[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QQmlInspectorResultRecipient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQmlInspectorResultRecipient::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQmlInspectorResultRecipient.data,
    qt_meta_data_QQmlInspectorResultRecipient,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlInspectorResultRecipient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlInspectorResultRecipient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlInspectorResultRecipient.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQmlInspectorResultRecipient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_ClientStateHandler_t {
    QByteArrayData data[3];
    char stringdata0[26];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ClientStateHandler_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ClientStateHandler_t qt_meta_stringdata_ClientStateHandler = {
    {
QT_MOC_LITERAL(0, 0, 18), // "ClientStateHandler"
QT_MOC_LITERAL(1, 19, 5), // "allOk"
QT_MOC_LITERAL(2, 25, 0) // ""

    },
    "ClientStateHandler\0allOk\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ClientStateHandler[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

       0        // eod
};

void ClientStateHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ClientStateHandler *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->allOk(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ClientStateHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ClientStateHandler::allOk)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ClientStateHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ClientStateHandler.data,
    qt_meta_data_ClientStateHandler,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ClientStateHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ClientStateHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ClientStateHandler.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ClientStateHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void ClientStateHandler::allOk()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
