/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <qwavefrontmesh.h>

void qml_register_types_Qt_labs_wavefrontmesh()
{
    qmlRegisterTypesAndRevisions<QWavefrontMesh>("Qt.labs.wavefrontmesh", 1);
    qmlRegisterModule("Qt.labs.wavefrontmesh", 1, 15);
}

static const QQmlModuleRegistration registration("Qt.labs.wavefrontmesh", 1, qml_register_types_Qt_labs_wavefrontmesh);
