/****************************************************************************
** Meta object code from reading C++ file 'statemachineforeign.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../git/src/imports/statemachine/statemachineforeign.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'statemachineforeign.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QHistoryStateForeign_t {
    QByteArrayData data[5];
    char stringdata0[72];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QHistoryStateForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QHistoryStateForeign_t qt_meta_stringdata_QHistoryStateForeign = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QHistoryStateForeign"
QT_MOC_LITERAL(1, 21, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 33, 13), // "QHistoryState"
QT_MOC_LITERAL(3, 47, 11), // "QML.Element"
QT_MOC_LITERAL(4, 59, 12) // "HistoryState"

    },
    "QHistoryStateForeign\0QML.Foreign\0"
    "QHistoryState\0QML.Element\0HistoryState"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QHistoryStateForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QHistoryStateForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QHistoryStateForeign.data,
    qt_meta_data_QHistoryStateForeign,
    nullptr,
    nullptr,
    nullptr
} };

struct qt_meta_stringdata_QStateForeign_t {
    QByteArrayData data[8];
    char stringdata0[122];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QStateForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QStateForeign_t qt_meta_stringdata_QStateForeign = {
    {
QT_MOC_LITERAL(0, 0, 13), // "QStateForeign"
QT_MOC_LITERAL(1, 14, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 26, 6), // "QState"
QT_MOC_LITERAL(3, 33, 11), // "QML.Element"
QT_MOC_LITERAL(4, 45, 13), // "QML.Creatable"
QT_MOC_LITERAL(5, 59, 5), // "false"
QT_MOC_LITERAL(6, 65, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(7, 87, 34) // "Don't use this, use State ins..."

    },
    "QStateForeign\0QML.Foreign\0QState\0"
    "QML.Element\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "Don't use this, use State instead."
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QStateForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    2,
       4,    5,
       6,    7,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QStateForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QStateForeign.data,
    qt_meta_data_QStateForeign,
    nullptr,
    nullptr,
    nullptr
} };

struct qt_meta_stringdata_QAbstractStateForeign_t {
    QByteArrayData data[8];
    char stringdata0[138];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QAbstractStateForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QAbstractStateForeign_t qt_meta_stringdata_QAbstractStateForeign = {
    {
QT_MOC_LITERAL(0, 0, 21), // "QAbstractStateForeign"
QT_MOC_LITERAL(1, 22, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 34, 14), // "QAbstractState"
QT_MOC_LITERAL(3, 49, 11), // "QML.Element"
QT_MOC_LITERAL(4, 61, 13), // "QML.Creatable"
QT_MOC_LITERAL(5, 75, 5), // "false"
QT_MOC_LITERAL(6, 81, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(7, 103, 34) // "Don't use this, use State ins..."

    },
    "QAbstractStateForeign\0QML.Foreign\0"
    "QAbstractState\0QML.Element\0QML.Creatable\0"
    "false\0QML.UncreatableReason\0"
    "Don't use this, use State instead."
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QAbstractStateForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    2,
       4,    5,
       6,    7,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QAbstractStateForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QAbstractStateForeign.data,
    qt_meta_data_QAbstractStateForeign,
    nullptr,
    nullptr,
    nullptr
} };

struct qt_meta_stringdata_QSignalTransitionForeign_t {
    QByteArrayData data[8];
    char stringdata0[155];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QSignalTransitionForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QSignalTransitionForeign_t qt_meta_stringdata_QSignalTransitionForeign = {
    {
QT_MOC_LITERAL(0, 0, 24), // "QSignalTransitionForeign"
QT_MOC_LITERAL(1, 25, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 37, 17), // "QSignalTransition"
QT_MOC_LITERAL(3, 55, 11), // "QML.Element"
QT_MOC_LITERAL(4, 67, 13), // "QML.Creatable"
QT_MOC_LITERAL(5, 81, 5), // "false"
QT_MOC_LITERAL(6, 87, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(7, 109, 45) // "Don't use this, use SignalTra..."

    },
    "QSignalTransitionForeign\0QML.Foreign\0"
    "QSignalTransition\0QML.Element\0"
    "QML.Creatable\0false\0QML.UncreatableReason\0"
    "Don't use this, use SignalTransition instead."
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QSignalTransitionForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    2,
       4,    5,
       6,    7,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QSignalTransitionForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QSignalTransitionForeign.data,
    qt_meta_data_QSignalTransitionForeign,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
