/****************************************************************************
** Meta object code from reading C++ file 'qquickforeignutils_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/util/qquickforeignutils_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickforeignutils_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QValidatorForeign_t {
    QByteArrayData data[5];
    char stringdata0[63];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QValidatorForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QValidatorForeign_t qt_meta_stringdata_QValidatorForeign = {
    {
QT_MOC_LITERAL(0, 0, 17), // "QValidatorForeign"
QT_MOC_LITERAL(1, 18, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 30, 10), // "QValidator"
QT_MOC_LITERAL(3, 41, 11), // "QML.Element"
QT_MOC_LITERAL(4, 53, 9) // "anonymous"

    },
    "QValidatorForeign\0QML.Foreign\0QValidator\0"
    "QML.Element\0anonymous"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QValidatorForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QValidatorForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QValidatorForeign.data,
    qt_meta_data_QValidatorForeign,
    nullptr,
    nullptr,
    nullptr
} };

struct qt_meta_stringdata_QRegExpValidatorForeign_t {
    QByteArrayData data[5];
    char stringdata0[81];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QRegExpValidatorForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QRegExpValidatorForeign_t qt_meta_stringdata_QRegExpValidatorForeign = {
    {
QT_MOC_LITERAL(0, 0, 23), // "QRegExpValidatorForeign"
QT_MOC_LITERAL(1, 24, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 36, 16), // "QRegExpValidator"
QT_MOC_LITERAL(3, 53, 11), // "QML.Element"
QT_MOC_LITERAL(4, 65, 15) // "RegExpValidator"

    },
    "QRegExpValidatorForeign\0QML.Foreign\0"
    "QRegExpValidator\0QML.Element\0"
    "RegExpValidator"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QRegExpValidatorForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QRegExpValidatorForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QRegExpValidatorForeign.data,
    qt_meta_data_QRegExpValidatorForeign,
    nullptr,
    nullptr,
    nullptr
} };

struct qt_meta_stringdata_QRegularExpressionValidatorForeign_t {
    QByteArrayData data[7];
    char stringdata0[141];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QRegularExpressionValidatorForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QRegularExpressionValidatorForeign_t qt_meta_stringdata_QRegularExpressionValidatorForeign = {
    {
QT_MOC_LITERAL(0, 0, 34), // "QRegularExpressionValidatorFo..."
QT_MOC_LITERAL(1, 35, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 47, 27), // "QRegularExpressionValidator"
QT_MOC_LITERAL(3, 75, 11), // "QML.Element"
QT_MOC_LITERAL(4, 87, 26), // "RegularExpressionValidator"
QT_MOC_LITERAL(5, 114, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(6, 138, 2) // "14"

    },
    "QRegularExpressionValidatorForeign\0"
    "QML.Foreign\0QRegularExpressionValidator\0"
    "QML.Element\0RegularExpressionValidator\0"
    "QML.AddedInMinorVersion\0""14"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QRegularExpressionValidatorForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       3,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QRegularExpressionValidatorForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QRegularExpressionValidatorForeign.data,
    qt_meta_data_QRegularExpressionValidatorForeign,
    nullptr,
    nullptr,
    nullptr
} };

struct qt_meta_stringdata_QInputMethodForeign_t {
    QByteArrayData data[9];
    char stringdata0[145];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QInputMethodForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QInputMethodForeign_t qt_meta_stringdata_QInputMethodForeign = {
    {
QT_MOC_LITERAL(0, 0, 19), // "QInputMethodForeign"
QT_MOC_LITERAL(1, 20, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 32, 12), // "QInputMethod"
QT_MOC_LITERAL(3, 45, 11), // "QML.Element"
QT_MOC_LITERAL(4, 57, 11), // "InputMethod"
QT_MOC_LITERAL(5, 69, 13), // "QML.Creatable"
QT_MOC_LITERAL(6, 83, 5), // "false"
QT_MOC_LITERAL(7, 89, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(8, 111, 33) // "InputMethod is an abstract cl..."

    },
    "QInputMethodForeign\0QML.Foreign\0"
    "QInputMethod\0QML.Element\0InputMethod\0"
    "QML.Creatable\0false\0QML.UncreatableReason\0"
    "InputMethod is an abstract class."
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QInputMethodForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QInputMethodForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QInputMethodForeign.data,
    qt_meta_data_QInputMethodForeign,
    nullptr,
    nullptr,
    nullptr
} };

struct qt_meta_stringdata_QKeySequenceForeign_t {
    QByteArrayData data[11];
    char stringdata0[179];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QKeySequenceForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QKeySequenceForeign_t qt_meta_stringdata_QKeySequenceForeign = {
    {
QT_MOC_LITERAL(0, 0, 19), // "QKeySequenceForeign"
QT_MOC_LITERAL(1, 20, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 32, 12), // "QKeySequence"
QT_MOC_LITERAL(3, 45, 11), // "QML.Element"
QT_MOC_LITERAL(4, 57, 11), // "StandardKey"
QT_MOC_LITERAL(5, 69, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(6, 93, 1), // "2"
QT_MOC_LITERAL(7, 95, 13), // "QML.Creatable"
QT_MOC_LITERAL(8, 109, 5), // "false"
QT_MOC_LITERAL(9, 115, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(10, 137, 41) // "Cannot create an instance of ..."

    },
    "QKeySequenceForeign\0QML.Foreign\0"
    "QKeySequence\0QML.Element\0StandardKey\0"
    "QML.AddedInMinorVersion\0""2\0QML.Creatable\0"
    "false\0QML.UncreatableReason\0"
    "Cannot create an instance of StandardKey."
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QKeySequenceForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       5,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,
       9,   10,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QKeySequenceForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QKeySequenceForeign.data,
    qt_meta_data_QKeySequenceForeign,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
