/****************************************************************************
** Meta object code from reading C++ file 'qqmltablemodel_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../git/src/imports/labsmodels/qqmltablemodel_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qqmltablemodel_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQmlTableModel_t {
    QByteArrayData data[30];
    char stringdata0[312];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlTableModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlTableModel_t qt_meta_stringdata_QQmlTableModel = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QQmlTableModel"
QT_MOC_LITERAL(1, 15, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 31, 7), // "columns"
QT_MOC_LITERAL(3, 39, 11), // "QML.Element"
QT_MOC_LITERAL(4, 51, 10), // "TableModel"
QT_MOC_LITERAL(5, 62, 18), // "columnCountChanged"
QT_MOC_LITERAL(6, 81, 0), // ""
QT_MOC_LITERAL(7, 82, 15), // "rowCountChanged"
QT_MOC_LITERAL(8, 98, 11), // "rowsChanged"
QT_MOC_LITERAL(9, 110, 9), // "appendRow"
QT_MOC_LITERAL(10, 120, 3), // "row"
QT_MOC_LITERAL(11, 124, 5), // "clear"
QT_MOC_LITERAL(12, 130, 6), // "getRow"
QT_MOC_LITERAL(13, 137, 8), // "rowIndex"
QT_MOC_LITERAL(14, 146, 9), // "insertRow"
QT_MOC_LITERAL(15, 156, 7), // "moveRow"
QT_MOC_LITERAL(16, 164, 12), // "fromRowIndex"
QT_MOC_LITERAL(17, 177, 10), // "toRowIndex"
QT_MOC_LITERAL(18, 188, 4), // "rows"
QT_MOC_LITERAL(19, 193, 9), // "removeRow"
QT_MOC_LITERAL(20, 203, 6), // "setRow"
QT_MOC_LITERAL(21, 210, 4), // "data"
QT_MOC_LITERAL(22, 215, 11), // "QModelIndex"
QT_MOC_LITERAL(23, 227, 5), // "index"
QT_MOC_LITERAL(24, 233, 4), // "role"
QT_MOC_LITERAL(25, 238, 7), // "setData"
QT_MOC_LITERAL(26, 246, 5), // "value"
QT_MOC_LITERAL(27, 252, 11), // "columnCount"
QT_MOC_LITERAL(28, 264, 8), // "rowCount"
QT_MOC_LITERAL(29, 273, 38) // "QQmlListProperty<QQmlTableMod..."

    },
    "QQmlTableModel\0DefaultProperty\0columns\0"
    "QML.Element\0TableModel\0columnCountChanged\0"
    "\0rowCountChanged\0rowsChanged\0appendRow\0"
    "row\0clear\0getRow\0rowIndex\0insertRow\0"
    "moveRow\0fromRowIndex\0toRowIndex\0rows\0"
    "removeRow\0setRow\0data\0QModelIndex\0"
    "index\0role\0setData\0value\0columnCount\0"
    "rowCount\0QQmlListProperty<QQmlTableModelColumn>"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlTableModel[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
      14,   18, // methods
       4,  140, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    0,   88,    6, 0x06 /* Public */,
       7,    0,   89,    6, 0x06 /* Public */,
       8,    0,   90,    6, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       9,    1,   91,    6, 0x02 /* Public */,
      11,    0,   94,    6, 0x02 /* Public */,
      12,    1,   95,    6, 0x02 /* Public */,
      14,    2,   98,    6, 0x02 /* Public */,
      15,    3,  103,    6, 0x02 /* Public */,
      15,    2,  110,    6, 0x22 /* Public | MethodCloned */,
      19,    2,  115,    6, 0x02 /* Public */,
      19,    1,  120,    6, 0x22 /* Public | MethodCloned */,
      20,    2,  123,    6, 0x02 /* Public */,
      21,    2,  128,    6, 0x02 /* Public */,
      25,    3,  133,    6, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void, QMetaType::QVariant,   10,
    QMetaType::Void,
    QMetaType::QVariant, QMetaType::Int,   13,
    QMetaType::Void, QMetaType::Int, QMetaType::QVariant,   13,   10,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,   16,   17,   18,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   16,   17,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   13,   18,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, QMetaType::Int, QMetaType::QVariant,   13,   10,
    QMetaType::QVariant, 0x80000000 | 22, QMetaType::QString,   23,   24,
    QMetaType::Bool, 0x80000000 | 22, QMetaType::QString, QMetaType::QVariant,   23,   24,   26,

 // properties: name, type, flags
      27, QMetaType::Int, 0x00495801,
      28, QMetaType::Int, 0x00495801,
      18, QMetaType::QVariant, 0x00495903,
       2, 0x80000000 | 29, 0x00095c09,

 // properties: notify_signal_id
       0,
       1,
       2,
       0,

       0        // eod
};

void QQmlTableModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQmlTableModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->columnCountChanged(); break;
        case 1: _t->rowCountChanged(); break;
        case 2: _t->rowsChanged(); break;
        case 3: _t->appendRow((*reinterpret_cast< const QVariant(*)>(_a[1]))); break;
        case 4: _t->clear(); break;
        case 5: { QVariant _r = _t->getRow((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QVariant*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->insertRow((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2]))); break;
        case 7: _t->moveRow((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 8: _t->moveRow((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 9: _t->removeRow((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 10: _t->removeRow((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->setRow((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2]))); break;
        case 12: { QVariant _r = _t->data((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QVariant*>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->setData((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQmlTableModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModel::columnCountChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQmlTableModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModel::rowCountChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQmlTableModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlTableModel::rowsChanged)) {
                *result = 2;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QQmlTableModelColumn> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQmlTableModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->columnCount(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->rowCount(); break;
        case 2: *reinterpret_cast< QVariant*>(_v) = _t->rows(); break;
        case 3: *reinterpret_cast< QQmlListProperty<QQmlTableModelColumn>*>(_v) = _t->columns(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQmlTableModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setRows(*reinterpret_cast< QVariant*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQmlTableModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractTableModel::staticMetaObject>(),
    qt_meta_stringdata_QQmlTableModel.data,
    qt_meta_data_QQmlTableModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlTableModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlTableModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlTableModel.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QAbstractTableModel::qt_metacast(_clname);
}

int QQmlTableModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractTableModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQmlTableModel::columnCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQmlTableModel::rowCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQmlTableModel::rowsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
