/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <finalstate.h>
#include <signaltransition.h>
#include <state.h>
#include <statemachine.h>
#include <statemachineforeign.h>
#include <timeouttransition.h>

void qml_register_types_QtQml_StateMachine()
{
    qmlRegisterTypesAndRevisions<FinalState>("QtQml.StateMachine", 1);
    qmlRegisterTypesAndRevisions<QAbstractStateForeign>("QtQml.StateMachine", 1);
    qmlRegisterTypesAndRevisions<QHistoryStateForeign>("QtQml.StateMachine", 1);
    qmlRegisterTypesAndRevisions<QSignalTransitionForeign>("QtQml.StateMachine", 1);
    qmlRegisterTypesAndRevisions<QStateForeign>("QtQml.StateMachine", 1);
    qmlRegisterTypesAndRevisions<SignalTransition>("QtQml.StateMachine", 1);
    qmlRegisterTypesAndRevisions<State>("QtQml.StateMachine", 1);
    qmlRegisterTypesAndRevisions<StateMachine>("QtQml.StateMachine", 1);
    qmlRegisterTypesAndRevisions<TimeoutTransition>("QtQml.StateMachine", 1);
    qmlRegisterModule("QtQml.StateMachine", 1, 15);
}

static const QQmlModuleRegistration registration("QtQml.StateMachine", 1, qml_register_types_QtQml_StateMachine);
