/****************************************************************************
** Meta object code from reading C++ file 'qqmldebugserviceinterfaces_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/qml/debugger/qqmldebugserviceinterfaces_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qqmldebugserviceinterfaces_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QV4DebugService_t {
    QByteArrayData data[1];
    char stringdata0[16];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QV4DebugService_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QV4DebugService_t qt_meta_stringdata_QV4DebugService = {
    {
QT_MOC_LITERAL(0, 0, 15) // "QV4DebugService"

    },
    "QV4DebugService"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QV4DebugService[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QV4DebugService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QV4DebugService::staticMetaObject = { {
    QMetaObject::SuperData::link<QQmlDebugService::staticMetaObject>(),
    qt_meta_stringdata_QV4DebugService.data,
    qt_meta_data_QV4DebugService,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QV4DebugService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QV4DebugService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QV4DebugService.stringdata0))
        return static_cast<void*>(this);
    return QQmlDebugService::qt_metacast(_clname);
}

int QV4DebugService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQmlDebugService::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQmlProfilerService_t {
    QByteArrayData data[1];
    char stringdata0[20];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlProfilerService_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlProfilerService_t qt_meta_stringdata_QQmlProfilerService = {
    {
QT_MOC_LITERAL(0, 0, 19) // "QQmlProfilerService"

    },
    "QQmlProfilerService"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlProfilerService[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QQmlProfilerService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQmlProfilerService::staticMetaObject = { {
    QMetaObject::SuperData::link<QQmlDebugService::staticMetaObject>(),
    qt_meta_stringdata_QQmlProfilerService.data,
    qt_meta_data_QQmlProfilerService,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlProfilerService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlProfilerService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlProfilerService.stringdata0))
        return static_cast<void*>(this);
    return QQmlDebugService::qt_metacast(_clname);
}

int QQmlProfilerService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQmlDebugService::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQmlEngineDebugService_t {
    QByteArrayData data[1];
    char stringdata0[23];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlEngineDebugService_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlEngineDebugService_t qt_meta_stringdata_QQmlEngineDebugService = {
    {
QT_MOC_LITERAL(0, 0, 22) // "QQmlEngineDebugService"

    },
    "QQmlEngineDebugService"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlEngineDebugService[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QQmlEngineDebugService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQmlEngineDebugService::staticMetaObject = { {
    QMetaObject::SuperData::link<QQmlDebugService::staticMetaObject>(),
    qt_meta_stringdata_QQmlEngineDebugService.data,
    qt_meta_data_QQmlEngineDebugService,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlEngineDebugService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlEngineDebugService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlEngineDebugService.stringdata0))
        return static_cast<void*>(this);
    return QQmlDebugService::qt_metacast(_clname);
}

int QQmlEngineDebugService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQmlDebugService::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQmlDebugTranslationService_t {
    QByteArrayData data[1];
    char stringdata0[28];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlDebugTranslationService_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlDebugTranslationService_t qt_meta_stringdata_QQmlDebugTranslationService = {
    {
QT_MOC_LITERAL(0, 0, 27) // "QQmlDebugTranslationService"

    },
    "QQmlDebugTranslationService"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlDebugTranslationService[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QQmlDebugTranslationService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQmlDebugTranslationService::staticMetaObject = { {
    QMetaObject::SuperData::link<QQmlDebugService::staticMetaObject>(),
    qt_meta_stringdata_QQmlDebugTranslationService.data,
    qt_meta_data_QQmlDebugTranslationService,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlDebugTranslationService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlDebugTranslationService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlDebugTranslationService.stringdata0))
        return static_cast<void*>(this);
    return QQmlDebugService::qt_metacast(_clname);
}

int QQmlDebugTranslationService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQmlDebugService::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQmlInspectorService_t {
    QByteArrayData data[1];
    char stringdata0[21];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlInspectorService_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlInspectorService_t qt_meta_stringdata_QQmlInspectorService = {
    {
QT_MOC_LITERAL(0, 0, 20) // "QQmlInspectorService"

    },
    "QQmlInspectorService"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlInspectorService[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QQmlInspectorService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQmlInspectorService::staticMetaObject = { {
    QMetaObject::SuperData::link<QQmlDebugService::staticMetaObject>(),
    qt_meta_stringdata_QQmlInspectorService.data,
    qt_meta_data_QQmlInspectorService,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlInspectorService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlInspectorService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlInspectorService.stringdata0))
        return static_cast<void*>(this);
    return QQmlDebugService::qt_metacast(_clname);
}

int QQmlInspectorService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQmlDebugService::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QDebugMessageService_t {
    QByteArrayData data[1];
    char stringdata0[21];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QDebugMessageService_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QDebugMessageService_t qt_meta_stringdata_QDebugMessageService = {
    {
QT_MOC_LITERAL(0, 0, 20) // "QDebugMessageService"

    },
    "QDebugMessageService"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QDebugMessageService[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QDebugMessageService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QDebugMessageService::staticMetaObject = { {
    QMetaObject::SuperData::link<QQmlDebugService::staticMetaObject>(),
    qt_meta_stringdata_QDebugMessageService.data,
    qt_meta_data_QDebugMessageService,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QDebugMessageService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDebugMessageService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QDebugMessageService.stringdata0))
        return static_cast<void*>(this);
    return QQmlDebugService::qt_metacast(_clname);
}

int QDebugMessageService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQmlDebugService::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQmlEngineControlService_t {
    QByteArrayData data[1];
    char stringdata0[25];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlEngineControlService_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlEngineControlService_t qt_meta_stringdata_QQmlEngineControlService = {
    {
QT_MOC_LITERAL(0, 0, 24) // "QQmlEngineControlService"

    },
    "QQmlEngineControlService"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlEngineControlService[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QQmlEngineControlService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQmlEngineControlService::staticMetaObject = { {
    QMetaObject::SuperData::link<QQmlDebugService::staticMetaObject>(),
    qt_meta_stringdata_QQmlEngineControlService.data,
    qt_meta_data_QQmlEngineControlService,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlEngineControlService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlEngineControlService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlEngineControlService.stringdata0))
        return static_cast<void*>(this);
    return QQmlDebugService::qt_metacast(_clname);
}

int QQmlEngineControlService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQmlDebugService::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQmlNativeDebugService_t {
    QByteArrayData data[1];
    char stringdata0[23];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlNativeDebugService_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlNativeDebugService_t qt_meta_stringdata_QQmlNativeDebugService = {
    {
QT_MOC_LITERAL(0, 0, 22) // "QQmlNativeDebugService"

    },
    "QQmlNativeDebugService"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlNativeDebugService[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QQmlNativeDebugService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQmlNativeDebugService::staticMetaObject = { {
    QMetaObject::SuperData::link<QQmlDebugService::staticMetaObject>(),
    qt_meta_stringdata_QQmlNativeDebugService.data,
    qt_meta_data_QQmlNativeDebugService,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlNativeDebugService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlNativeDebugService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlNativeDebugService.stringdata0))
        return static_cast<void*>(this);
    return QQmlDebugService::qt_metacast(_clname);
}

int QQmlNativeDebugService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQmlDebugService::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
