/****************************************************************************
** Meta object code from reading C++ file 'qqmldelegatemodel_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/qmlmodels/qqmldelegatemodel_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qqmldelegatemodel_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQmlDelegateModel_t {
    QByteArrayData data[50];
    char stringdata0[695];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlDelegateModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlDelegateModel_t qt_meta_stringdata_QQmlDelegateModel = {
    {
QT_MOC_LITERAL(0, 0, 17), // "QQmlDelegateModel"
QT_MOC_LITERAL(1, 18, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 34, 8), // "delegate"
QT_MOC_LITERAL(3, 43, 11), // "QML.Element"
QT_MOC_LITERAL(4, 55, 13), // "DelegateModel"
QT_MOC_LITERAL(5, 69, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(6, 93, 1), // "1"
QT_MOC_LITERAL(7, 95, 12), // "QML.Attached"
QT_MOC_LITERAL(8, 108, 25), // "QQmlDelegateModelAttached"
QT_MOC_LITERAL(9, 134, 18), // "filterGroupChanged"
QT_MOC_LITERAL(10, 153, 0), // ""
QT_MOC_LITERAL(11, 154, 20), // "defaultGroupsChanged"
QT_MOC_LITERAL(12, 175, 16), // "rootIndexChanged"
QT_MOC_LITERAL(13, 192, 15), // "delegateChanged"
QT_MOC_LITERAL(14, 208, 15), // "_q_itemsChanged"
QT_MOC_LITERAL(15, 224, 5), // "index"
QT_MOC_LITERAL(16, 230, 5), // "count"
QT_MOC_LITERAL(17, 236, 12), // "QVector<int>"
QT_MOC_LITERAL(18, 249, 5), // "roles"
QT_MOC_LITERAL(19, 255, 16), // "_q_itemsInserted"
QT_MOC_LITERAL(20, 272, 15), // "_q_itemsRemoved"
QT_MOC_LITERAL(21, 288, 13), // "_q_itemsMoved"
QT_MOC_LITERAL(22, 302, 4), // "from"
QT_MOC_LITERAL(23, 307, 2), // "to"
QT_MOC_LITERAL(24, 310, 13), // "_q_modelReset"
QT_MOC_LITERAL(25, 324, 15), // "_q_rowsInserted"
QT_MOC_LITERAL(26, 340, 11), // "QModelIndex"
QT_MOC_LITERAL(27, 352, 23), // "_q_rowsAboutToBeRemoved"
QT_MOC_LITERAL(28, 376, 6), // "parent"
QT_MOC_LITERAL(29, 383, 5), // "begin"
QT_MOC_LITERAL(30, 389, 3), // "end"
QT_MOC_LITERAL(31, 393, 14), // "_q_rowsRemoved"
QT_MOC_LITERAL(32, 408, 12), // "_q_rowsMoved"
QT_MOC_LITERAL(33, 421, 14), // "_q_dataChanged"
QT_MOC_LITERAL(34, 436, 16), // "_q_layoutChanged"
QT_MOC_LITERAL(35, 453, 28), // "QList<QPersistentModelIndex>"
QT_MOC_LITERAL(36, 482, 36), // "QAbstractItemModel::LayoutCha..."
QT_MOC_LITERAL(37, 519, 10), // "modelIndex"
QT_MOC_LITERAL(38, 530, 3), // "idx"
QT_MOC_LITERAL(39, 534, 16), // "parentModelIndex"
QT_MOC_LITERAL(40, 551, 5), // "model"
QT_MOC_LITERAL(41, 557, 14), // "QQmlComponent*"
QT_MOC_LITERAL(42, 572, 13), // "filterOnGroup"
QT_MOC_LITERAL(43, 586, 5), // "items"
QT_MOC_LITERAL(44, 592, 23), // "QQmlDelegateModelGroup*"
QT_MOC_LITERAL(45, 616, 14), // "persistedItems"
QT_MOC_LITERAL(46, 631, 6), // "groups"
QT_MOC_LITERAL(47, 638, 40), // "QQmlListProperty<QQmlDelegate..."
QT_MOC_LITERAL(48, 679, 5), // "parts"
QT_MOC_LITERAL(49, 685, 9) // "rootIndex"

    },
    "QQmlDelegateModel\0DefaultProperty\0"
    "delegate\0QML.Element\0DelegateModel\0"
    "QML.AddedInMinorVersion\0""1\0QML.Attached\0"
    "QQmlDelegateModelAttached\0filterGroupChanged\0"
    "\0defaultGroupsChanged\0rootIndexChanged\0"
    "delegateChanged\0_q_itemsChanged\0index\0"
    "count\0QVector<int>\0roles\0_q_itemsInserted\0"
    "_q_itemsRemoved\0_q_itemsMoved\0from\0"
    "to\0_q_modelReset\0_q_rowsInserted\0"
    "QModelIndex\0_q_rowsAboutToBeRemoved\0"
    "parent\0begin\0end\0_q_rowsRemoved\0"
    "_q_rowsMoved\0_q_dataChanged\0"
    "_q_layoutChanged\0QList<QPersistentModelIndex>\0"
    "QAbstractItemModel::LayoutChangeHint\0"
    "modelIndex\0idx\0parentModelIndex\0model\0"
    "QQmlComponent*\0filterOnGroup\0items\0"
    "QQmlDelegateModelGroup*\0persistedItems\0"
    "groups\0QQmlListProperty<QQmlDelegateModelGroup>\0"
    "parts\0rootIndex"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlDelegateModel[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
      17,   22, // methods
       8,  184, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

 // signals: name, argc, parameters, tag, flags
       9,    0,  107,   10, 0x06 /* Public */,
      11,    0,  108,   10, 0x06 /* Public */,
      12,    0,  109,   10, 0x06 /* Public */,
      13,    0,  110,   10, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      14,    3,  111,   10, 0x08 /* Private */,
      19,    2,  118,   10, 0x08 /* Private */,
      20,    2,  123,   10, 0x08 /* Private */,
      21,    3,  128,   10, 0x08 /* Private */,
      24,    0,  135,   10, 0x08 /* Private */,
      25,    3,  136,   10, 0x08 /* Private */,
      27,    3,  143,   10, 0x08 /* Private */,
      31,    3,  150,   10, 0x08 /* Private */,
      32,    5,  157,   10, 0x08 /* Private */,
      33,    3,  168,   10, 0x08 /* Private */,
      34,    2,  175,   10, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      37,    1,  180,   10, 0x02 /* Public */,
      39,    0,  183,   10, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int, 0x80000000 | 17,   15,   16,   18,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   15,   16,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   15,   16,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,   22,   23,   16,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 26, QMetaType::Int, QMetaType::Int,   10,   10,   10,
    QMetaType::Void, 0x80000000 | 26, QMetaType::Int, QMetaType::Int,   28,   29,   30,
    QMetaType::Void, 0x80000000 | 26, QMetaType::Int, QMetaType::Int,   10,   10,   10,
    QMetaType::Void, 0x80000000 | 26, QMetaType::Int, QMetaType::Int, 0x80000000 | 26, QMetaType::Int,   10,   10,   10,   10,   10,
    QMetaType::Void, 0x80000000 | 26, 0x80000000 | 26, 0x80000000 | 17,   10,   10,   10,
    QMetaType::Void, 0x80000000 | 35, 0x80000000 | 36,   10,   10,

 // methods: parameters
    QMetaType::QVariant, QMetaType::Int,   38,
    QMetaType::QVariant,

 // properties: name, type, flags
      40, QMetaType::QVariant, 0x00095103,
       2, 0x80000000 | 41, 0x0049510b,
      42, QMetaType::QString, 0x00495007,
      43, 0x80000000 | 44, 0x00095409,
      45, 0x80000000 | 44, 0x00095409,
      46, 0x80000000 | 47, 0x00095409,
      48, QMetaType::QObjectStar, 0x00095401,
      49, QMetaType::QVariant, 0x00495103,

 // properties: notify_signal_id
       0,
       3,
       0,
       0,
       0,
       0,
       0,
       2,

       0        // eod
};

void QQmlDelegateModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQmlDelegateModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->filterGroupChanged(); break;
        case 1: _t->defaultGroupsChanged(); break;
        case 2: _t->rootIndexChanged(); break;
        case 3: _t->delegateChanged(); break;
        case 4: _t->_q_itemsChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QVector<int>(*)>(_a[3]))); break;
        case 5: _t->_q_itemsInserted((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 6: _t->_q_itemsRemoved((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 7: _t->_q_itemsMoved((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 8: _t->_q_modelReset(); break;
        case 9: _t->_q_rowsInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 10: _t->_q_rowsAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 11: _t->_q_rowsRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 12: _t->_q_rowsMoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const QModelIndex(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5]))); break;
        case 13: _t->_q_dataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2])),(*reinterpret_cast< const QVector<int>(*)>(_a[3]))); break;
        case 14: _t->_q_layoutChanged((*reinterpret_cast< const QList<QPersistentModelIndex>(*)>(_a[1])),(*reinterpret_cast< QAbstractItemModel::LayoutChangeHint(*)>(_a[2]))); break;
        case 15: { QVariant _r = _t->modelIndex((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QVariant*>(_a[0]) = std::move(_r); }  break;
        case 16: { QVariant _r = _t->parentModelIndex();
            if (_a[0]) *reinterpret_cast< QVariant*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<int> >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<int> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQmlDelegateModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDelegateModel::filterGroupChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQmlDelegateModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDelegateModel::defaultGroupsChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQmlDelegateModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDelegateModel::rootIndexChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQmlDelegateModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDelegateModel::delegateChanged)) {
                *result = 3;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlDelegateModelGroup* >(); break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QQmlDelegateModelGroup> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQmlDelegateModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QVariant*>(_v) = _t->model(); break;
        case 1: *reinterpret_cast< QQmlComponent**>(_v) = _t->delegate(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->filterGroup(); break;
        case 3: *reinterpret_cast< QQmlDelegateModelGroup**>(_v) = _t->items(); break;
        case 4: *reinterpret_cast< QQmlDelegateModelGroup**>(_v) = _t->persistedItems(); break;
        case 5: *reinterpret_cast< QQmlListProperty<QQmlDelegateModelGroup>*>(_v) = _t->groups(); break;
        case 6: *reinterpret_cast< QObject**>(_v) = _t->parts(); break;
        case 7: *reinterpret_cast< QVariant*>(_v) = _t->rootIndex(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQmlDelegateModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setModel(*reinterpret_cast< QVariant*>(_v)); break;
        case 1: _t->setDelegate(*reinterpret_cast< QQmlComponent**>(_v)); break;
        case 2: _t->setFilterGroup(*reinterpret_cast< QString*>(_v)); break;
        case 7: _t->setRootIndex(*reinterpret_cast< QVariant*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQmlDelegateModel *_t = static_cast<QQmlDelegateModel *>(_o);
        (void)_t;
        switch (_id) {
        case 2: _t->resetFilterGroup(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQmlDelegateModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QQmlInstanceModel::staticMetaObject>(),
    qt_meta_stringdata_QQmlDelegateModel.data,
    qt_meta_data_QQmlDelegateModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlDelegateModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlDelegateModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlDelegateModel.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QQmlInstanceModel::qt_metacast(_clname);
}

int QQmlDelegateModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQmlInstanceModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 8;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQmlDelegateModel::filterGroupChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQmlDelegateModel::defaultGroupsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQmlDelegateModel::rootIndexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQmlDelegateModel::delegateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
struct qt_meta_stringdata_QQmlDelegateModelGroup_t {
    QByteArrayData data[27];
    char stringdata0[283];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlDelegateModelGroup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlDelegateModelGroup_t qt_meta_stringdata_QQmlDelegateModelGroup = {
    {
QT_MOC_LITERAL(0, 0, 22), // "QQmlDelegateModelGroup"
QT_MOC_LITERAL(1, 23, 11), // "QML.Element"
QT_MOC_LITERAL(2, 35, 18), // "DelegateModelGroup"
QT_MOC_LITERAL(3, 54, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 78, 1), // "1"
QT_MOC_LITERAL(5, 80, 12), // "countChanged"
QT_MOC_LITERAL(6, 93, 0), // ""
QT_MOC_LITERAL(7, 94, 11), // "nameChanged"
QT_MOC_LITERAL(8, 106, 21), // "defaultIncludeChanged"
QT_MOC_LITERAL(9, 128, 7), // "changed"
QT_MOC_LITERAL(10, 136, 8), // "QJSValue"
QT_MOC_LITERAL(11, 145, 7), // "removed"
QT_MOC_LITERAL(12, 153, 8), // "inserted"
QT_MOC_LITERAL(13, 162, 6), // "insert"
QT_MOC_LITERAL(14, 169, 15), // "QQmlV4Function*"
QT_MOC_LITERAL(15, 185, 6), // "create"
QT_MOC_LITERAL(16, 192, 7), // "resolve"
QT_MOC_LITERAL(17, 200, 6), // "remove"
QT_MOC_LITERAL(18, 207, 9), // "addGroups"
QT_MOC_LITERAL(19, 217, 12), // "removeGroups"
QT_MOC_LITERAL(20, 230, 9), // "setGroups"
QT_MOC_LITERAL(21, 240, 4), // "move"
QT_MOC_LITERAL(22, 245, 3), // "get"
QT_MOC_LITERAL(23, 249, 5), // "index"
QT_MOC_LITERAL(24, 255, 5), // "count"
QT_MOC_LITERAL(25, 261, 4), // "name"
QT_MOC_LITERAL(26, 266, 16) // "includeByDefault"

    },
    "QQmlDelegateModelGroup\0QML.Element\0"
    "DelegateModelGroup\0QML.AddedInMinorVersion\0"
    "1\0countChanged\0\0nameChanged\0"
    "defaultIncludeChanged\0changed\0QJSValue\0"
    "removed\0inserted\0insert\0QQmlV4Function*\0"
    "create\0resolve\0remove\0addGroups\0"
    "removeGroups\0setGroups\0move\0get\0index\0"
    "count\0name\0includeByDefault"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlDelegateModelGroup[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
      13,   18, // methods
       3,  118, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    0,   83,    6, 0x06 /* Public */,
       7,    0,   84,    6, 0x06 /* Public */,
       8,    0,   85,    6, 0x06 /* Public */,
       9,    2,   86,    6, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      13,    1,   91,    6, 0x0a /* Public */,
      15,    1,   94,    6, 0x0a /* Public */,
      16,    1,   97,    6, 0x0a /* Public */,
      17,    1,  100,    6, 0x0a /* Public */,
      18,    1,  103,    6, 0x0a /* Public */,
      19,    1,  106,    6, 0x0a /* Public */,
      20,    1,  109,    6, 0x0a /* Public */,
      21,    1,  112,    6, 0x0a /* Public */,

 // methods: name, argc, parameters, tag, flags
      22,    1,  115,    6, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10, 0x80000000 | 10,   11,   12,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 14,    6,
    QMetaType::Void, 0x80000000 | 14,    6,
    QMetaType::Void, 0x80000000 | 14,    6,
    QMetaType::Void, 0x80000000 | 14,    6,
    QMetaType::Void, 0x80000000 | 14,    6,
    QMetaType::Void, 0x80000000 | 14,    6,
    QMetaType::Void, 0x80000000 | 14,    6,
    QMetaType::Void, 0x80000000 | 14,    6,

 // methods: parameters
    0x80000000 | 10, QMetaType::Int,   23,

 // properties: name, type, flags
      24, QMetaType::Int, 0x00495001,
      25, QMetaType::QString, 0x00495103,
      26, QMetaType::Bool, 0x00495003,

 // properties: notify_signal_id
       0,
       1,
       2,

       0        // eod
};

void QQmlDelegateModelGroup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQmlDelegateModelGroup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->countChanged(); break;
        case 1: _t->nameChanged(); break;
        case 2: _t->defaultIncludeChanged(); break;
        case 3: _t->changed((*reinterpret_cast< const QJSValue(*)>(_a[1])),(*reinterpret_cast< const QJSValue(*)>(_a[2]))); break;
        case 4: _t->insert((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 5: _t->create((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 6: _t->resolve((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 7: _t->remove((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 8: _t->addGroups((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 9: _t->removeGroups((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 10: _t->setGroups((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 11: _t->move((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 12: { QJSValue _r = _t->get((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QJSValue*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQmlDelegateModelGroup::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDelegateModelGroup::countChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQmlDelegateModelGroup::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDelegateModelGroup::nameChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQmlDelegateModelGroup::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDelegateModelGroup::defaultIncludeChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQmlDelegateModelGroup::*)(const QJSValue & , const QJSValue & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDelegateModelGroup::changed)) {
                *result = 3;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQmlDelegateModelGroup *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->count(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->name(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->defaultInclude(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQmlDelegateModelGroup *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setName(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setDefaultInclude(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQmlDelegateModelGroup::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQmlDelegateModelGroup.data,
    qt_meta_data_QQmlDelegateModelGroup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlDelegateModelGroup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlDelegateModelGroup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlDelegateModelGroup.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQmlDelegateModelGroup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQmlDelegateModelGroup::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQmlDelegateModelGroup::nameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQmlDelegateModelGroup::defaultIncludeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQmlDelegateModelGroup::changed(const QJSValue & _t1, const QJSValue & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
struct qt_meta_stringdata_QQmlDelegateModelAttached_t {
    QByteArrayData data[8];
    char stringdata0[104];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlDelegateModelAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlDelegateModelAttached_t qt_meta_stringdata_QQmlDelegateModelAttached = {
    {
QT_MOC_LITERAL(0, 0, 25), // "QQmlDelegateModelAttached"
QT_MOC_LITERAL(1, 26, 13), // "groupsChanged"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 17), // "unresolvedChanged"
QT_MOC_LITERAL(4, 59, 5), // "model"
QT_MOC_LITERAL(5, 65, 18), // "QQmlDelegateModel*"
QT_MOC_LITERAL(6, 84, 6), // "groups"
QT_MOC_LITERAL(7, 91, 12) // "isUnresolved"

    },
    "QQmlDelegateModelAttached\0groupsChanged\0"
    "\0unresolvedChanged\0model\0QQmlDelegateModel*\0"
    "groups\0isUnresolved"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlDelegateModelAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       3,   26, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x06 /* Public */,
       3,    0,   25,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       4, 0x80000000 | 5, 0x00095409,
       6, QMetaType::QStringList, 0x00495103,
       7, QMetaType::Bool, 0x00495001,

 // properties: notify_signal_id
       0,
       0,
       1,

       0        // eod
};

void QQmlDelegateModelAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQmlDelegateModelAttached *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->groupsChanged(); break;
        case 1: _t->unresolvedChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQmlDelegateModelAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDelegateModelAttached::groupsChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQmlDelegateModelAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDelegateModelAttached::unresolvedChanged)) {
                *result = 1;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlDelegateModel* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQmlDelegateModelAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQmlDelegateModel**>(_v) = _t->model(); break;
        case 1: *reinterpret_cast< QStringList*>(_v) = _t->groups(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->isUnresolved(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQmlDelegateModelAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setGroups(*reinterpret_cast< QStringList*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQmlDelegateModelAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQmlDelegateModelAttached.data,
    qt_meta_data_QQmlDelegateModelAttached,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlDelegateModelAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlDelegateModelAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlDelegateModelAttached.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQmlDelegateModelAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQmlDelegateModelAttached::groupsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQmlDelegateModelAttached::unresolvedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
