/********************************************************************************
** Form generated from reading UI file 'import.ui'
**
** Created by: Qt User Interface Compiler version 5.15.4
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMPORT_H
#define UI_IMPORT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ImportDialog
{
public:
    QDialogButtonBox *buttonBox;
    QWidget *formLayoutWidget;
    QFormLayout *formLayout;
    QLabel *label;
    QLabel *label_3;
    QLabel *label_4;
    QLineEdit *inInfluenceEdit;
    QLineEdit *inSlopeEdit;
    QLineEdit *outInfluenceEdit;
    QLineEdit *outSlopeEdit;
    QLabel *label_2;

    void setupUi(QDialog *ImportDialog)
    {
        if (ImportDialog->objectName().isEmpty())
            ImportDialog->setObjectName(QString::fromUtf8("ImportDialog"));
        ImportDialog->resize(400, 164);
        buttonBox = new QDialogButtonBox(ImportDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setGeometry(QRect(40, 130, 341, 32));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
        formLayoutWidget = new QWidget(ImportDialog);
        formLayoutWidget->setObjectName(QString::fromUtf8("formLayoutWidget"));
        formLayoutWidget->setGeometry(QRect(20, 10, 361, 101));
        formLayout = new QFormLayout(formLayoutWidget);
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        formLayout->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(formLayoutWidget);
        label->setObjectName(QString::fromUtf8("label"));

        formLayout->setWidget(0, QFormLayout::LabelRole, label);

        label_3 = new QLabel(formLayoutWidget);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        formLayout->setWidget(2, QFormLayout::LabelRole, label_3);

        label_4 = new QLabel(formLayoutWidget);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        formLayout->setWidget(3, QFormLayout::LabelRole, label_4);

        inInfluenceEdit = new QLineEdit(formLayoutWidget);
        inInfluenceEdit->setObjectName(QString::fromUtf8("inInfluenceEdit"));

        formLayout->setWidget(0, QFormLayout::FieldRole, inInfluenceEdit);

        inSlopeEdit = new QLineEdit(formLayoutWidget);
        inSlopeEdit->setObjectName(QString::fromUtf8("inSlopeEdit"));

        formLayout->setWidget(1, QFormLayout::FieldRole, inSlopeEdit);

        outInfluenceEdit = new QLineEdit(formLayoutWidget);
        outInfluenceEdit->setObjectName(QString::fromUtf8("outInfluenceEdit"));

        formLayout->setWidget(2, QFormLayout::FieldRole, outInfluenceEdit);

        outSlopeEdit = new QLineEdit(formLayoutWidget);
        outSlopeEdit->setObjectName(QString::fromUtf8("outSlopeEdit"));

        formLayout->setWidget(3, QFormLayout::FieldRole, outSlopeEdit);

        label_2 = new QLabel(formLayoutWidget);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        formLayout->setWidget(1, QFormLayout::LabelRole, label_2);


        retranslateUi(ImportDialog);
        QObject::connect(buttonBox, SIGNAL(accepted()), ImportDialog, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), ImportDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(ImportDialog);
    } // setupUi

    void retranslateUi(QDialog *ImportDialog)
    {
        ImportDialog->setWindowTitle(QCoreApplication::translate("ImportDialog", "Import After Effects Curve", nullptr));
        label->setText(QCoreApplication::translate("ImportDialog", "Input Influence:", nullptr));
        label_3->setText(QCoreApplication::translate("ImportDialog", "Output Influence:", nullptr));
        label_4->setText(QCoreApplication::translate("ImportDialog", "Output Slope:", nullptr));
        inInfluenceEdit->setText(QCoreApplication::translate("ImportDialog", "33", nullptr));
        inSlopeEdit->setText(QCoreApplication::translate("ImportDialog", "0", nullptr));
        outInfluenceEdit->setText(QCoreApplication::translate("ImportDialog", "33", nullptr));
        outSlopeEdit->setText(QCoreApplication::translate("ImportDialog", "0", nullptr));
        label_2->setText(QCoreApplication::translate("ImportDialog", "Input Slope:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ImportDialog: public Ui_ImportDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_IMPORT_H
