/****************************************************************************
** Meta object code from reading C++ file 'qquickitem_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/quick/items/qquickitem_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickitem_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickItemLayer_t {
    QByteArrayData data[36];
    char stringdata0[479];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickItemLayer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickItemLayer_t qt_meta_stringdata_QQuickItemLayer = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QQuickItemLayer"
QT_MOC_LITERAL(1, 16, 11), // "QML.Element"
QT_MOC_LITERAL(2, 28, 9), // "anonymous"
QT_MOC_LITERAL(3, 38, 14), // "enabledChanged"
QT_MOC_LITERAL(4, 53, 0), // ""
QT_MOC_LITERAL(5, 54, 7), // "enabled"
QT_MOC_LITERAL(6, 62, 11), // "sizeChanged"
QT_MOC_LITERAL(7, 74, 4), // "size"
QT_MOC_LITERAL(8, 79, 13), // "mipmapChanged"
QT_MOC_LITERAL(9, 93, 6), // "mipmap"
QT_MOC_LITERAL(10, 100, 15), // "wrapModeChanged"
QT_MOC_LITERAL(11, 116, 34), // "QQuickShaderEffectSource::Wra..."
QT_MOC_LITERAL(12, 151, 4), // "mode"
QT_MOC_LITERAL(13, 156, 11), // "nameChanged"
QT_MOC_LITERAL(14, 168, 4), // "name"
QT_MOC_LITERAL(15, 173, 13), // "effectChanged"
QT_MOC_LITERAL(16, 187, 14), // "QQmlComponent*"
QT_MOC_LITERAL(17, 202, 9), // "component"
QT_MOC_LITERAL(18, 212, 13), // "smoothChanged"
QT_MOC_LITERAL(19, 226, 6), // "smooth"
QT_MOC_LITERAL(20, 233, 13), // "formatChanged"
QT_MOC_LITERAL(21, 247, 32), // "QQuickShaderEffectSource::Format"
QT_MOC_LITERAL(22, 280, 6), // "format"
QT_MOC_LITERAL(23, 287, 17), // "sourceRectChanged"
QT_MOC_LITERAL(24, 305, 10), // "sourceRect"
QT_MOC_LITERAL(25, 316, 23), // "textureMirroringChanged"
QT_MOC_LITERAL(26, 340, 42), // "QQuickShaderEffectSource::Tex..."
QT_MOC_LITERAL(27, 383, 9), // "mirroring"
QT_MOC_LITERAL(28, 393, 14), // "samplesChanged"
QT_MOC_LITERAL(29, 408, 5), // "count"
QT_MOC_LITERAL(30, 414, 11), // "textureSize"
QT_MOC_LITERAL(31, 426, 8), // "wrapMode"
QT_MOC_LITERAL(32, 435, 11), // "samplerName"
QT_MOC_LITERAL(33, 447, 6), // "effect"
QT_MOC_LITERAL(34, 454, 16), // "textureMirroring"
QT_MOC_LITERAL(35, 471, 7) // "samples"

    },
    "QQuickItemLayer\0QML.Element\0anonymous\0"
    "enabledChanged\0\0enabled\0sizeChanged\0"
    "size\0mipmapChanged\0mipmap\0wrapModeChanged\0"
    "QQuickShaderEffectSource::WrapMode\0"
    "mode\0nameChanged\0name\0effectChanged\0"
    "QQmlComponent*\0component\0smoothChanged\0"
    "smooth\0formatChanged\0"
    "QQuickShaderEffectSource::Format\0"
    "format\0sourceRectChanged\0sourceRect\0"
    "textureMirroringChanged\0"
    "QQuickShaderEffectSource::TextureMirroring\0"
    "mirroring\0samplesChanged\0count\0"
    "textureSize\0wrapMode\0samplerName\0"
    "effect\0textureMirroring\0samples"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickItemLayer[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      11,   16, // methods
      11,  104, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      11,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,   71,    4, 0x06 /* Public */,
       6,    1,   74,    4, 0x06 /* Public */,
       8,    1,   77,    4, 0x06 /* Public */,
      10,    1,   80,    4, 0x06 /* Public */,
      13,    1,   83,    4, 0x06 /* Public */,
      15,    1,   86,    4, 0x06 /* Public */,
      18,    1,   89,    4, 0x06 /* Public */,
      20,    1,   92,    4, 0x06 /* Public */,
      23,    1,   95,    4, 0x06 /* Public */,
      25,    1,   98,    4, 0x06 /* Public */,
      28,    1,  101,    4, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::QSize,    7,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, QMetaType::QByteArray,   14,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void, QMetaType::Bool,   19,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void, QMetaType::QRectF,   24,
    QMetaType::Void, 0x80000000 | 26,   27,
    QMetaType::Void, QMetaType::Int,   29,

 // properties: name, type, flags
       5, QMetaType::Bool, 0x00495103,
      30, QMetaType::QSize, 0x00495003,
      24, QMetaType::QRectF, 0x00495103,
       9, QMetaType::Bool, 0x00495103,
      19, QMetaType::Bool, 0x00495103,
      31, 0x80000000 | 11, 0x0049510b,
      22, 0x80000000 | 21, 0x0049510b,
      32, QMetaType::QByteArray, 0x00495003,
      33, 0x80000000 | 16, 0x0049510b,
      34, 0x80000000 | 26, 0x0049510b,
      35, QMetaType::Int, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       8,
       2,
       6,
       3,
       7,
       4,
       5,
       9,
      10,

       0        // eod
};

void QQuickItemLayer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickItemLayer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->enabledChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->sizeChanged((*reinterpret_cast< const QSize(*)>(_a[1]))); break;
        case 2: _t->mipmapChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->wrapModeChanged((*reinterpret_cast< QQuickShaderEffectSource::WrapMode(*)>(_a[1]))); break;
        case 4: _t->nameChanged((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 5: _t->effectChanged((*reinterpret_cast< QQmlComponent*(*)>(_a[1]))); break;
        case 6: _t->smoothChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->formatChanged((*reinterpret_cast< QQuickShaderEffectSource::Format(*)>(_a[1]))); break;
        case 8: _t->sourceRectChanged((*reinterpret_cast< const QRectF(*)>(_a[1]))); break;
        case 9: _t->textureMirroringChanged((*reinterpret_cast< QQuickShaderEffectSource::TextureMirroring(*)>(_a[1]))); break;
        case 10: _t->samplesChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickItemLayer::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemLayer::enabledChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickItemLayer::*)(const QSize & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemLayer::sizeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickItemLayer::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemLayer::mipmapChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickItemLayer::*)(QQuickShaderEffectSource::WrapMode );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemLayer::wrapModeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickItemLayer::*)(const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemLayer::nameChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickItemLayer::*)(QQmlComponent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemLayer::effectChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickItemLayer::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemLayer::smoothChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickItemLayer::*)(QQuickShaderEffectSource::Format );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemLayer::formatChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickItemLayer::*)(const QRectF & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemLayer::sourceRectChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickItemLayer::*)(QQuickShaderEffectSource::TextureMirroring );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemLayer::textureMirroringChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickItemLayer::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemLayer::samplesChanged)) {
                *result = 10;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 8:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickItemLayer *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->enabled(); break;
        case 1: *reinterpret_cast< QSize*>(_v) = _t->size(); break;
        case 2: *reinterpret_cast< QRectF*>(_v) = _t->sourceRect(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->mipmap(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->smooth(); break;
        case 5: *reinterpret_cast< QQuickShaderEffectSource::WrapMode*>(_v) = _t->wrapMode(); break;
        case 6: *reinterpret_cast< QQuickShaderEffectSource::Format*>(_v) = _t->format(); break;
        case 7: *reinterpret_cast< QByteArray*>(_v) = _t->name(); break;
        case 8: *reinterpret_cast< QQmlComponent**>(_v) = _t->effect(); break;
        case 9: *reinterpret_cast< QQuickShaderEffectSource::TextureMirroring*>(_v) = _t->textureMirroring(); break;
        case 10: *reinterpret_cast< int*>(_v) = _t->samples(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickItemLayer *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setSize(*reinterpret_cast< QSize*>(_v)); break;
        case 2: _t->setSourceRect(*reinterpret_cast< QRectF*>(_v)); break;
        case 3: _t->setMipmap(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setSmooth(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setWrapMode(*reinterpret_cast< QQuickShaderEffectSource::WrapMode*>(_v)); break;
        case 6: _t->setFormat(*reinterpret_cast< QQuickShaderEffectSource::Format*>(_v)); break;
        case 7: _t->setName(*reinterpret_cast< QByteArray*>(_v)); break;
        case 8: _t->setEffect(*reinterpret_cast< QQmlComponent**>(_v)); break;
        case 9: _t->setTextureMirroring(*reinterpret_cast< QQuickShaderEffectSource::TextureMirroring*>(_v)); break;
        case 10: _t->setSamples(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

static const QMetaObject::SuperData qt_meta_extradata_QQuickItemLayer[] = {
    QMetaObject::SuperData::link<QQuickShaderEffectSource::staticMetaObject>(),
    nullptr
};

QT_INIT_METAOBJECT const QMetaObject QQuickItemLayer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickItemLayer.data,
    qt_meta_data_QQuickItemLayer,
    qt_static_metacall,
    qt_meta_extradata_QQuickItemLayer,
    nullptr
} };


const QMetaObject *QQuickItemLayer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickItemLayer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickItemLayer.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQuickItemChangeListener"))
        return static_cast< QQuickItemChangeListener*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickItemLayer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 11;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickItemLayer::enabledChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickItemLayer::sizeChanged(const QSize & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickItemLayer::mipmapChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickItemLayer::wrapModeChanged(QQuickShaderEffectSource::WrapMode _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickItemLayer::nameChanged(const QByteArray & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QQuickItemLayer::effectChanged(QQmlComponent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QQuickItemLayer::smoothChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QQuickItemLayer::formatChanged(QQuickShaderEffectSource::Format _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QQuickItemLayer::sourceRectChanged(const QRectF & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QQuickItemLayer::textureMirroringChanged(QQuickShaderEffectSource::TextureMirroring _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QQuickItemLayer::samplesChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}
struct qt_meta_stringdata_QQuickKeyNavigationAttached_t {
    QByteArrayData data[27];
    char stringdata0[338];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickKeyNavigationAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickKeyNavigationAttached_t qt_meta_stringdata_QQuickKeyNavigationAttached = {
    {
QT_MOC_LITERAL(0, 0, 27), // "QQuickKeyNavigationAttached"
QT_MOC_LITERAL(1, 28, 11), // "QML.Element"
QT_MOC_LITERAL(2, 40, 13), // "KeyNavigation"
QT_MOC_LITERAL(3, 54, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 68, 5), // "false"
QT_MOC_LITERAL(5, 74, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 96, 56), // "KeyNavigation is only availab..."
QT_MOC_LITERAL(7, 153, 12), // "QML.Attached"
QT_MOC_LITERAL(8, 166, 11), // "leftChanged"
QT_MOC_LITERAL(9, 178, 0), // ""
QT_MOC_LITERAL(10, 179, 12), // "rightChanged"
QT_MOC_LITERAL(11, 192, 9), // "upChanged"
QT_MOC_LITERAL(12, 202, 11), // "downChanged"
QT_MOC_LITERAL(13, 214, 10), // "tabChanged"
QT_MOC_LITERAL(14, 225, 14), // "backtabChanged"
QT_MOC_LITERAL(15, 240, 15), // "priorityChanged"
QT_MOC_LITERAL(16, 256, 4), // "left"
QT_MOC_LITERAL(17, 261, 11), // "QQuickItem*"
QT_MOC_LITERAL(18, 273, 5), // "right"
QT_MOC_LITERAL(19, 279, 2), // "up"
QT_MOC_LITERAL(20, 282, 4), // "down"
QT_MOC_LITERAL(21, 287, 3), // "tab"
QT_MOC_LITERAL(22, 291, 7), // "backtab"
QT_MOC_LITERAL(23, 299, 8), // "priority"
QT_MOC_LITERAL(24, 308, 8), // "Priority"
QT_MOC_LITERAL(25, 317, 10), // "BeforeItem"
QT_MOC_LITERAL(26, 328, 9) // "AfterItem"

    },
    "QQuickKeyNavigationAttached\0QML.Element\0"
    "KeyNavigation\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "KeyNavigation is only available via attached properties.\0"
    "QML.Attached\0leftChanged\0\0rightChanged\0"
    "upChanged\0downChanged\0tabChanged\0"
    "backtabChanged\0priorityChanged\0left\0"
    "QQuickItem*\0right\0up\0down\0tab\0backtab\0"
    "priority\0Priority\0BeforeItem\0AfterItem"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickKeyNavigationAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       7,   22, // methods
       7,   64, // properties
       1,   92, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    0,

 // signals: name, argc, parameters, tag, flags
       8,    0,   57,    9, 0x06 /* Public */,
      10,    0,   58,    9, 0x06 /* Public */,
      11,    0,   59,    9, 0x06 /* Public */,
      12,    0,   60,    9, 0x06 /* Public */,
      13,    0,   61,    9, 0x06 /* Public */,
      14,    0,   62,    9, 0x06 /* Public */,
      15,    0,   63,    9, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      16, 0x80000000 | 17, 0x0049510b,
      18, 0x80000000 | 17, 0x0049510b,
      19, 0x80000000 | 17, 0x0049510b,
      20, 0x80000000 | 17, 0x0049510b,
      21, 0x80000000 | 17, 0x0049510b,
      22, 0x80000000 | 17, 0x0049510b,
      23, 0x80000000 | 24, 0x0049510b,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,

 // enums: name, alias, flags, count, data
      24,   24, 0x0,    2,   97,

 // enum data: key, value
      25, uint(QQuickKeyNavigationAttached::BeforeItem),
      26, uint(QQuickKeyNavigationAttached::AfterItem),

       0        // eod
};

void QQuickKeyNavigationAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickKeyNavigationAttached *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->leftChanged(); break;
        case 1: _t->rightChanged(); break;
        case 2: _t->upChanged(); break;
        case 3: _t->downChanged(); break;
        case 4: _t->tabChanged(); break;
        case 5: _t->backtabChanged(); break;
        case 6: _t->priorityChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickKeyNavigationAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeyNavigationAttached::leftChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickKeyNavigationAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeyNavigationAttached::rightChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickKeyNavigationAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeyNavigationAttached::upChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickKeyNavigationAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeyNavigationAttached::downChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickKeyNavigationAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeyNavigationAttached::tabChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickKeyNavigationAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeyNavigationAttached::backtabChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickKeyNavigationAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeyNavigationAttached::priorityChanged)) {
                *result = 6;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
        case 4:
        case 3:
        case 2:
        case 1:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickKeyNavigationAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQuickItem**>(_v) = _t->left(); break;
        case 1: *reinterpret_cast< QQuickItem**>(_v) = _t->right(); break;
        case 2: *reinterpret_cast< QQuickItem**>(_v) = _t->up(); break;
        case 3: *reinterpret_cast< QQuickItem**>(_v) = _t->down(); break;
        case 4: *reinterpret_cast< QQuickItem**>(_v) = _t->tab(); break;
        case 5: *reinterpret_cast< QQuickItem**>(_v) = _t->backtab(); break;
        case 6: *reinterpret_cast< Priority*>(_v) = _t->priority(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickKeyNavigationAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLeft(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 1: _t->setRight(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 2: _t->setUp(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 3: _t->setDown(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 4: _t->setTab(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 5: _t->setBacktab(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 6: _t->setPriority(*reinterpret_cast< Priority*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickKeyNavigationAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickKeyNavigationAttached.data,
    qt_meta_data_QQuickKeyNavigationAttached,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickKeyNavigationAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickKeyNavigationAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickKeyNavigationAttached.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQuickItemKeyFilter"))
        return static_cast< QQuickItemKeyFilter*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickKeyNavigationAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 7;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickKeyNavigationAttached::leftChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickKeyNavigationAttached::rightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickKeyNavigationAttached::upChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickKeyNavigationAttached::downChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickKeyNavigationAttached::tabChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickKeyNavigationAttached::backtabChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickKeyNavigationAttached::priorityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
struct qt_meta_stringdata_QQuickLayoutMirroringAttached_t {
    QByteArrayData data[13];
    char stringdata0[235];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickLayoutMirroringAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickLayoutMirroringAttached_t qt_meta_stringdata_QQuickLayoutMirroringAttached = {
    {
QT_MOC_LITERAL(0, 0, 29), // "QQuickLayoutMirroringAttached"
QT_MOC_LITERAL(1, 30, 11), // "QML.Element"
QT_MOC_LITERAL(2, 42, 15), // "LayoutMirroring"
QT_MOC_LITERAL(3, 58, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 72, 5), // "false"
QT_MOC_LITERAL(5, 78, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 100, 58), // "LayoutMirroring is only avail..."
QT_MOC_LITERAL(7, 159, 12), // "QML.Attached"
QT_MOC_LITERAL(8, 172, 14), // "enabledChanged"
QT_MOC_LITERAL(9, 187, 0), // ""
QT_MOC_LITERAL(10, 188, 22), // "childrenInheritChanged"
QT_MOC_LITERAL(11, 211, 7), // "enabled"
QT_MOC_LITERAL(12, 219, 15) // "childrenInherit"

    },
    "QQuickLayoutMirroringAttached\0QML.Element\0"
    "LayoutMirroring\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "LayoutMirroring is only available via attached properties.\0"
    "QML.Attached\0enabledChanged\0\0"
    "childrenInheritChanged\0enabled\0"
    "childrenInherit"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickLayoutMirroringAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       2,   22, // methods
       2,   34, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    0,

 // signals: name, argc, parameters, tag, flags
       8,    0,   32,    9, 0x06 /* Public */,
      10,    0,   33,    9, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      11, QMetaType::Bool, 0x00495107,
      12, QMetaType::Bool, 0x00495103,

 // properties: notify_signal_id
       0,
       1,

       0        // eod
};

void QQuickLayoutMirroringAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickLayoutMirroringAttached *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->enabledChanged(); break;
        case 1: _t->childrenInheritChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickLayoutMirroringAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickLayoutMirroringAttached::enabledChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickLayoutMirroringAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickLayoutMirroringAttached::childrenInheritChanged)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickLayoutMirroringAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->enabled(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->childrenInherit(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickLayoutMirroringAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setChildrenInherit(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickLayoutMirroringAttached *_t = static_cast<QQuickLayoutMirroringAttached *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->resetEnabled(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickLayoutMirroringAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickLayoutMirroringAttached.data,
    qt_meta_data_QQuickLayoutMirroringAttached,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickLayoutMirroringAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickLayoutMirroringAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickLayoutMirroringAttached.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickLayoutMirroringAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickLayoutMirroringAttached::enabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickLayoutMirroringAttached::childrenInheritChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_QQuickEnterKeyAttached_t {
    QByteArrayData data[14];
    char stringdata0[211];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickEnterKeyAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickEnterKeyAttached_t qt_meta_stringdata_QQuickEnterKeyAttached = {
    {
QT_MOC_LITERAL(0, 0, 22), // "QQuickEnterKeyAttached"
QT_MOC_LITERAL(1, 23, 11), // "QML.Element"
QT_MOC_LITERAL(2, 35, 8), // "EnterKey"
QT_MOC_LITERAL(3, 44, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 58, 5), // "false"
QT_MOC_LITERAL(5, 64, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 86, 50), // "EnterKey is only available vi..."
QT_MOC_LITERAL(7, 137, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(8, 161, 1), // "6"
QT_MOC_LITERAL(9, 163, 12), // "QML.Attached"
QT_MOC_LITERAL(10, 176, 11), // "typeChanged"
QT_MOC_LITERAL(11, 188, 0), // ""
QT_MOC_LITERAL(12, 189, 4), // "type"
QT_MOC_LITERAL(13, 194, 16) // "Qt::EnterKeyType"

    },
    "QQuickEnterKeyAttached\0QML.Element\0"
    "EnterKey\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "EnterKey is only available via attached properties\0"
    "QML.AddedInMinorVersion\0""6\0QML.Attached\0"
    "typeChanged\0\0type\0Qt::EnterKeyType"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickEnterKeyAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       5,   14, // classinfo
       1,   24, // methods
       1,   30, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,
       9,    0,

 // signals: name, argc, parameters, tag, flags
      10,    0,   29,   11, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

 // properties: name, type, flags
      12, 0x80000000 | 13, 0x0049510b,

 // properties: notify_signal_id
       0,

       0        // eod
};

void QQuickEnterKeyAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickEnterKeyAttached *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->typeChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickEnterKeyAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickEnterKeyAttached::typeChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickEnterKeyAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< Qt::EnterKeyType*>(_v) = _t->type(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickEnterKeyAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setType(*reinterpret_cast< Qt::EnterKeyType*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickEnterKeyAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickEnterKeyAttached.data,
    qt_meta_data_QQuickEnterKeyAttached,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickEnterKeyAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickEnterKeyAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickEnterKeyAttached.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickEnterKeyAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickEnterKeyAttached::typeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_QQuickKeysAttached_t {
    QByteArrayData data[61];
    char stringdata0[836];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickKeysAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickKeysAttached_t qt_meta_stringdata_QQuickKeysAttached = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QQuickKeysAttached"
QT_MOC_LITERAL(1, 19, 11), // "QML.Element"
QT_MOC_LITERAL(2, 31, 4), // "Keys"
QT_MOC_LITERAL(3, 36, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 50, 5), // "false"
QT_MOC_LITERAL(5, 56, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 78, 46), // "Keys is only available via at..."
QT_MOC_LITERAL(7, 125, 12), // "QML.Attached"
QT_MOC_LITERAL(8, 138, 14), // "enabledChanged"
QT_MOC_LITERAL(9, 153, 0), // ""
QT_MOC_LITERAL(10, 154, 15), // "priorityChanged"
QT_MOC_LITERAL(11, 170, 7), // "pressed"
QT_MOC_LITERAL(12, 178, 15), // "QQuickKeyEvent*"
QT_MOC_LITERAL(13, 194, 5), // "event"
QT_MOC_LITERAL(14, 200, 8), // "released"
QT_MOC_LITERAL(15, 209, 16), // "shortcutOverride"
QT_MOC_LITERAL(16, 226, 13), // "digit0Pressed"
QT_MOC_LITERAL(17, 240, 13), // "digit1Pressed"
QT_MOC_LITERAL(18, 254, 13), // "digit2Pressed"
QT_MOC_LITERAL(19, 268, 13), // "digit3Pressed"
QT_MOC_LITERAL(20, 282, 13), // "digit4Pressed"
QT_MOC_LITERAL(21, 296, 13), // "digit5Pressed"
QT_MOC_LITERAL(22, 310, 13), // "digit6Pressed"
QT_MOC_LITERAL(23, 324, 13), // "digit7Pressed"
QT_MOC_LITERAL(24, 338, 13), // "digit8Pressed"
QT_MOC_LITERAL(25, 352, 13), // "digit9Pressed"
QT_MOC_LITERAL(26, 366, 11), // "leftPressed"
QT_MOC_LITERAL(27, 378, 12), // "rightPressed"
QT_MOC_LITERAL(28, 391, 9), // "upPressed"
QT_MOC_LITERAL(29, 401, 11), // "downPressed"
QT_MOC_LITERAL(30, 413, 10), // "tabPressed"
QT_MOC_LITERAL(31, 424, 14), // "backtabPressed"
QT_MOC_LITERAL(32, 439, 15), // "asteriskPressed"
QT_MOC_LITERAL(33, 455, 17), // "numberSignPressed"
QT_MOC_LITERAL(34, 473, 13), // "escapePressed"
QT_MOC_LITERAL(35, 487, 13), // "returnPressed"
QT_MOC_LITERAL(36, 501, 12), // "enterPressed"
QT_MOC_LITERAL(37, 514, 13), // "deletePressed"
QT_MOC_LITERAL(38, 528, 12), // "spacePressed"
QT_MOC_LITERAL(39, 541, 11), // "backPressed"
QT_MOC_LITERAL(40, 553, 13), // "cancelPressed"
QT_MOC_LITERAL(41, 567, 13), // "selectPressed"
QT_MOC_LITERAL(42, 581, 10), // "yesPressed"
QT_MOC_LITERAL(43, 592, 9), // "noPressed"
QT_MOC_LITERAL(44, 602, 15), // "context1Pressed"
QT_MOC_LITERAL(45, 618, 15), // "context2Pressed"
QT_MOC_LITERAL(46, 634, 15), // "context3Pressed"
QT_MOC_LITERAL(47, 650, 15), // "context4Pressed"
QT_MOC_LITERAL(48, 666, 11), // "callPressed"
QT_MOC_LITERAL(49, 678, 13), // "hangupPressed"
QT_MOC_LITERAL(50, 692, 11), // "flipPressed"
QT_MOC_LITERAL(51, 704, 11), // "menuPressed"
QT_MOC_LITERAL(52, 716, 15), // "volumeUpPressed"
QT_MOC_LITERAL(53, 732, 17), // "volumeDownPressed"
QT_MOC_LITERAL(54, 750, 7), // "enabled"
QT_MOC_LITERAL(55, 758, 9), // "forwardTo"
QT_MOC_LITERAL(56, 768, 28), // "QQmlListProperty<QQuickItem>"
QT_MOC_LITERAL(57, 797, 8), // "priority"
QT_MOC_LITERAL(58, 806, 8), // "Priority"
QT_MOC_LITERAL(59, 815, 10), // "BeforeItem"
QT_MOC_LITERAL(60, 826, 9) // "AfterItem"

    },
    "QQuickKeysAttached\0QML.Element\0Keys\0"
    "QML.Creatable\0false\0QML.UncreatableReason\0"
    "Keys is only available via attached properties\0"
    "QML.Attached\0enabledChanged\0\0"
    "priorityChanged\0pressed\0QQuickKeyEvent*\0"
    "event\0released\0shortcutOverride\0"
    "digit0Pressed\0digit1Pressed\0digit2Pressed\0"
    "digit3Pressed\0digit4Pressed\0digit5Pressed\0"
    "digit6Pressed\0digit7Pressed\0digit8Pressed\0"
    "digit9Pressed\0leftPressed\0rightPressed\0"
    "upPressed\0downPressed\0tabPressed\0"
    "backtabPressed\0asteriskPressed\0"
    "numberSignPressed\0escapePressed\0"
    "returnPressed\0enterPressed\0deletePressed\0"
    "spacePressed\0backPressed\0cancelPressed\0"
    "selectPressed\0yesPressed\0noPressed\0"
    "context1Pressed\0context2Pressed\0"
    "context3Pressed\0context4Pressed\0"
    "callPressed\0hangupPressed\0flipPressed\0"
    "menuPressed\0volumeUpPressed\0"
    "volumeDownPressed\0enabled\0forwardTo\0"
    "QQmlListProperty<QQuickItem>\0priority\0"
    "Priority\0BeforeItem\0AfterItem"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickKeysAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
      43,   22, // methods
       3,  362, // properties
       1,  374, // enums/sets
       0,    0, // constructors
       0,       // flags
      43,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    0,

 // signals: name, argc, parameters, tag, flags
       8,    0,  237,    9, 0x06 /* Public */,
      10,    0,  238,    9, 0x06 /* Public */,
      11,    1,  239,    9, 0x06 /* Public */,
      14,    1,  242,    9, 0x06 /* Public */,
      15,    1,  245,    9, 0x06 /* Public */,
      16,    1,  248,    9, 0x06 /* Public */,
      17,    1,  251,    9, 0x06 /* Public */,
      18,    1,  254,    9, 0x06 /* Public */,
      19,    1,  257,    9, 0x06 /* Public */,
      20,    1,  260,    9, 0x06 /* Public */,
      21,    1,  263,    9, 0x06 /* Public */,
      22,    1,  266,    9, 0x06 /* Public */,
      23,    1,  269,    9, 0x06 /* Public */,
      24,    1,  272,    9, 0x06 /* Public */,
      25,    1,  275,    9, 0x06 /* Public */,
      26,    1,  278,    9, 0x06 /* Public */,
      27,    1,  281,    9, 0x06 /* Public */,
      28,    1,  284,    9, 0x06 /* Public */,
      29,    1,  287,    9, 0x06 /* Public */,
      30,    1,  290,    9, 0x06 /* Public */,
      31,    1,  293,    9, 0x06 /* Public */,
      32,    1,  296,    9, 0x06 /* Public */,
      33,    1,  299,    9, 0x06 /* Public */,
      34,    1,  302,    9, 0x06 /* Public */,
      35,    1,  305,    9, 0x06 /* Public */,
      36,    1,  308,    9, 0x06 /* Public */,
      37,    1,  311,    9, 0x06 /* Public */,
      38,    1,  314,    9, 0x06 /* Public */,
      39,    1,  317,    9, 0x06 /* Public */,
      40,    1,  320,    9, 0x06 /* Public */,
      41,    1,  323,    9, 0x06 /* Public */,
      42,    1,  326,    9, 0x06 /* Public */,
      43,    1,  329,    9, 0x06 /* Public */,
      44,    1,  332,    9, 0x06 /* Public */,
      45,    1,  335,    9, 0x06 /* Public */,
      46,    1,  338,    9, 0x06 /* Public */,
      47,    1,  341,    9, 0x06 /* Public */,
      48,    1,  344,    9, 0x06 /* Public */,
      49,    1,  347,    9, 0x06 /* Public */,
      50,    1,  350,    9, 0x06 /* Public */,
      51,    1,  353,    9, 0x06 /* Public */,
      52,    1,  356,    9, 0x06 /* Public */,
      53,    1,  359,    9, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,

 // properties: name, type, flags
      54, QMetaType::Bool, 0x00495103,
      55, 0x80000000 | 56, 0x00095009,
      57, 0x80000000 | 58, 0x0049510b,

 // properties: notify_signal_id
       0,
       0,
       1,

 // enums: name, alias, flags, count, data
      58,   58, 0x0,    2,  379,

 // enum data: key, value
      59, uint(QQuickKeysAttached::BeforeItem),
      60, uint(QQuickKeysAttached::AfterItem),

       0        // eod
};

void QQuickKeysAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickKeysAttached *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->enabledChanged(); break;
        case 1: _t->priorityChanged(); break;
        case 2: _t->pressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 3: _t->released((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 4: _t->shortcutOverride((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 5: _t->digit0Pressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 6: _t->digit1Pressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 7: _t->digit2Pressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 8: _t->digit3Pressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 9: _t->digit4Pressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 10: _t->digit5Pressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 11: _t->digit6Pressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 12: _t->digit7Pressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 13: _t->digit8Pressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 14: _t->digit9Pressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 15: _t->leftPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 16: _t->rightPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 17: _t->upPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 18: _t->downPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 19: _t->tabPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 20: _t->backtabPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 21: _t->asteriskPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 22: _t->numberSignPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 23: _t->escapePressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 24: _t->returnPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 25: _t->enterPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 26: _t->deletePressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 27: _t->spacePressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 28: _t->backPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 29: _t->cancelPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 30: _t->selectPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 31: _t->yesPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 32: _t->noPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 33: _t->context1Pressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 34: _t->context2Pressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 35: _t->context3Pressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 36: _t->context4Pressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 37: _t->callPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 38: _t->hangupPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 39: _t->flipPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 40: _t->menuPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 41: _t->volumeUpPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        case 42: _t->volumeDownPressed((*reinterpret_cast< QQuickKeyEvent*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 26:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 27:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 28:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 29:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 32:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 33:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 34:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 35:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 36:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 37:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 38:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 39:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 40:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 41:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        case 42:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickKeyEvent* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickKeysAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::enabledChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::priorityChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::pressed)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::released)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::shortcutOverride)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::digit0Pressed)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::digit1Pressed)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::digit2Pressed)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::digit3Pressed)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::digit4Pressed)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::digit5Pressed)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::digit6Pressed)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::digit7Pressed)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::digit8Pressed)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::digit9Pressed)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::leftPressed)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::rightPressed)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::upPressed)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::downPressed)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::tabPressed)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::backtabPressed)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::asteriskPressed)) {
                *result = 21;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::numberSignPressed)) {
                *result = 22;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::escapePressed)) {
                *result = 23;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::returnPressed)) {
                *result = 24;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::enterPressed)) {
                *result = 25;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::deletePressed)) {
                *result = 26;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::spacePressed)) {
                *result = 27;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::backPressed)) {
                *result = 28;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::cancelPressed)) {
                *result = 29;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::selectPressed)) {
                *result = 30;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::yesPressed)) {
                *result = 31;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::noPressed)) {
                *result = 32;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::context1Pressed)) {
                *result = 33;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::context2Pressed)) {
                *result = 34;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::context3Pressed)) {
                *result = 35;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::context4Pressed)) {
                *result = 36;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::callPressed)) {
                *result = 37;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::hangupPressed)) {
                *result = 38;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::flipPressed)) {
                *result = 39;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::menuPressed)) {
                *result = 40;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::volumeUpPressed)) {
                *result = 41;
                return;
            }
        }
        {
            using _t = void (QQuickKeysAttached::*)(QQuickKeyEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickKeysAttached::volumeDownPressed)) {
                *result = 42;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QQuickItem> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickKeysAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->enabled(); break;
        case 1: *reinterpret_cast< QQmlListProperty<QQuickItem>*>(_v) = _t->forwardTo(); break;
        case 2: *reinterpret_cast< Priority*>(_v) = _t->priority(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickKeysAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 2: _t->setPriority(*reinterpret_cast< Priority*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickKeysAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickKeysAttached.data,
    qt_meta_data_QQuickKeysAttached,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickKeysAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickKeysAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickKeysAttached.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQuickItemKeyFilter"))
        return static_cast< QQuickItemKeyFilter*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickKeysAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 43)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 43;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 43)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 43;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickKeysAttached::enabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickKeysAttached::priorityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickKeysAttached::pressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickKeysAttached::released(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickKeysAttached::shortcutOverride(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QQuickKeysAttached::digit0Pressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QQuickKeysAttached::digit1Pressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QQuickKeysAttached::digit2Pressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QQuickKeysAttached::digit3Pressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QQuickKeysAttached::digit4Pressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QQuickKeysAttached::digit5Pressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QQuickKeysAttached::digit6Pressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QQuickKeysAttached::digit7Pressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QQuickKeysAttached::digit8Pressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QQuickKeysAttached::digit9Pressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QQuickKeysAttached::leftPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void QQuickKeysAttached::rightPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void QQuickKeysAttached::upPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void QQuickKeysAttached::downPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void QQuickKeysAttached::tabPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}

// SIGNAL 20
void QQuickKeysAttached::backtabPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 20, _a);
}

// SIGNAL 21
void QQuickKeysAttached::asteriskPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 21, _a);
}

// SIGNAL 22
void QQuickKeysAttached::numberSignPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 22, _a);
}

// SIGNAL 23
void QQuickKeysAttached::escapePressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 23, _a);
}

// SIGNAL 24
void QQuickKeysAttached::returnPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 24, _a);
}

// SIGNAL 25
void QQuickKeysAttached::enterPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 25, _a);
}

// SIGNAL 26
void QQuickKeysAttached::deletePressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 26, _a);
}

// SIGNAL 27
void QQuickKeysAttached::spacePressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 27, _a);
}

// SIGNAL 28
void QQuickKeysAttached::backPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 28, _a);
}

// SIGNAL 29
void QQuickKeysAttached::cancelPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 29, _a);
}

// SIGNAL 30
void QQuickKeysAttached::selectPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 30, _a);
}

// SIGNAL 31
void QQuickKeysAttached::yesPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 31, _a);
}

// SIGNAL 32
void QQuickKeysAttached::noPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 32, _a);
}

// SIGNAL 33
void QQuickKeysAttached::context1Pressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 33, _a);
}

// SIGNAL 34
void QQuickKeysAttached::context2Pressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 34, _a);
}

// SIGNAL 35
void QQuickKeysAttached::context3Pressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 35, _a);
}

// SIGNAL 36
void QQuickKeysAttached::context4Pressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 36, _a);
}

// SIGNAL 37
void QQuickKeysAttached::callPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 37, _a);
}

// SIGNAL 38
void QQuickKeysAttached::hangupPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 38, _a);
}

// SIGNAL 39
void QQuickKeysAttached::flipPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 39, _a);
}

// SIGNAL 40
void QQuickKeysAttached::menuPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 40, _a);
}

// SIGNAL 41
void QQuickKeysAttached::volumeUpPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 41, _a);
}

// SIGNAL 42
void QQuickKeysAttached::volumeDownPressed(QQuickKeyEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 42, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
