/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <conf.h>

void qml_register_types_QmlRuntime_Config()
{
    qmlRegisterTypesAndRevisions<Config>("QmlRuntime.Config", 1);
    qmlRegisterTypesAndRevisions<PartialScene>("QmlRuntime.Config", 1);
    qmlRegisterModule("QmlRuntime.Config", 1, 0);
}

static const QQmlModuleRegistration registration("QmlRuntime.Config", 1, qml_register_types_QmlRuntime_Config);
